unit uMDK_CDSCommon;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 8/06/09 8:02a $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   dddddddddomain.user@domain.ext
*       Description:  this unit contains the Hemodialysis application
*                     Common CDS objects
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/UTILS/uMDK_CDSCommon.pas $
*
*       $ History: uMDK_CDSCommon.pas $
}
interface

uses
  DB
  , DBClient
  , DBCtrls
  , Classes
  , Variants
  , SysUtils
  , Dialogs
  , stdCtrls
  ;

const
  fwDateTime = 12;
  fwFloat = 12;
  fwBoolean = 5;

  cXMLPrefix = '<?xml version="1.0" standalone="yes"?>';
  cXMLDataPacketPrefix = '<DATAPACKET Version="2.0">';
  cXMLPostfix = '</ROWDATA></DATAPACKET>';

  cToBeContinued = '...';

  gMDK_DateFMT = 'mm/dd/yyyy';
  gMDK_TimeFMT = 'hh:nn:ss';
  gMDK_DateTimeFMT = 'mm/dd/yyyy hh:nn:ss';
  gMDK_DebugTimeFMT = 'hh:MM:ss.zzz';

const
  LENGTH_ID = 32;
  LENGTH_NAME = 40;
  LENGTH_DESCRIPTION = 80;
  LENGTH_COMMENTS = 255;

function NewIntegerField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer): TIntegerField;
function NewFloatField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aDisplayFormat: string = '#.##'; aWidth: Integer = fwFloat): TFloatField;
function NewBooleanField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer = fwBoolean): TBooleanField;
function NewStringField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer; aSize: Integer): TStringField;
function NewTimeField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer = fwDateTime;
  aDisplayFMT: string = gMDK_DateTimeFMT): TDateTimeField;
function NewDateTimeField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer = fwDateTime;
  aDisplayFMT: string = gMDK_DateFMT): TDateTimeField;

function NewAggregateField(aCDS: TClientDataSet; aFieldname, anExpression:
  string): TAggregateField;

function CDSGetData(aCDS: TClientDataSet; aValue: string): Boolean;
procedure CDSMerge(aSource, aTarget: TClientDataSet; anID: string);
procedure CDSAppend(aSource, aTarget: TClientDataSet; FieldMap: string = '';
  Defaults: string = '');
function CDSLoadData(aCDS: TClientDataSet; aValue: string; MergeID: string =
  ''): Boolean;
function CDSInitByString(aDS: TClientDataSet; aString: string; Clear: Boolean =
  True): string;

function DataTypeName(aType: TFieldType): string;
function DataTypeComment(aType: TFieldType): string;

procedure RestoreLink(aMaster: TDataSource; aDetail: TClientDataSet; aFieldName:
  string);
procedure BreakLink(aDetail: TClientDataSet);

procedure TestValue(Sender: TObject; Ignore: Boolean = False);

{$IFDEF XMLVIEW}
{$ELSE}
procedure UpdateMemo(Sender: TObject; Signature: string);
procedure UpdateEdit(Sender: TObject; Signature: string);
{$ENDIF}

procedure CopyIntoNullField(cdsFrom, cdsTo: TClientDataSet; aField: string);
procedure MergeCDS(aCDS: TClientDataSet);
procedure CleanUpCDS(aDS: TClientDataSet);
function getFieldByName(aCDS: TClientDataSet; aField: string): TField;
function getFieldDefByName(aCDS: TClientDataSet; aField: string): TFieldDef;
function getActiveField(Sender: TObject): TField;

function CDSsetUpField(aF: TField;
  TheDisplayFormat,
  TheEditMask,
  TheWidth,
  TheUnit,
  TheDefault,
  ThePage,
  TheLabel,
  TheMax, TheMin: string
  ): string;

function setCDSFilter(aCDS: TClientDataSet; aFilter: string): string;

type
  TReportEvent = procedure(aTypeName, aComments: string) of object;

implementation
{$IFDEF XMLVIEW}
uses
  uMDK_Utils;
{$ELSE}

uses
  uMDK_Utils
  , uMDK_Options
  , uMDK_Common
  ;

{$ENDIF}
const
  al: TAlignment = taLeftJustify;

function NewAggregateField(aCDS: TClientDataSet; aFieldname, anExpression:
  string): TAggregateField;
var
  aF: TAggregateField;
begin
  Result := nil;
  try
    aCDS.Close;
    aF := TAggregateField.Create(aCDS);
    aF.DisplayName := aFieldName;
    aF.Name := aCDS.Name + aFieldName;
    aF.Expression := anExpression;
    aF.Visible := True;
    aF.ReadOnly := True;
    aF.Active := True;
    aF.GroupingLevel := 1;
    Result := aF;
  except
  end;
end;

function NewIntegerField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer): TIntegerField;
var
  fF: TIntegerField;
begin
  aCDS.Close;
  fF := TIntegerField.Create(aCDS);
  fF.FieldName := aFieldName;
  fF.Name := aCDS.Name + fF.FieldName;
  fF.Index := aCDS.FieldCount;
  fF.DataSet := aCDS;
  fF.DisplayLabel := aLabel;
  fF.DisplayWidth := aWidth;

  fF.Alignment := al;

  Result := fF;
end;

function NewFloatField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aDisplayFormat: string = '#.##'; aWidth: Integer = fwFloat): TFloatField;
var
  fF: TFloatField;
begin
  aCDS.Close;
  fF := TFloatField.Create(aCDS);
  fF.FieldName := aFieldName;
  fF.Name := aCDS.Name + fF.FieldName;
  fF.Index := aCDS.FieldCount;
  fF.DataSet := aCDS;
  fF.DisplayLabel := aLabel;
  fF.DisplayWidth := aWidth;
{$IFDEF CONTROL_FORMAT}
  fF.DisplayFormat := aDisplayFormat;
{$ENDIF}
  fF.Alignment := al;

  Result := fF;
end;

function NewBooleanField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer = fwBoolean): TBooleanField;
var
  fF: TBooleanField;
begin
  aCDS.Close;
  fF := TBooleanField.Create(aCDS);
  fF.FieldName := aFieldName;
  fF.Name := aCDS.Name + fF.FieldName;
  fF.Index := aCDS.FieldCount;
  fF.DataSet := aCDS;
  fF.DisplayLabel := aLabel;
  fF.DisplayWidth := aWidth;

  fF.Alignment := al;

  Result := fF;
end;

function NewStringField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer; aSize: Integer): TStringField;
var
  fS: TStringField;
begin
  aCDS.Close;
  fS := TStringField.Create(aCDS);
  fS.FieldName := aFieldName;
  fS.Name := aCDS.Name + fS.FieldName;
  fS.Index := aCDS.FieldCount;
  fS.DataSet := aCDS;
  fs.Size := aSize;
  fs.DisplayLabel := aLabel;
  fs.DisplayWidth := aWidth;

  fs.Alignment := al;

  Result := fS;
end;

function NewDateTimeField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer = fwDateTime;
  aDisplayFMT: string = gMDK_DateFMT): TDateTimeField;
var
  fDT: TDateTimeField;
begin
  aCDS.Close;
  fDT := TDateTimeField.Create(aCDS);
  fDT.FieldName := aFieldName;
  fDT.Name := aCDS.Name + fDT.FieldName;
  fDT.Index := aCDS.FieldCount;
  fDT.DataSet := aCDS;
  fDT.DisplayLabel := aLabel;
  fDT.DisplayWidth := aWidth;
  fDT.DisplayFormat := aDisplayFMT;

  fDT.Alignment := al;

  Result := fDT;
end;

function NewTimeField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer = fwDateTime;
  aDisplayFMT: string = gMDK_DateTimeFMT): TDateTimeField;
var
  fDT: TTimeField;
begin
  aCDS.Close;
  fDT := TTimeField.Create(aCDS);
  fDT.FieldName := aFieldName;
  fDT.Name := aCDS.Name + fDT.FieldName;
  fDT.Index := aCDS.FieldCount;
  fDT.DataSet := aCDS;
  fDT.DisplayLabel := aLabel;
  fDT.DisplayWidth := aWidth;
  fDT.DisplayFormat := aDisplayFMT;

  fDT.Alignment := al;

  Result := fDT;
end;

function CDSGetData(aCDS: TClientDataSet; aValue: string): Boolean;
var
  s: string;
  i: Integer;
begin
  aCDS.Close;
  try
    s := aValue;
    i := pos(cXMLPrefix, aValue);
    if i > 0 then
      begin
        s := copy(s, i, Length(s));
        i := pos(cXMLPostfix, s);
        if i > 0 then
          s := copy(s, 1, i + Length(cXMLPostfix) - 1);
        aCDS.XMLData := s;
      end
    else
      s := '';
  except
    s := '';
  end;
  aCDS.Open;
  Result := aCDS.Active;
end;

procedure CDSMerge(aSource, aTarget: TClientDataSet; anID: string);
var
  IndexName: string;

  function FindRecord: Boolean;
  begin
    Result := False;
    if anID = '' then
      Exit;

    aTarget.First;
    Result := aTarget.Locate(anID,
      VarArrayOf([aSource.FieldByName(anID).AsString]),
      [loPartialKey]);

  end;

  procedure UpdateFields;
  var
    j: Integer;
    fn: string;
  begin
    for j := 0 to aTarget.FieldDefs.Count - 1 do
      begin
        try
          fn := aTarget.FieldDefs[j].Name;
          aTarget.FieldByName(fn).Value := aSource.FieldByName(fn).Value;
        except
        end;
      end;
    aTarget.Post;
  end;

  procedure UpdateRecord;
  begin
    if not FindRecord then
      aTarget.Insert
    else
      aTarget.Edit;
    UpdateFields;
  end;

begin
  try
    IndexName := aTarget.IndexFieldNames;
    aTarget.IndexFieldNames := anID;
  except
  end;

  try
    if not aTarget.Active then
      aTarget.Open;
    aSource.First;
    while not aSource.EOF do
      begin
        UpdateRecord;
        aSource.Next;
      end;
  except
    on E: Exception do
      ShowMessage(E.Message);
  end;

  try
    aTarget.IndexFieldNames := Indexname;
  except
  end;
end;

procedure CDSAppend(aSource, aTarget: TClientDataSet; FieldMap: string = '';
  Defaults: string = '');
var
  sl: TStringList;
  dl: TStringList;

  procedure UpdateRecord;
  var
    j: Integer;
    nn,
      fn: string;
  begin
    aTarget.Insert;
    for j := 0 to aTarget.FieldDefs.Count - 1 do
      begin
        try
          fn := aTarget.FieldDefs[j].Name;
          nn := sl.Values[fn];
          if nn = '' then
            nn := fn;
          aTarget.FieldByName(fn).Value := aSource.FieldByName(nn).Value;
        except
          on E: Exception do
            begin
              MessageDLG(E.Message, mtWarning, [mbOK], 0);
              nn := dl.Values[fn];
              if nn <> '' then
                aTarget.FieldByName(fn).AsString := nn;
            end;
        end;
      end;
    aTarget.Post;
  end;

begin
  sl := TStringList.Create;
  sl.Text := FieldMap;
  dl := TStringList.Create;
  dl.Text := Defaults;
  try
    if not aTarget.Active then
      aTarget.Open;
    aSource.First;
    while not aSource.EOF do
      begin
        UpdateRecord;
        aSource.Next;
      end;
  except
    on E: Exception do
      ShowMessage(E.Message);
  end;
  sl.Free;
  dl.Free;
end;

function CDSLoadData(aCDS: TClientDataSet; aValue: string; MergeID: string =
  ''): Boolean;
var
  s: string;
  i: Integer;
  aCopyCDS: TClientDataSet;
begin
  try
    s := aValue;
    i := pos(cXMLPrefix, aValue);
    if (i > 0) and (pos(cXMLDatapacketPrefix, aValue) > 0) then
      begin
        s := copy(s, i, Length(s));
        i := pos(cXMLPostfix, s);
        if i > 0 then
          s := copy(s, 1, i + Length(cXMLPostfix) - 1);
      end
    else
      s := '';
  except
    s := '';
  end;

  if trim(s) <> '' then
    begin
      aCopyCDS := TClientDataSet.Create(nil);
      aCopyCDS.XMLData := s;
      try
        if not aCopyCDS.Active then
          aCopyCDS.Open;
      except
        on E: Exception do
          FreeAndNil(aCopyCDS);
      end;

      if Assigned(aCopyCDS) then
        begin
          if MergeID = '' then
            CDSAppend(aCopyCDS, aCDS)
          else
            CDSMerge(aCopyCDS, aCDS, MergeID);
          aCopyCDS.Close;
          aCopyCDS.Free;
        end;
    end;

  try
    aCDS.Open;
    Result := aCDS.Active;
  except
    Result := False;
  end;

end;

function CDSInitByString(aDS: TClientDataSet; aString: string; Clear: Boolean =
  True): string;
var
  sCopyValue,
    sValue: string;
  cdsCopy: TClientDataSet;
  fld: TFieldDef;
  fn: string;
  i: Integer;
begin
  Result := '';
  try
    if not aDS.Active then
      aDS.Open;
    if Clear then
      aDS.EmptyDataSet;
  except
    begin
      Result := 'Failed to Empty Data set <' + aDS.Name + '>';
      exit;
    end;
  end;

  sValue := aString;
  while pos(#13 + #10, sValue) = 1 do
    sValue := copy(sValue, 3, Length(sValue));
  while pos(#10 + #13, sValue) = 1 do
    sValue := copy(sValue, 3, Length(sValue));
  if (aString = '') or (aString = #13 + #10) or (pos('-1^', sValue) = 1) then
    begin
      Result := 'Assigning Blank String to the Dataset <' + aDS.Name + '>';
      exit;
    end;
  cdsCopy := TClientDataSet.Create(nil);
  try
    cdsCopy.XMLData := sValue;
    cdsCopy.Open;

    while not cdsCopy.Eof do
      begin
        aDS.Insert;
        for i := 0 to aDS.FieldCount - 1 do
          begin
            if i >= aDS.FieldDefs.Count then
              break; // zzzzzzandria 050407
            fld := aDS.FieldDefs[i];
            fn := fld.Name;
            try
              try
                sCopyValue := cdsCopy.FieldByName(fn).AsString;
              except
                on E: Exception do
                  begin
                    sCopyValue := '';
                    //                  report(etError,'Assigning value '+cdsCopy.FieldByName(fn).AsString+' to <'+aDS.Name+'.'+fn+'>'+#13+#10+#13+#10+E.Message,
                    //                    '',0,Classname,'Get Field Data');
                  end;
              end;
              aDS.FieldByName(fn).AsString := sCopyValue;
            except
              //              on E: Exception do
              //              report(etError,'Assigning value '+cdsCopy.FieldByName(fn).AsString+' to <'+aDS.Name+'.'+fn+'>'+#13+#10+#13+#10+E.Message,
              //                '',0,Classname,'Assign Value');
            end;
            try
              if (fld.DataType = ftBoolean)
                and (aDS.FieldByName(fn).AsBoolean <> True) then
                aDS.FieldByName(fn).AsBoolean := False;
            except
              aDS.FieldByName(fn).AsBoolean := False;
            end;
          end;
        aDS.Post;
        cdsCopy.Next;
      end;
  except
  end;
  cdsCopy.Free;
end;

function DataTypeComment(aType: TFieldType): string;
begin
  case aType of
    ftUnknown: Result := 'Unknown or undetermined';
    ftString: Result := 'Character or string field';
    ftSmallint: Result := '16-bit integer field';
    ftInteger: Result := '32-bit integer field';
    ftWord: Result := '16-bit unsigned integer field';
    ftBoolean: Result := 'Boolean field';
    ftFloat: Result := 'Floating-point numeric field';
    ftCurrency: Result := 'Money field';
    ftBCD: Result :=
      'Binary-Coded Decimal field that can be converted to Currency type without a loss of precision.';
    ftDate: Result := 'Date field';
    ftTime: Result := 'Time field';
    ftDateTime: Result := 'Date and time field';
    ftBytes: Result := 'Fixed number of bytes (binary storage)';

    ftVarBytes: Result := 'Variable number of bytes (binary storage)';
    ftAutoInc: Result := 'Auto-incrementing 32-bit integer counter field';
    ftBlob: Result := 'Binary Large OBject field';
    ftMemo: Result := 'Text memo field';
    ftGraphic: Result := 'Bitmap field';
    ftFmtMemo: Result := 'Formatted text memo field';
    ftParadoxOle: Result := 'Paradox OLE field';
    ftDBaseOle: Result := 'dBASE OLE field';
    ftTypedBinary: Result := 'Typed binary field';
    ftCursor: Result :=
      'Output cursor from an Oracle stored procedure (TParam only)';
    ftFixedChar: Result := 'Fixed character field';
    ftWideString: Result := 'Wide string field';
    ftLargeint: Result := 'Large integer field';

    ftADT: Result := 'Abstract Data Type field';
    ftArray: Result := 'Array field';
    ftReference: Result := 'REF field';
    ftDataSet: Result := 'DataSet field';
    ftOraBlob: Result := 'BLOB fields in Oracle 8 tables';
    ftOraClob: Result := 'CLOB fields in Oracle 8 tables';
    ftVariant: Result := 'Data of unknown or undetermined type';
    ftInterface: Result := 'References to interfaces (IUnknown)';
    ftIDispatch: Result := 'References to IDispatch interfaces';
    ftGuid: Result := 'globally unique identifier (GUID) values';
    ftTimeStamp: Result := 'Date and time field accessed through dbExpress';
    ftFMTBcd: Result :=
      'Binary-Coded Decimal field that is too large for ftBCD.';
  end;
end;

function DataTypeName(aType: TFieldType): string;
begin
  Result := IntToStr(Integer(aType));
end;

procedure TestValue(Sender: TObject; Ignore: Boolean = False);
var
  af: TField;
  aDBE: TDBEdit;
  afVal: Real;
  aIVal: Integer;

  function GetFMax(aMask: string): Real;
  var
    s: string;
    i: Integer;
  begin
    s := Piece(aMask, '.', 1);
    Result := 0;
    if Length(s) < 1 then
      Exit;
    Result := 1;
    for i := 1 to Length(s) do
      Result := Result * 10.0;
  end;

  function GetIMax(aMask: string): Integer;
  var
    s: string;
    i: Integer;
  begin
    s := Piece(aMask, '.', 1);
    Result := 0;
    if Length(s) < 1 then
      Exit;
    Result := 1;
    for i := 1 to Length(s) do
      Result := Result * 10;
  end;

  procedure setSelection(aDBEField: TDBEdit);
  begin
    aDBE.SelStart := Length(aDBE.Text) - 1;
    aDBE.SelLength := 1;
  end;

begin
  if Ignore then
    Exit;

  aDBE := TDBEdit(Sender);
  af := aDBE.Field;

  if af is TFloatField then
    begin
      try
        afVal := StrToFloat(aDBE.Text);
        if afVal > TFloatField(af).MaxValue then
          begin
            afVal := TFloatField(af).MaxValue;
            ShowMessage(Format('The value <%s> exceeds the maximum input value <%f>',
              [aDBE.Text, afVal]));
            setSelection(aDBE);
          end;
      except
        if (trim(aDBE.Text) <> '-') and (trim(aDBE.Text) <> '') then
          begin
            ShowMessage('Invalid Value <' + aDBE.Text + '>');
            setSelection(aDBE);
          end;
      end;
    end
  else if af is TIntegerField then
    begin
      try
        aIVal := StrToInt(aDBE.Text);
        if aIVal > TIntegerField(af).MaxValue then
          begin
            aIVal := TIntegerField(af).MaxValue;
            ShowMessage(Format('The value <%s> exceeds the maximum input value <%d>',
              [aDBE.Text, aIVal]));
            setSelection(aDBE);
          end;
      except
        if (trim(aDBE.Text) <> '-') and (trim(aDBE.Text) <> '') then
          begin
            ShowMessage('Invalid Value <' + aDBE.Text + '>');
            setSelection(aDBE);
          end;
      end;
    end
  else if af is TStringField then
    begin
    end
  else if af is TDateTimeField then
    begin
    end
  else if af is TDateField then
    begin
    end;
end;

procedure BreakLink(aDetail: TClientDataSet);
begin
  try
    aDetail.Active := False;
    aDetail.MasterFields := '';
    aDetail.MasterSource := nil;
    aDetail.Active := True;
  except
  end;
end;

procedure RestoreLink(aMaster: TDataSource; aDetail: TClientDataSet; aFieldName:
  string);
begin
  try
    aDetail.Active := False;
    aDetail.MasterFields := aFieldName;
    aDetail.MasterSource := aMaster;
    aDetail.Active := True;
  except
  end;
end;

{$IFDEF XMLVIEW}
{$ELSE}

procedure UpdateMemo(Sender: TObject; Signature: string);
var
  i: Integer;
  s: string;
  dbe: TCustomMemo;
  //  iSignatureLength,
  //  iSelStart,
  //  iSelLength,
  iLL: Integer;

  function SignatureLength: Integer;
  begin
    Result := 0;
    s := Preferences.sValue(onSignatureKey);
    if (s = yoUndefined) or (s = '') then
      Exit;
    i := pos(s, dbe.Text);
    if i = 0 then
      Exit;

    dbe.SelStart := i - 1;
    dbe.SelLength := Length(s);
    dbe.SelText := Signature;
    Result := Length(Signature);
    dbe.SelLength := Result;
  end;

  procedure CheckLineLength;
  var
    sLine, sHead, sTail: string;
    j, jj: Integer;
  begin
    j := 0;
    while j < dbe.Lines.Count do
      begin
        sLine := dbe.Lines[j];
        if Length(sLine) > iLL then
          begin
            sHead := copy(sLine, 1, iLL);
            sTail := copy(sLine, iLL + 1, Length(sLine));
            for jj := iLL downto 1 do
              begin
                if copy(sHead, jj, 1) = ' ' then
                  begin
                    sHead := copy(sHead, 1, jj - 1);
                    break;
                  end
                else
                  sTail := copy(sHead, jj, 1) + sTail;
              end;
            if jj = 0 then
              begin
                sTail := copy(sLine, iLL + 1, Length(sLine));
                sHead := copy(sLine, 1, iLL);
              end;

            dbe.Lines[j] := sHead;
            if sTail <> '' then
              begin
                if j = dbe.Lines.Count - 1 then
                  dbe.Lines[j] := dbe.Lines[j] + #13#10 + sTail
                else
                  dbe.Lines[j + 1] := sTail + ' ' + dbe.Lines[j + 1];
              end;
          end;
        inc(j);
      end;
  end;

begin
  iLL := StrToIntDef(devOptions.sValue(donReportLineLength), 78);
  dbe := TCustomMemo(Sender);

  //  iSelStart := dbe.SelStart;
  //  iSignatureLength :=
  SignatureLength;

  CheckLineLength;

  // uncomment if need to select the added signature
  //  if iSignatureLength > 0 then
  //    dbe.SelStart := iSelStart - 1; // set it equal 2 to include '['
  //  dbe.SelLength := iSignatureLength;
end;

procedure UpdateEdit(Sender: TObject; Signature: string);
var
  i: Integer;
  s: string;
  dbe: TDBEdit;
begin
  s := Preferences.sValue(onSignatureKey);
  if (s = yoUndefined) or (s = '') then
    Exit;
  dbe := TDBEdit(Sender);
  i := pos(s, dbe.Text);
  if i = 0 then
    Exit;
  dbe.SelStart := i - 1;
  dbe.SelLength := Length(s);
  dbe.SelText := Signature;
  dbe.SelStart := i + 3 + Length(Signature);
  dbe.SelLength := 0;
end;
{$ENDIF}

procedure CopyIntoNullField(cdsFrom, cdsTo: TClientDataSet; aField: string);
begin
//  if cdsFrom.Eof then
//    Exit;
  try
    if cdsTo.FieldByName(aField).AsString = '' then
      if cdsFrom.FieldByName(aField).AsString <> '' then
        begin
          cdsTo.Edit;
          cdsTo.FieldByName(aField).AsString :=
            cdsFrom.FieldByName(aField).AsString;
          cdsTo.Post;
        end;
  except
  end;
end;

procedure MergeCDS(aCDS: TClientDataSet);
begin
  try
    if aCDS.State = dsEdit then
      aCDS.Post;
    aCDS.MergeChangeLog;
  except
  end;
end;

procedure CleanUpCDS(aDS: TClientDataSet);
begin
  if not assigned(aDS) then
    Exit;
  try
    if not aDS.Active then
      try
        aDS.Open;
      except
        Exit;
      end;
    if aDS.Active then
      aDS.EmptyDataSet;
    aDS.Close;
  except
  end
end;

function getFieldByName(aCDS: TClientDataSet; aField: string): TField;
var
  i: Integer;
begin
  Result := nil;
  with aCDS do
    begin
      for i := 0 to Fields.Count - 1 do
        begin
          if Fields[i].Name <> aField then
            continue;
          Result := Fields[i];
          Exit;
        end;
    end;
end;

function getFieldDefByName(aCDS: TClientDataSet; aField: string): TFieldDef;
var
  i: Integer;
begin
  Result := nil;
  with aCDS do
    begin
      for i := 0 to FieldDefs.Count - 1 do
        begin
          if FieldDefs[i].Name <> aField then
            continue;
          Result := FieldDefs[i];
          Exit;
        end;
    end;
end;

function getActiveField(Sender: TObject): TField;
var
  fField: TField;
begin
  fField := nil;

  if Sender is TDBMemo then
    fField := TDBMemo(Sender).Field
  else if Sender is TDBComboBox then
    fField := TDBComboBox(Sender).Field
  else if Sender is TDBRadioGroup then
    fField := TDBRadioGroup(Sender).Field
  else if Sender is TDBEdit then
    fField := TDBEdit(Sender).Field
  else if Sender is TDBCheckBox then
    fField := TDBCheckBox(Sender).Field;

  Result := fField;
end;

function CDSsetUpField(aF: TField;
  TheDisplayFormat,
  TheEditMask,
  TheWidth,
  TheUnit,
  TheDefault,
  ThePage,
  TheLabel,
  TheMax, TheMin: string
  ): string;

const
  FLOAT_IGNORE = -999.0;

  procedure ReportIssue(aMessage: string);
  begin
    Result := aMessage;
    //    Report(rtError,'Update table Field <'+aF.Name+'> "'+#13+#10+'Format error '+#13+#10+aMessage);
  end;

begin
  Result := '';
  case aF.DataType of

    ftString: //' 	Character or string field';
      try
        if TheLabel <> '' then
          TStringField(aF).DisplayLabel := TheLabel;
        if TheWidth <> '' then
          TStringField(aF).DisplayWidth := StrToIntDef(TheWidth, 12);
      except
        on E: Exception do
          ReportIssue(E.Message);
      end;
    (**)
    ftInteger: //' 	32-bit integer field';
      try
        if TheLabel <> '' then
          TIntegerField(aF).DisplayLabel := TheLabel;
        if TheDisplayFormat <> '' then
          TIntegerField(aF).DisplayFormat := TheDisplayFormat;
        if TheEditMask <> '' then
          TIntegerField(aF).EditMask := TheEditMask;

        if (TheMin <> TheMax) then
          begin
            if trim(TheMin) <> '' then
              TIntegerField(aF).MinValue := round(StrToFloat(TheMin));
            if trim(TheMax) <> '' then
              TIntegerField(aF).MaxValue := round(StrToFloat(TheMax));
          end;
      except
        on E: Exception do
          ReportIssue(E.Message);
      end;
    (**)
    ftFloat: //' 	Floating-point numeric field';
      try
        if TheLabel <> '' then
          TFloatField(aF).DisplayLabel := TheLabel;

{$IFDEF CONTROL_FORMAT}
        if TheDisplayFormat <> '' then
          TFloatField(aF).DisplayFormat := TheDisplayFormat;
        if TheEditMask <> '' then
          TFloatField(aF).EditMask := TheEditMask;
{$ENDIF}
        if (TheMin <> TheMax) and (StrToFloat(TheMin) <> FLOAT_IGNORE) then
          begin
            if trim(TheMin) <> '' then
              TFloatField(aF).MinValue := StrToFloat(TheMin);
            if trim(TheMax) <> '' then
              TFloatField(aF).MaxValue := StrToFloat(TheMax);
          end;
      except
        on E: Exception do
          ReportIssue(E.Message);
      end;
    ftDateTime: //' 	Date and time field';
      try
        if TheLabel <> '' then
          TDateTimeField(aF).DisplayLabel := TheLabel;
        if TheDisplayFormat <> '' then
          TDateTimeField(aF).DisplayFormat := TheDisplayFormat;
        if TheEditMask <> '' then
          TDateTimeField(aF).EditMask := TheEditMask;
      except
        on E: Exception do
          ReportIssue(E.Message);
      end;

    ftUnknown: ; //' 	Unknown or undetermined';
    ftSmallint: ; //' 	16-bit integer field';
    ftWord: ; //' 	16-bit unsigned integer field';
    ftBoolean: ; //' 	Boolean field';
    ftCurrency: ; //' 	Money field';
    ftBCD: ;
    //' 	Binary-Coded Decimal field that can be converted to Currency type without a loss of precision.';
    ftDate: ; //' 	Date field';
    ftTime: ; //' 	Time field';
    ftBytes: ; //' 	Fixed number of bytes (binary storage)';
    ftVarBytes: ; //' 	Variable number of bytes (binary storage)';
    ftAutoInc: ; //' 	Auto-incrementing 32-bit integer counter field';
    ftBlob: ; //' 	Binary Large OBject field';
    ftMemo: ; //' 	Text memo field';
    ftGraphic: ; //' 	Bitmap field';
    ftFmtMemo: ; //' 	Formatted text memo field';
    ftParadoxOle: ; //' 	Paradox OLE field';
    ftDBaseOle: ; //' 	dBASE OLE field';
    ftTypedBinary: ; //' 	Typed binary field';
    ftCursor: ;
    //' 	Output cursor from an Oracle stored procedure (TParam only)';
    ftFixedChar: ; //' 	Fixed character field';
    ftWideString: ; //' 	Wide string field';
    ftLargeint: ; //' 	Large integer field';

    ftADT: ; //' 	Abstract Data Type field';
    ftArray: ; //' 	Array field';
    ftReference: ; //' 	REF field';
    ftDataSet: ; //' 	DataSet field';
    ftOraBlob: ; //' 	BLOB fields in Oracle 8 tables';
    ftOraClob: ; //' 	CLOB fields in Oracle 8 tables';
    ftVariant: ; //' 	Data of unknown or undetermined type';
    ftInterface: ; //' 	References to interfaces (IUnknown)';
    ftIDispatch: ; //' 	References to IDispatch interfaces';
    ftGuid: ; //' 	globally unique identifier (GUID) values';
    ftTimeStamp: ; //' 	Date and time field accessed through dbExpress';
    ftFMTBcd: ; //' 	Binary-Coded Decimal field that is too large for ftBCD.';
  end;
end;

function setCDSFilter(aCDS: TClientDataSet; aFilter: string): string;
var
  bActive: Boolean;
begin
  bActive := aCDS.Active;
  try
    if trim(aFilter) <> '' then
      aCDS.Filter := aFilter;
    Result := '';
  except
    on E: Exception do
      Result := 'setCDSFilter:' + #13#10 + E.Message;
  end;
  if aCDS.Active <> bActive then
    aCDS.Active := bActive;
end;

end.
