unit uMDK_Dictionary;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 8/06/09 8:02a $
*       Site Name:    Hines OIFO
*       Date Created: 3/28/06
*       Developers:   dddddddddomain.user@domain.ext
*       Description:  Contains the Hemodialysis application
*                     Dictionary
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Objects/uMDK_Dictionary.pas $
*
*       $ History: uMDK_Dictionary.pas $
}

interface

uses
  Classes
  , Graphics
  , SysUtils
  , DB
  , DBClient
  , DBCtrls
  , Dialogs
  , COntrols
  ;

const
  fnDict_ID = 'ID';
  fnDict_Tag = 'Tag';
  fnDict_iTag = 'iTag';
  fnDict_Table = 'TableName';
  fnDict_Column = 'ColumnName';
  fnDict_Type = 'DataType';
  fnDict_Length = 'DataLength';
  fnDict_Label = 'DataLabel';
  fnDict_Format = 'DisplayFMT';
  fnDict_EditMask = 'EditMask';
  fnDict_Width = 'DisplayWidth';
  fnDict_Units = 'DataUnits';
  fnDict_Required = 'Required';
  fnDict_Overwrite = 'Overwrite';
  //  fnDict_Page = 'Page';
  fnDict_Section = 'Section';
  fnDict_Group = 'Group';
  fnDict_Domain = 'Domain';
  fnDict_MinValue = 'MinValue';
  fnDict_MaxValue = 'MaxValue';
  fnDict_Default = 'Default';
  fnDict_Comments = 'Comments';

function prepareDictionaryCDS(cdsDict: TClientDataSet): Boolean;
function getDictionaryField(aTable, aColumn, aField: string): string;
function getDictionaryLabel(aTable, aColumn: string): string;
function getDictionaryTag(aTable, aColumn: string): string;
function getDictionaryUnits(aTable, aColumn, aDefault: string): string;
function getDictionaryFieldByDisplayLabel(aTable, aLabel, aField: string):
  string;
function getDictionaryLabelByITag(iTag: Integer): string;
function getDictionaryRequired(aTable, aColumn: string): Integer;
function getDictionaryEditMask(aTable, aColumn: string): string;
function getDictionaryEditMaskByDisplayLabel(aTable, aColumn: string): string;
procedure setFieldProperties(aTable, aField, aDisplayFormat, aEditMask, aWidth:
  string);

procedure UpdateDictionary(aDS: TClientDataSet);

function CheckDBField(Sender: TObject): string;
procedure addFieldToDictionary(aTable: string; aF: TField);

procedure VerifyDictionary(aDS: TClientDataSet);

var
  DictionaryID: Integer;

implementation

uses
  uMDK_CDSCommon
  , Variants
  , uMDK_Options
  , uMDK_Engine
  , uMDK_Common;

var
  Dictionary: TClientDataSet;

const
  UnknownField = '\UNKNOWN FIELD';

function getNextDictionaryID: string;
begin
  Result := format('%-8.8d', [(Dictionary.RecordCount + 1) * 10]);
end;

function prepareDictionaryCDS(cdsDict: TClientDataSet): Boolean;

  function DictionaryLoadFromFile(aName: string): Boolean;
  var
    SL: TStringLIst;
  begin
    Result := False;
    SL := TStringList.Create;
    try
      SL.LoadFromFile(aName);
      cdsDict.XMLData := SL.Text;
      cdsDict.Open;
      Result := cdsDict.Active;
    except
    end;
    SL.Free;
  end;

begin
  if not FileExists(cdsDict.FileName) then
    begin
      NewStringField(cdsDict, fnDict_ID, fnDict_ID, 8, 20);
      NewStringField(cdsDict, fnDict_Tag, fnDict_Tag, 8, 20);
      NewIntegerField(cdsDict, fnDict_iTag, fnDict_iTag, 8);
      NewStringField(cdsDict, fnDict_Table, fnDict_Table, 12, 20);
      NewStringField(cdsDict, fnDict_Column, 'Column', 12, 30);
      NewStringField(cdsDict, fnDict_Type, 'Type', 12, 20);
      NewIntegerField(cdsDict, fnDict_Length, 'Length', 12);
      NewStringField(cdsDict, fnDict_Label, fnDict_Label, 12, 20);
      NewStringField(cdsDict, fnDict_Format, 'Format', 12, 20);
      NewStringField(cdsDict, fnDict_EditMask, 'Edit Mask', 12, 20);
      NewIntegerField(cdsDict, fnDict_Width, 'Width', 12);
      NewStringField(cdsDict, fnDict_Units, 'Units', 12, 20);
      NewBooleanField(cdsDict, fnDict_Required, 'Required', 3);
      NewBooleanField(cdsDict, fnDict_Overwrite, 'Overwrite', 3);
      //          NewStringField(cdsDict,fnDict_Page,'Page',10,20);
      NewStringField(cdsDict, fnDict_Section, fnDict_Section, 10, 20);
      NewStringField(cdsDict, fnDict_Group, fnDict_Group, 10, 20);
      NewStringField(cdsDict, fnDict_Domain, fnDict_Domain, 30, 30);
      NewFloatField(cdsDict, fnDict_MinValue, 'MinValue');
      NewFloatField(cdsDict, fnDict_MaxValue, 'MaxValue');
      NewFloatField(cdsDict, fnDict_Default, 'Default');
      NewStringField(cdsDict, fnDict_Comments, 'Comments', 80, 200);

      cdsDict.IndexFieldNames := fnDict_ID;
      cdsDict.CreateDataSet;
    end;

  cdsDict.Open;

  Dictionary := cdsDict;
  Result := cdsDict.Active;
end;

function getDictionaryFieldByDisplayLabel(aTable, aLabel, aField: string):
  string;
begin
  result := '';
  if Dictionary = nil then
    Exit;
  if Dictionary.Locate(fnDict_Table + ';' + fnDict_Label, VarArrayOf([aTable,
    aLabel]), []) then
    Result := Dictionary.FieldByName(aField).AsString;
end;

function getDictionaryField(aTable, aColumn, aField: string): string;
begin
  result := '';
  if Dictionary = nil then
    Exit;
  if Dictionary.Locate(fnDict_Table + ';' + fnDict_Column, VarArrayOf([aTable,
    aColumn]), []) then
    Result := Dictionary.FieldByName(aField).AsString;
end;

function getDictionaryTag(aTable, aColumn: string): string;
begin
  result := '';
  if Dictionary = nil then
    Exit;
  if Dictionary.Locate(fnDict_Table + ';' + fnDict_Column, VarArrayOf([aTable,
    aColumn]), []) then
    Result := Dictionary.FieldByName(fnDict_Tag).AsString;
end;

function getDictionaryLabel(aTable, aColumn: string): string;
begin
  result := '';
  if Dictionary = nil then
    Exit;
  if Dictionary.Locate(fnDict_Table + ';' + fnDict_Column, VarArrayOf([aTable,
    aColumn]), []) then
    Result := Dictionary.FieldByName(fnDict_Label).AsString;
  if Result = '' then
    Result := Dictionary.FieldByName(fnDict_Tag).AsString;
end;

function getDictionaryUnits(aTable, aColumn, aDefault: string): string;
begin
  result := '';
  if Dictionary = nil then
    Exit;
  if Dictionary.Locate(fnDict_Table + ';' + fnDict_Column, VarArrayOf([aTable,
    aColumn]), []) then
    Result := Dictionary.FieldByName(fnDict_Units).AsString;
  if Result = '' then
    Result := aDefault;
end;

function getDictionaryLabelByITag(iTag: Integer): string;
begin
  result := '';
  if Dictionary = nil then
    Exit;
  if Dictionary.Locate(fnDict_iTag, VarArrayOf([iTag]), []) then
    Result := Dictionary.FieldByName(fnDict_Label).AsString;
  if Result = '' then
    Result := Dictionary.FieldByName(fnDict_Tag).AsString;
end;

function getDictionaryRequired(aTable, aColumn: string): Integer;
begin
  result := 0;
  if Dictionary = nil then
    Exit;
  if Dictionary.Locate(fnDict_Table + ';' + fnDict_Column, VarArrayOf([aTable,
    aColumn]), []) then
    begin
      if (Dictionary.FieldByName(fnDict_Required).AsBoolean)
        or (Uppercase(Dictionary.FieldByName(fnDict_Required).AsString) <>
        'FALSE') then
        Result := 1
      else
        Result := -1;
    end;
end;

function getDictionaryEditMask(aTable, aColumn: string): string;
begin

  if Dictionary = nil then
    Exit;
  if Dictionary.Locate(fnDict_Table + ';' + fnDict_Column, VarArrayOf([aTable,
    aColumn]), []) then
    Result := Dictionary.FieldByName(fnDict_EditMask).AsString
  else
    Result := '';
end;

function getDictionaryEditMaskByDisplayLabel(aTable, aColumn: string): string;
begin

  if Dictionary = nil then
    Exit;
  if Dictionary.Locate(fnDict_Table + ';' + fnDict_Label, VarArrayOf([aTable,
    aColumn]), []) then
    Result := Dictionary.FieldByName(fnDict_EditMask).AsString
  else
    Result := '';
end;
(*
function TdmData.getDictionaryDefault(aTable,aColumn:String):String;
begin
  result := '';
  if cdsDict.Locate(fnDict_Table+';'+fnDict_Column,VarArrayOf([aTable,aColumn]),[]) then
    Result := cdsDict.FieldByName(fnDict_Default).AsString;
end;
*)

function FieldReport(aF: TField; aText: string): string;
var
  iResult: Integer;
  aTable, aColumn: string;
begin
  Result := '';
  if aF = nil then
    Exit;
  if not aF.Visible then
    Exit;

  aTable := TClientDataSet(aF.DataSet).Name;
  aColumn := aF.FieldName;

  iResult := getDictionaryRequired(aTable, aColumn);
  if iResult > 0 then
    begin
      if aText = '' then // zzzzzzandria 060608
        Result := getDictionaryLabel(aTable, aColumn);
    end
  else if iResult = 0 then
    Result := UnknownField
  else
    Result := '';
end;

function CheckMemoField(aDBE: TDBMemo): string;
begin
  if aDBE.Enabled then
    begin
      Result := FieldReport(aDBE.Field, aDBE.Text);
      if not aDBE.ReadOnly then
        begin
          if Result = '' then
            aDBE.Color := Preferences.iValue(onNormalFieldColor)
          else if Result = UnknownField then
            aDBE.Color := Preferences.iValue(onColorUnknown)
          else
            aDBE.Color := Preferences.iValue(onProblemFieldColor);
        end
      else
        aDBE.Color := Preferences.iValue(onReadOnlyFieldColor);
    end
  else
    begin
      Result := '';
      aDBE.Color := Preferences.iValue(onDisabledFieldColor);
    end;
end;

function CheckField(aDBE: TDBEdit): string;
begin
  if aDBE.Enabled then
    begin
      Result := FieldReport(aDBE.Field, aDBE.Text);
      if Result = '' then
        aDBE.Color := Preferences.iValue(onNormalFieldColor)
      else if Result = UnknownField then
        aDBE.Color := Preferences.iValue(onColorUnknown)
      else
        aDBE.Color := Preferences.iValue(onProblemFieldColor);
    end
  else
    begin
      Result := '';
      aDBE.Color := Preferences.iValue(onDisabledFieldColor);
    end;
end;

function CheckComboField(aDBE: TDBComboBox): string;
begin
  if aDBE.Enabled then
    begin
      Result := FieldReport(aDBE.Field, aDBE.Text);
      if Result = '' then
        aDBE.Color := Preferences.iValue(onNormalFieldColor)
      else if Result = UnknownField then
        aDBE.Color := Preferences.iValue(onColorUnknown)
      else
        aDBE.Color := Preferences.iValue(onProblemFieldColor);
    end
  else
    begin
      Result := '';
      aDBE.Color := Preferences.iValue(onDisabledFieldColor);
    end;
end;

function CheckDBField(Sender: TObject): string;
begin
  Result := '';
  if Sender = nil then
    Exit;
  if Sender is TDBEdit then
    Result := CheckField(TDBEdit(Sender))
  else if Sender is TDBComboBox then
    Result := CheckComboField(TDBComboBox(Sender))
  else if Sender is TDBMemo then
    Result := CheckMemoField(TDBMemo(Sender))
end;

procedure setFieldProperties(aTable, aField, aDisplayFormat, aEditMask, aWidth:
  string);
begin
  if Dictionary.Locate(fnDict_Table + ';' + fnDict_Column, VarArrayOf([aTable,
    aField]), []) then
    begin
      Dictionary.Edit;
      Dictionary.FieldByName(fnDict_Width).AsString := aWidth;
      Dictionary.FieldByName(fnDict_Format).AsString := aDisplayFormat;
      Dictionary.FieldByName(fnDict_EditMask).AsString := aEditMask;
      Dictionary.Post;
    end;
end;

procedure UpdateDictionary(aDS: TClientDataSet);
var
  i: Integer;
  aF: TField;

begin
  for i := 0 to aDS.Fields.Count - 1 do
    begin
      aF := aDS.Fields[i];
      Dictionary.Insert;
      Dictionary.FieldByName(fnDict_ID).AsString := getNextDictionaryID;
      Dictionary.FieldByName(fnDict_Tag).AsString := aDS.Name + '.' +
        aF.FieldName;
      Dictionary.FieldByName(fnDict_iTag).AsInteger := I;
      Dictionary.FieldByName(fnDict_Table).AsString := aDS.Name;
      Dictionary.FieldByName(fnDict_Column).AsString := aF.FieldName;
      Dictionary.FieldByName(fnDict_Type).AsString := DataTypeName(aF.DataType);
      Dictionary.FieldByName(fnDict_Length).AsInteger := aF.Size;
      Dictionary.FieldByName(fnDict_Label).AsString := '<' + aF.DisplayLabel +
        '>';
      try
        if (aF.DataType = ftInteger) or
          (aF.DataType = ftFloat) then
          begin
            Dictionary.FieldByName(fnDict_Format).AsString :=
              TIntegerField(aF).DisplayFormat;
            Dictionary.FieldByName(fnDict_EditMask).AsString :=
              TIntegerField(aF).EditMask;
          end;
      except
      end;
      Dictionary.FieldByName(fnDict_Width).AsInteger := aF.DisplayWidth;
      Dictionary.FieldByName(fnDict_Units).AsString := '';
      Dictionary.FieldByName(fnDict_Comments).AsString :=
        DataTypeComment(aF.DataType);
      Dictionary.Post;
    end;
end;

procedure addFieldToDictionary(aTable: string; aF: TField);
begin
  Dictionary.Insert;
  Dictionary.FieldByName(fnDict_ID).AsString := getNextDictionaryID;
  Dictionary.FieldByName(fnDict_Tag).AsString := aTable + '.' + aF.FieldName;
  Dictionary.FieldByName(fnDict_iTag).AsInteger := 0;
  Dictionary.FieldByName(fnDict_Table).AsString := aTable;
  Dictionary.FieldByName(fnDict_Column).AsString := aF.FieldName;
  Dictionary.FieldByName(fnDict_Type).AsString := DataTypeName(aF.DataType);
  Dictionary.FieldByName(fnDict_Length).AsInteger := aF.Size;
  Dictionary.FieldByName(fnDict_Label).AsString := '<' + aF.DisplayLabel + '>';
  try
    if (aF.DataType = ftInteger) or
      (aF.DataType = ftFloat) then
      begin
        Dictionary.FieldByName(fnDict_Format).AsString :=
          TIntegerField(aF).DisplayFormat;
        Dictionary.FieldByName(fnDict_EditMask).AsString :=
          TIntegerField(aF).EditMask;
      end;
  except
  end;
  Dictionary.FieldByName(fnDict_Width).AsInteger := aF.DisplayWidth;
  Dictionary.FieldByName(fnDict_Units).AsString := '';
  Dictionary.FieldByName(fnDict_Comments).AsString :=
    DataTypeComment(aF.DataType);
  Dictionary.Post;
end;
(*
procedure TdmData.SaveDictionaryToFile;
var
  SL: TStringList;
const
  DictionaryFN = 'Dictionary.xml';

begin
  SL := TStringList.Create;
  SL.Text := cdsDict.XMLData;
  SL.SaveToFile(DictionaryFN);
  SL.Free;
end;
*)

procedure VerifyDictionary(aDS: TClientDataSet);
var
  iResult,
    i: Integer;
  aF: TField;
  bUpdated: Boolean;
  aTableName, aColumnName: string;

  procedure AddField;
  begin
    Dictionary.Insert;
    Dictionary.FieldByName(fnDict_ID).AsString := getNextDictionaryID;
    Dictionary.FieldByName(fnDict_Tag).AsString := aDS.Name + '.' +
      aF.FieldName;
    Dictionary.FieldByName(fnDict_iTag).AsInteger := I;
    Dictionary.FieldByName(fnDict_Table).AsString := aDS.Name;
    Dictionary.FieldByName(fnDict_Column).AsString := aF.FieldName;
    Dictionary.FieldByName(fnDict_Type).AsString := DataTypeName(aF.DataType);
    Dictionary.FieldByName(fnDict_Length).AsInteger := aF.Size;
    Dictionary.FieldByName(fnDict_Label).AsString := '<' + aF.DisplayLabel +
      '>';
    try
      if (aF.DataType = ftInteger) or
        (aF.DataType = ftFloat) then
        begin
          Dictionary.FieldByName(fnDict_Format).AsString :=
            TIntegerField(aF).DisplayFormat;
          Dictionary.FieldByName(fnDict_EditMask).AsString :=
            TIntegerField(aF).EditMask;
        end;
    except
    end;
    Dictionary.FieldByName(fnDict_Width).AsInteger := aF.DisplayWidth;
    Dictionary.FieldByName(fnDict_Units).AsString := '';
    Dictionary.FieldByName(fnDict_Comments).AsString :=
      DataTypeComment(aF.DataType);
    Dictionary.Post;
  end;

begin
  bUpdated := False;
  aTableName := aDS.Name;
  for i := 0 to aDS.Fields.Count - 1 do
    begin
      aF := aDS.Fields[i];
      aColumnName := aF.FieldName;

      if not Dictionary.Locate(fnDict_Table + ';' + fnDict_Column,
        VarArrayOf([aTableName, aColumnName]), []) then
        begin
          iResult := MessageDLG(
            'Dictionary Warning:' + #13#10 +
            'No Data found for:' + #13#10 +
            '            Table: <' + aTableName + '>' + #13#10 +
            '            Field: <' + aColumnName + '>' + #13#10#13#10 +
            'Do you want to add a new field description to the application dictionary?' + #13#10#13#10
            +
            '        (Select "Cancel" to ignore the rest of the table)',
            mtConfirmation, [mbYes, mbNo, mbCancel], 0);
          case iResult of
            mrYes:
              begin
                AddField;
                bUpdated := True;
              end;
            mrCancel: Break;
          end;
        end;
    end;
  if bUpdated then
    Engine.SaveDictionaryToDB;
end;

initialization
  Dictionary := nil;
  DictionaryID := 0;
end.
