unit fWV_EventLog;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ActnList, Menus, StdActns;

type
  TfrmEventLog = class(TForm)
    pnlTimeline: TPanel;
    pnlMainTitle: TPanel;
    pnlBottom: TPanel;
    lbLog: TListBox;
    Splitter1: TSplitter;
    Panel1: TPanel;
    mm: TMemo;
    pnlTitle: TPanel;
    ActionList1: TActionList;
    acRefreshLog: TAction;
    MainMenu1: TMainMenu;
    WindowClose1: TWindowClose;
    File1: TMenuItem;
    Refresh1: TMenuItem;
    N1: TMenuItem;
    Close1: TMenuItem;
    procedure lbLogClick(Sender: TObject);
    procedure acRefreshLogExecute(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    fEventLog: TStringList;
    Current: TObject;
    function findObject(anObject: TObject): Integer;
    procedure restorePosition;
    procedure RefreshLog;
  public
    { Public declarations }
    procedure UpdateLog(aLog:TStringList=nil);
    procedure EventLogPresenterOnChange(Sender:TObject);
    procedure setViewMode(aParented:Boolean=false);
  end;

var
  frmEventLog: TfrmEventLog;

procedure ShowEventLog(aModal: Boolean=True);

implementation

uses
    uWV_CCOWEvent
  , uWV_RPCEvent
  , uWV_EventLog // needed only if we want access to EL. See FormCreate
  ;

{$R *.dfm}
procedure ShowEventLog(aModal: Boolean=True);
begin
  if not Assigned(frmEventLog) then
    Application.CreateForm(TfrmEventLog,frmEventLog);
  if not Assigned(frmEventLog) then Exit;

  frmEventLog.RefreshLog;

  if frmEventLog.Parent = nil  then
    if aModal then
      frmEventLog.ShowModal
    else
      frmEventLog.Show;
end;

////////////////////////////////////////////////////////////////////////////////

procedure TfrmEventLog.FormCreate(Sender: TObject);
begin
  if Assigned(EL) then
    EL.ChangeTracker := {frmEventLog.}EventLogPresenterOnChange;
end;

procedure TfrmEventLog.RefreshLog;
begin
  if not Assigned(fEventLog) then
    exit;
  lbLog.Items.Assign(fEventLog);
  RestorePosition;
end;

procedure TfrmEventLog.UpdateLog(aLog:TStringList=nil);
begin
  if Assigned(aLog) then
    fEventLog := aLog;
  RefreshLog;
  lbLog.ItemIndex := 0;
  lbLogClick(nil);
end;

procedure TfrmEventLog.lbLogClick(Sender: TObject);
var
  ccItem: TCCOWEventItem;
  anItem: TRPCEventItem;
begin
  if lbLog.ItemIndex >= 0 then
    begin
      Current := lbLog.Items.Objects[lbLog.ItemIndex];
      if Current is TCCOWEventItem then
        begin
          ccItem := TCCOWEventItem(Current);
          mm.Text := ccItem.Context.Text;
          pnlTitle.Caption := ccItem.ItemName;
        end
      else
      if Current is TRPCEventItem then
        begin
          anItem := TRPCEventItem(Current);
          if anItem <> nil then
            begin
              mm.Text :=
                Format('%-25.25s',[anItem.RPC]) +
                  #13#10#13#10+
                FormatDateTime('hh:mm:ss.zzz',anItem.Start) + ' -- ' +
                FormatDateTime('hh:mm:ss.zzz',anItem.Stop) + '    ' +
                FormatDateTime('  (hh:mm:ss.zzz)',anItem.Stop-anItem.Start) + '    ' +
                  #13#10#13#10+
                anItem.Params.Text + #13#10 +
                anItem.Results.Text;
              pnlTitle.Caption := anItem.RPC;
            end
          else
            begin
              mm.Text := '';
              pnlTitle.Caption := '';
            end;
      end;
    end
  else
    begin
      mm.Text := '';
      pnltitle.Caption := '';
      Current := nil;
    end;
end;

function TfrmEventLog.findObject(anObject:TObject): Integer;
var
  i: Integer;
begin
  Result := -1;
  for i := 0 to lbLog.Items.Count - 1 do
    begin
      if anObject = lbLog.Items.Objects[i] then
        begin
          Result := i;
          break;
        end;
    end;
end;

procedure TfrmEventLog.restorePosition;
var
  i: Integer;
begin
  if not Assigned(Current) then
    Exit;
  i := findObject(Current);
  if i >=0 then
    lbLog.ItemIndex := i;
end;

procedure TfrmEventLog.acRefreshLogExecute(Sender: TObject);
begin
  UpdateLog(fEventLog);
end;

procedure TfrmEventLog.Close1Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmEventLog.EventLogPresenterOnChange(Sender:TObject);
begin
  UpdateLog(TStringList(Sender));
end;

procedure TfrmEventLog.setViewMode(aParented:Boolean=false);
begin
  pnlTitle.ParentColor := aParented;
  pnlMainTitle.ParentColor := aParented;
  pnlBottom.ParentColor := aParented;
end;

end.
