unit uWV_EventLog;

interface
uses
  Classes
  , SysUtils
  ;

type
  TWV_EventLog = class(TObject)
  private
    fLogLimit: Integer;
    fEventLog: TStringList;
    fChangeTracker: TNotifyEvent;
    procedure setChangeTracker(aTracker: TNotifyEvent);
    procedure setLogLimit(aLimit: Integer);
  public
    property ChangeTracker: TNotifyEvent read fChangeTracker write setChangeTracker;
    property EventLog: TStringList read fEventLog write fEventLog;
    property LogLimit: Integer read fLogLimit write setLogLimit;
    constructor Create;
    destructor Destroy; override;
  end;

var
  EL: TWV_EventLog;

function newEventLog: TWV_EventLog;

implementation

function newEventLog: TWV_EventLog;
begin
  if not Assigned(EL) then
    EL := TWV_EventLog.Create;
  result := EL;
end;

////////////////////////////////////////////////////////////////////////////////

constructor TWV_EventLog.Create;
begin
  inherited;
  fEventLog := TStringList.Create;
end;

destructor TWV_EventLog.Destroy;
begin
  EventLog.OnChange := nil;
  if Assigned(EventLog) then
    while EventLog.Count > 0 do
      begin
        if Assigned(EventLog.Objects[0]) then
          try
            TObject(EventLog.Objects[0]).Free;
          except
          end;
        EventLog.Delete(0);
      end;
end;

procedure TWV_EventLog.setChangeTracker(aTracker: TNotifyEvent);
begin
  fChangeTracker := aTracker;
  EventLog.OnChange := aTracker;
end;

procedure TWV_EventLog.setLogLimit(aLimit: Integer);
begin
  if aLimit < fLogLimit then
    begin
      while EventLog.Count > aLimit do
      begin
        if EventLog.Objects[EventLog.Count - 1] <> nil then
          EventLog.Objects[EventLog.Count - 1].Free;
        EventLog.Delete(EventLog.Count - 1);
      end;
    end;
  fLogLimit := aLimit;
end;

initialization
  newEventLog;

finalization
  FreeAndNil(EL);
end.
