unit uWV_RPCBroker;
{
================================================================================
*
*       Package:        WV - Women Health
*       Date Created:   08-07-2008
*       Site:           Hines OIFO
*       Developers:     dddddddddomain.user@domain.ext
*
*       Description:    TVAConnection implementation
*
*       Notes:          Updated version of uROR_RPCBroker
*                       by dddddomain.user@domain.ext
*
================================================================================
}
interface

uses
  SysUtils, Classes, Controls
  , Dialogs, Forms
  , TRPCB
  , CCOWRPCBroker
  , RpcConf1
  , VERGENCECONTEXTORLib_TLB
  , uWV_Utils
  , uWV_BrokerParams
  ;

type
  TVAConnection = class(TObject)
  private
    UseServerList: Boolean;
    BrokerParams: TRPCBrokerParams;
    function GetBrokerParameters(var ParamRecord: TRPCBrokerParams): boolean;
    function SetBrokerParameters(ParamRecord: TRPCBrokerParams; var RPCB:
      TCCOWRPCBroker): boolean;
  public
    ContextName: string;
    RPCBroker: TCCOWRPCBroker;
    LastErrorMessage: String;

    function Connect(aContext:String;aContextor: TContextorControl=nil):Boolean;
    function setContext(aContext:String): Boolean;
    function SelectBroker(Context: string; aContextor: TContextorControl): TCCOWRPCBroker;

    function ConnectBroker(Context:String): Boolean;

    function Disconnect: Boolean;
    function getConnectionInfo:String;
    function RPCExecute(RemoteProcedure: string;
      Parameters: array of string; MultList: TStringList;
      RetList: TStringList = nil): Boolean;overload;
    function RPCExecute(RemoteProcedure: string;
      Parameters: TStrings;
      MultList: TStringList;
      RetList: TStringList = nil): Boolean;overload;
    function WasDisconnected:Boolean;
    constructor Create;
    destructor Destroy; override;
    procedure InitBrokerParams;
  end;

implementation

const
  fmtSetContext = 'You do not have the <%s> options'+#13+ 'Please contact IRMS';
  fmtConnectionLimit = 'You exceeded the limit of connection attempts.'#13#13 +
    'Try again later.';
  fmtSignOnCanceled = 'Sign-On Cancelled while setting connection parameters';
  fmtErrAssignContextor = 'Error assigning Contextor to TCCOWRPCBroker' + #13 +
    '%s';
  fmtErrBrokerCreation = 'TCCOWRPCBroker creation error' + #13 + '%s';
  fmtSignOnIsNotComplete = 'User Sign-on is not complete.'#13#13 +
    'Error Message: '#13 + '%s'+#13#13 + 'Do you want to repeat the attempt?';

////////////////////////////////////////////////////////////////////////////////
constructor TVAConnection.Create;
begin
  inherited;
  InitBrokerParams;
end;


destructor TVAConnection.Destroy;
begin
  Disconnect;
  inherited;
end;

procedure TVAConnection.InitBrokerParams;

  procedure setDefaults;
  begin
    BrokerParams.Server := VA_BROKER_SERVER;
    BrokerParams.ListenerPort := VA_BROKER_PORT;
    BrokerParams.ClearParameters := True;
    BrokerParams.ClearResults := True;
    UseServerList := True;
  end;

  procedure SetByParams;
  var
    i: Integer;
  begin
    for i := 1 to ParamCount do
      begin
        if InString(ParamStr(i), ['/s=','-s=','s=','/server=','-server='], False) then
          begin
            BrokerParams.Server := Piece(ParamStr(i), '=', 2);
            UseServerList := False;
          end;

        if InString(ParamStr(i), ['/p=','-p=','p=','/port=','-port='], False) then
          begin
            BrokerParams.ListenerPort := StrToIntDef(Piece(ParamStr(i), '=', 2),VA_BROKER_PORT);
            UseServerList := False;
          end;

        if InString(ParamStr(i), ['/debug', '-debug'], False) then
          BrokerParams.DebugMode := True;
      end;
  end;
begin
  SetDefaults;
  SetByParams;
end;

function TVAConnection.Disconnect: Boolean;
begin
  if Assigned(RPCBroker) then
    begin
      RPCBroker.Connected := False; { Disconnect and destroy the RPC Broker }
      FreeAndNil(RPCBroker);
    end;
  Result := Assigned(RPCBroker);
end;

function TVAConnection.GetBrokerParameters(var ParamRecord: TRPCBrokerParams): boolean;
var
  SLServer, SLPort: string;

begin
  Result := False;

  initBrokerParams;

  if UseServerList then
    begin
      if GetServerInfo(SLServer, SLPort) <> 1 then
        begin
          MessageDlg(fmtSignOnCanceled, mtInformation, [mbok], 0);
          Exit; // result = False
        end
      else
        begin
          ParamRecord.Server := SLServer;
          ParamRecord.ListenerPort := StrToIntDef(SLPort, VA_BROKER_PORT);
          Application.ProcessMessages; {Refresh screen prior to connecting}
        end;
    end;
  Result := True;
end;

function TVAConnection.SetBrokerParameters(ParamRecord: TRPCBrokerParams;
  var RPCB: TCCOWRPCBroker): boolean;
//  var RPCB: TRPCBroker): boolean;

  procedure ErrorReport(aMsg:String);
  begin
    MessageDlg('Error Encountered' + #13#13 + 'User Sign-on is not complete.' + #13 +
      'Attempted connection using the following:' + #13 +
      'VistA Server: ' + RPCB.Server + #13 +
      'Listener Port: ' + IntToStr(RPCB.ListenerPort) + #13 +
      'Error Message: ' + #13 + aMsg,
      mtError,
      [mbok],
      0);
  end;

begin
  RPCB.Server := ParamRecord.Server;
  RPCB.ListenerPort := ParamRecord.ListenerPort;
  RPCB.ClearParameters := ParamRecord.ClearParameters;
  RPCB.ClearResults := ParamRecord.ClearResults;
  RPCB.AccessVerifyCodes := ParamRecord.AccessVerifyCodes;
  RPCB.DebugMode := ParamRecord.DebugMode;

  if RPCB.Socket > 0 then // ???
    Result := True
  else
    try
      RPCB.Connected := True;
      Application.ProcessMessages;
      Result := True;
    except
      on E: EBrokerError do
        begin
          ErrorReport(E.Message);
          Result := False;
        end;
    else
      raise;
    end;
end;

function TVAConnection.SelectBroker(Context: string; aContextor:TContextorControl): TCCOWRPCBroker;
var
  NewAttempt: Boolean;
  AttemptCount: Integer;
  RPCB: TCCOWRPCBroker;
const
  AttemptLimit = 3;

begin
  Result := nil;
  if not GetBrokerParameters(BrokerParams) then // process parameter string
    Exit;                                                              ;

  AttemptCount := 0;
  repeat
    NewAttempt := False;
    try
      Result := TCCOWRPCBroker.Create(Application);
    except
      on E: Exception do
        ShowMessage(Format(fmtErrBrokerCreation,[E.Message]));
    end;
    try
      Result.Contextor := aContextor;
    except
      on E: Exception do
        ShowMessage(Format(fmtErrAssignContextor,[E.Message]));
    end;

    RPCB := Result;
    if not SetBrokerParameters(BrokerParams, RPCB) then
      FreeAndNil(Result);

    if Result <> nil then
      try
        if not Result.CreateContext(Context) then
          FreeAndNil(Result);
      except
        on E: Exception do
          begin
            FreeAndNil(Result);
            Inc(AttemptCount);
            if AttemptCount > AttemptLimit then
              MessageDlg(fmtConnectionLimit, mtError, [mbok], 0)
            else
              NewAttempt := MessageDlg(Format(fmtSignOnIsNotComplete,[E.Message]),
                mtError, [mbok, mbCancel], 0) = mrOK;
          end;
      end
  until (Result <> nil) or (AttemptCount > AttemptLimit) or not NewAttempt;
end;

////////////////////////////////////////////////////////////////////////////////

function TVAConnection.Connect(aContext:String;aContextor: TContextorControl=nil):Boolean;
begin
  if Assigned(RPCBroker) then
    Disconnect;
  ContextName := aContext;
  RPCBroker := SelectBroker(ContextName,aContextor);
  Result := Assigned(RPCBroker);
end;

function TVAConnection.ConnectBroker(Context:String): Boolean;
var
  RPCB: TCCOWRPCBroker;
begin
  if Assigned(RPCBroker) then
    Disconnect;

  Result := False;
  if not GetBrokerParameters(BrokerParams) then // process parameter string
    Exit;

  ContextName := Context;

  try
    RPCB := TCCOWRPCBroker.Create(Application);
  except
    on E: Exception do
      ShowMessage(Format(fmtErrBrokerCreation,[E.Message]));
  end;

  if not SetBrokerParameters(BrokerParams, RPCB) then
    FreeAndNil(RPCB);

  if Assigned(RPCB) and not RPCB.CreateContext(Context) then
    FreeAndNil(RPCB);

  Result := Assigned(RPCB) and (RPCB.Connected);
end;

////////////////////////////////////////////////////////////////////////////////

function TVAConnection.getConnectionInfo:String;
begin
  Result := ' Disconnected';
  if Assigned(RPCBroker) then
    Result := Format(' %s -- %s @ %d -- %s ',
      [RPCBroker.User.Name,RPCBroker.Server,RPCBroker.ListenerPort,ContextName]);
end;

function TVAConnection.setContext(aContext:String): Boolean;
begin
  if (aContext <> '') and not RPCBroker.CreateContext(aContext) then
    begin
      LastErrorMessage := Err(fmtSetContext,[aContext]);
      Result := False;
    end
  else
    begin
      Result := True;
      ContextName := aContext;
    end;
end;

function TVAConnection.WasDisconnected:Boolean;
begin
  Result := Assigned(RPCBroker) and
    RPCBroker.WasUserDefined and
    RPCBroker.IsUserCleared
    ;
end;

function TVAConnection.RPCExecute(RemoteProcedure: string;
  Parameters: TStrings;
  MultList: TStringList;
  RetList: TStringList = nil): Boolean;
var
  i, j: integer;
  Broker: TRPCBroker;
begin
  if not Assigned(RPCBroker) then
    begin
      Result := False;
      Exit;
    end;

  Broker := RPCBroker;
  Broker.RemoteProcedure := RemoteProcedure;
  i := 0;
  while i < Parameters.Count do
    if (Copy(Parameters[i], 1, 1) = '@') and (Parameters[i] <> '@') then
      begin
        Broker.Param[i].Value := Copy(Parameters[i], 2, Length(Parameters[i]));
        Broker.Param[i].PType := Reference;
        inc(i);
      end
    else
      begin
        Broker.Param[i].Value := Parameters[i];
        Broker.Param[i].PType := Literal;
        inc(i);
      end;

  if MultList <> nil then
    begin
      for j := 0 to MultList.Count - 1 do
        Broker.Param[i + 1].Mult[IntToStr(j)] := MultList[j];
      Broker.Param[i + 1].PType := List;
    end;

  try
    if RetList = nil then
      Broker.Call
    else
      begin
        RetList.Clear;
        Broker.lstCall(RetList);
      end;
    Result := True;

  except
    on e: EBrokerError do
      begin
        Broker.Results.Insert(0, '-1^' + e.Message);
        Result := False;
      end
  else
    raise;
  end;
end;

function TVAConnection.RPCExecute(RemoteProcedure: string;
  Parameters: array of string; MultList: TStringList; RetList: TStringList = nil): Boolean;
var
  i: Integer;
  SLParams: TStringList;
begin
  try
    SLParams := TStringList.Create;
    i := 0;
    while i <= High(Parameters) do
      begin
        SLParams.Add(Parameters[i]);
        Inc(i);
      end;
    Result := RPCExecute(RemoteProcedure,SLParams,MultList,RetList);
  finally
    FreeAndNil(SLParams);
  end;
end;

end.

