unit uWV_RPCEvent;

interface
uses
  Classes;

type
  TRPCEventItem = class(TObject)
  public
    Start: TDateTime;
    Stop: TDateTime;
    RPCContext: string;
    RPC: string;
    Params: TStrings;
    Results: TStrings;

    constructor Create;
    destructor Destroy; override;
  end;

function getRPCEventItem(
  aStart, aStop: TDateTime;
  RemoteProcedure: string;
  Parameters: TStrings; // array of string;
  MultList: TStringList = nil;
  RetList: TStrings = nil): TRPCEventItem;

var
  ServerDelay: TDateTime;
  CheckBrokerFlag: Boolean;
  RPCCount: Integer;
//  CurrentRPCContext: String;

implementation
uses
  SysUtils
  , uWV_EventLog
  ;

////////////////////////////////////////////////////////////////////////////////

function getRPCEventItem(
  aStart, aStop: TDateTime;
  RemoteProcedure: string;
  Parameters: TStrings;  //array of string;
  MultList: TStringList = nil;
  RetList: TStrings = nil): TRPCEventItem;
var
  Item: TRPCEventItem;
  i, j: Integer;
begin
  Item := TRPCEventItem.Create;
  Item.Start := aStart;
  Item.Stop := aStop;
  Item.RPC := RemoteProcedure;

  Item.Params.Add('Params: ---------------------------------------------------');
  for i := 0 to Parameters.Count - 1 do
  begin
    if (Copy(Parameters[i], 1, 1) = '@') and (Parameters[i] <> '@') then
    begin
      Item.Params.Add('Reference ' + IntToStr(i) + ' <' + Copy(Parameters[i], 2, Length(Parameters[i])) + '>');
    end
    else
    begin
      Item.Params.Add('Literal ' + IntToStr(i) + ' <' + Parameters[i] + '>');
    end;
  end;

  if MultList <> nil then
    if MultList.Count > 0 then
    begin
      Item.Params.Add('List:');
      for j := 1 to MultList.Count do
        Item.Params.Add(MultList[j - 1]);
    end;

  if RetList <> nil then
    if RetList.Count > 0 then
    begin
      Item.Results.Add('Results: --------------------------------------------------');
      for j := 0 to RetList.Count - 1 do
        Item.Results.Add(RetList[j]);
    end;
  Result := Item;

  inc(RPCCount);
  if Assigned(EL) then
    EL.EventLog.InsertObject(0,
      Format('%4.4d ', [RPCCount]) +
      FormatDateTime('hh:mm:ss.zzz', aStart) + '   ' + RemoteProcedure
      + ' (' + Parameters.CommaText + ')' // zzzzzzandria 060724
      ,
      Result);
end;

////////////////////////////////////////////////////////////////////////////////

constructor TRPCEventItem.Create;
begin
  inherited;
  Params := TStringList.Create;
  Results := TStringList.Create;
end;

destructor TRPCEventItem.Destroy;
begin
  Params.Free;
  Results.Free;
  inherited;
end;

////////////////////////////////////////////////////////////////////////////////

initialization
  CheckBrokerFlag := False;
  RPCCount := 0;

finalization
end.
