unit uWV_RPCExecutor;
{
================================================================================
*
*       Package:        WV - Women Health
*       Date Created:   01-27-2009
*       Site:           Hines OIFO
*       Developers:     dddddddddomain.user@domain.ext
*
*       Description:    RPC Executor
*
*
================================================================================
}

interface
uses
    SysUtils
  , Classes
  , Dialogs
  , uWV_RPCBroker
  , VERGENCECONTEXTORLib_TLB
  , uROR_ContextorOnly
  , CCOWRPCBroker
  ;

type
  TWV_RPCExecutor = class(TObject)
  private
  public
    VAConnection: TVAConnection;
    LastErrorMessage: String;

    constructor Create;
    destructor Destroy;override;
    function Connect(aContextor:TContextorControl): Boolean;
    function Disconnect: Boolean;

    function RPCExecute(RemoteProcedure: string;
      Parameters: TStrings;
      RetList: TStringList): Boolean; overload;
    function RPCExecute(RemoteProcedure: string;
      Parameters: Array of String;
      RetList: TStringList): Boolean;overload;

    function IsConnected: Boolean;
    function WasDisconnected: Boolean;
    function getSubjectSuffix(aSuffix:String):String;

    function getRPCBroker: TCCOWRPCBroker; // for use with DLL
  end;

var
  WV: TWV_RPCExecutor;

function newRPCExecutor(aContextor:TCCR_CCOWContextor): TWV_RPCExecutor;
function getConnectionStatusString(aWV:TWV_RpcExecutor):String;

implementation

uses
  uWV_Utils
  , uWV_RPCEvent
  ;

function getConnectionStatusString(aWV:TWV_RpcExecutor):String;
begin
  if Assigned(aWV) then
    Result := aWV.VAConnection.getConnectionInfo
  else
    Result := 'Undefined RPC Executor';
end;


function newRPCExecutor(aContextor:TCCR_CCOWContextor): TWV_RPCExecutor;
var
  Suffix: String;
begin
  if not Assigned(WV) then
    WV := TWV_RPCExecutor.Create;
  if Assigned(WV) then
    begin
      if (not Assigned(aContextor)) or (not Assigned(aContextor.Contextor)) then
        begin
          if not WV.Connect(nil) then
            FreeAndNil(WV);
        end
      else
        begin
          aContextor.Contextor.Enabled := True;
          if WV.Connect(aContextor.Contextor) then
            begin
              aContextor.RPCBroker := WV.VAConnection.RPCBroker;
              Suffix := WV.getSubjectSuffix(aContextor.DFNItemName);
              { If the station number is not available by any reason then the
                participation in the context will be suspended and the contextor
                will be disabled. }
              if Suffix = '' then
                begin
                  aContextor.Suspend;
                  aContextor.Contextor.Enabled := False;
                end
              else
                begin
                  { Append the station number to the base name stored in
                    the property ('patient.id.mrn.dfn_'). }
                  aContextor.DFNItemName := Suffix;
                  { Process the known subjects of the current context and
                    extract available patient identifiers (convert the DFN
                    to/from the ICN if necessary). }
//                aContextor.ProcessKnownSubjects(true); - processed on upper level
                end;
            end
          else
            FreeAndNil(WV); // no delayed connection supported
        end;
    end;
  Result := WV;
end;

////////////////////////////////////////////////////////////////////////////////

constructor TWV_RPCExecutor.Create;
begin
  inherited;
  VAConnection := TVAConnection.Create;
end;

destructor TWV_RPCExecutor.Destroy;
begin
  if Assigned(VAConnection) then
    FreeAndNil(VAConnection);
  inherited;
end;

function TWV_RPCExecutor.Connect(aContextor:TContextorControl): Boolean;
var
  Context: String;
begin
  cmdLineSwitch(['/CONTEXTNAME'], Context); // zzzzzzandria 050908
  if Context = '' then
    Context := 'GMV V/M GUI';
  Result := VAConnection.Connect(Context,aContextor);
end;

function TWV_RPCExecutor.getSubjectSuffix(aSuffix:String): String;
var
  s, stn: String;

  function getStationInfo: string;
  var
    SL: TStringList;
  begin
    Result := '';
    SL := TStringList.Create;
    if RPCExecute('GMV PTSELECT', ['CCOW'], SL) then
      Result := sL[0];
    SL.Free;
  end;

begin
  { Here you should get a real station number from VistA }
  stn := getStationInfo;
  s := Piece(stn,'^',2);
  stn := Piece(stn,'^',1);
  if s = '0' then s := '_test';
  if s = '1' then s := '';
  if stn <> '' then
    Result := Piece(aSuffix, '_') + '_' + stn + s
  else
    Result := '';
end;

function TWV_RPCExecutor.Disconnect: Boolean;
begin
  Result := VAConnection.DisConnect;
end;

function TWV_RPCExecutor.RPCExecute(RemoteProcedure: string;
  Parameters: TStrings; RetList: TStringList): Boolean;
var
  aStart, aStop: TDateTime;
begin
  aStart := Now;
  Result := VAConnection.RPCExecute(RemoteProcedure,Parameters,nil,RetList);
  aStop := Now;
  getRPCEventItem(aStart, aStop, RemoteProcedure, Parameters, nil, {MultList,} RetList );
end;

function TWV_RPCExecutor.RPCExecute(RemoteProcedure: string;
  Parameters: Array of String; RetList: TStringList): Boolean;
var
  i: Integer;
  SL: TStringList;
begin
  SL := TStringList.Create;
  for i := Low(Parameters) to High(Parameters) do
    SL.Add(Parameters[i]);
  Result := RPCExecute(RemoteProcedure,SL,RetList);
  SL.Free;
end;

function TWV_RPCExecutor.WasDisconnected: Boolean;
begin
  if not assigned(VAConnection) then
    Result := False
  else
    Result := VAConnection.WasDisconnected;
end;

function TWV_RPCExecutor.getRPCBroker: TCCOWRPCBroker;
begin
  Result := nil;
  if Assigned(VAConnection) then
    Result := VAConnection.RPCBroker;
end;

function TWV_RPCExecutor.IsConnected: Boolean;
begin
  Result := Assigned(VAConnection)
    and Assigned(VAConnection.RPCBroker)
    and VAConnection.RPCBroker.Connected;
end;

end.


