unit fGMV_PtSelect;
{
================================================================================
*
*       Application:  Vitals
*       Revision:     $Revision: 1 $  $Modtime: 5/26/09 2:26p $
*       Developer:
*       Site:         Hines OIFO
*
*       Description:  Allows for a calling appliction to obtain a patient
*                     DFN either by lookup or directly and display a form
*                     showing patient identifiers and any messages that may
*                     be needed to be displayed to the user prior to selection.
*
*       Notes:
*
================================================================================
}

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls
  , uGMV_Patient
  ;

type
  TfrmGMV_PtSelect = class(TForm)
    btnCancel: TButton;
    btnOk: TButton;
    pnlPtInfo: TPanel;
    btnAgree: TButton;
    lblAgree: TLabel;
    procedure btnAgreeClick(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    fSelected:Boolean;
    PatientName: String;
    PatientDFN: String;
    procedure LoadPatient(Patient: TPatient);
    procedure LoadMessages(Patient: TPatient);
  end;

function SelectPatientByDFN(DFN: string;IgnoreCheck:Boolean=False): TPatient;

implementation

uses
  uWV_Utils
  , uGMV_FileEngine;

{$R *.DFM}
{$R RGMV_PTSELECT.RES}

function SelectPatientByDFN(DFN: string;IgnoreCheck:Boolean=False): TPatient;
var
  Patient: TPatient;
begin
  try
    Screen.Cursor := crHourGlass;
    Patient := TPatient.CreatePatientByDFN(DFN);
    Screen.Cursor := crDefault;
    if IgnoreCheck then
      begin
        Result := Patient;
        Exit;
      end;
    with TfrmGMV_PtSelect.Create(Application) do
      begin
        LoadPatient(Patient);
        LoadMessages(Patient);
        lblAgree.Visible := Patient.Sensitive;
        PatientDFN := DFN;
        PatientName := Patient.Name;
        showModal;
        if fSelected then
          Result := Patient
        else
          begin
            FreeAndNil(Patient); //Patient.free;
            Result := nil;
          end;
      end;
  finally
  end;
end;

function PatientCheckSelectByDFN(DFN: string;var Name,Info:String): Boolean;
var
  Patient: TPatient;
begin
  {Create Patient Object}
  try
    Patient := TPatient.CreatePatientByDFN(DFN);
    Name := Patient.Name;
    Info := Patient.SSN+' '+Patient.Age;
    Patient.DFN := DFN; // zzzzzzandria 2007-11-08 registering access by DFN
    if Patient.Sensitive then
      begin
        with TfrmGMV_PtSelect.Create(Application) do
          begin
            LoadPatient(Patient);
            LoadMessages(Patient);
            Position := poScreenCenter;
            PatientDFN := DFN; // zzzzzzandria 2007-11-08 registering access by DFN
            PatientName := Patient.Name;  // zzzzzzandria 2007-11-08
            showmodal;
            if fSelected then
              Result := True
            else
              Result := False;
          end;
      end
    else
      Result := True;
  finally
    FreeAndNil(Patient);
  end;
end;

////////////////////////////////////////////////////////////////////////////////

//* TfrmPtSelect *//

procedure TfrmGMV_PtSelect.LoadPatient(Patient: TPatient);
var
  i: Integer;
  PtId: TPtIdentifier;
begin
  PatientDFN := Patient.DFN;
  for i := 0 to Patient.Identifiers.Count - 1 do
    begin
      PtId := TPtIdentifier(Patient.Identifiers[i]);
      with TLabel.Create(pnlPtInfo) do
        begin
          Parent := pnlPtInfo;
          Left := 15;
          Top := i * (Canvas.TextHeight('X') + 5) + 10;
          Caption := PtId.Caption + ':';
        end;
      PtId.DisplayLabel := TLabel.Create(pnlPtInfo);
      with PtId.DisplayLabel do
        begin
          Parent := pnlPtInfo;
          Left := 170;
          Top := i * (Canvas.TextHeight('X') + 5) + 10;
          Font.Style := [fsBold];
          if Patient.Sensitive then
            Caption := PtId.DisplaySensitive
          else
            Caption := PtId.DisplayNonSensitive;
        end;
    end;

  pnlPtInfo.Height := Patient.Identifiers.Count * (Self.Canvas.TextHeight('X') + 8);

  Height := pnlPtInfo.Height + 70;
  Resize;
end;

procedure TfrmGMV_PtSelect.LoadMessages(Patient: TPatient);
var
  ErrorReturned: Boolean;
  msgBitMap: TBitMap;
  PtMsg: TPtMessage;
  i: Integer;
begin
  ErrorReturned := False;

  for i := 0 to Patient.Messages.Count - 1 do
    begin
      msgBitMap := TBitMap.Create;
      msgBitMap.Transparent := True;
      msgBitMap.TransparentMode := tmAuto;

      PtMsg := TPtMessage(Patient.Messages[i]);

      PtMsg.Panel := TPanel.Create(Self);

      with PtMsg.Panel do
        begin
          Caption := '';
          BevelInner := bvNone;
          BevelOuter := bvNone;
          Parent := Self;

          if i > 0 then
            begin
              Top := TPtMessage(Patient.Messages[i - 1]).Panel.Top +
                TPtMessage(Patient.Messages[i - 1]).Panel.Height;
            end
          else
            Top := pnlPtInfo.Top + pnlPtInfo.Height + 5;

          Left := 5;
          Width := Self.Width - (self.BorderWidth * 2) - 5;
          Height := PtMsg.Text.Count * (Canvas.TextHeight('X') + 4);
          if Height < 40 then
            Height := 40;
        end;

      with TLabel.Create(PtMsg.Panel) do
        begin
          Parent := PtMsg.Panel;
          AutoSize := False;
          Top := 5;
          Left := 45;
          Width := PtMsg.Panel.Width - 5;
          WordWrap := True;
          Caption := PtMsg.Header;
          Font.Style := [fsBold];
          AutoSize := True;
        end;

      with TLabel.Create(PtMsg.Panel) do
        begin
          Parent := PtMsg.Panel;
          AutoSize := False;
          Top := 10 + Self.Canvas.TextHeight('X');
          Left := 45;
          Width := PtMsg.Panel.Width - 50;
          WordWrap := True;
          Caption := PtMsg.Text.Text;
          AutoSize := True;
          PtMsg.Panel.Height := Height + 15;
          self.Height := PtMsg.Panel.Top + PtMsg.Panel.Height + 70;
          self.Resize;
        end;

      with TImage.Create(Self) do
        begin
          Transparent := True;
          Parent := PtMsg.Panel;
          Left := 5;
          Top := 5;
          Height := 32;
          Width := 32;
          case PtMsg.MessageLevel of
            mlInfo:
              try
                msgBitMap.LoadFromResourceName(HInstance, 'MLINFO');
              except
                on E: Exception do
                  SHOWMESSAGE('Info Error:'+E.Message);
              end;
            mlWarning:
              try
                msgBitmap.LoadFromResourceName(HInstance, 'MLWARNING');
              except
                on E: Exception do
                  SHOWMESSAGE('Warning Error:'+E.Message);
              end;
            mlSecurity:
              try
                msgBitmap.LoadFromResourceName(HInstance, 'MLSECURITY');
//                btnOk.Enabled := False;
//                btnOk.TabStop := False;
                btnOk.Default := False;
//                btnAgree.Visible := True;
//                btnAgree.TabStop := True;
              except
                on E: Exception do
                  SHOWMESSAGE('Info Security:'+E.Message);
              end;
            mlError:
              try
                msgBitmap.LoadFromResourceName(HInstance, 'MLERROR');
                btnOk.ModalResult := mrCancel;
                btnCancel.Visible := False;
                ErrorReturned := True;
              except
                on E: Exception do
                  SHOWMESSAGE('Error Error:'+E.Message);
              end;
          end;
          Picture.Bitmap := msgBitMap;
        end;

        try
          msgBitMap.Free; //AAN Feb 25, 04
        except
        end;
    end;

  if ErrorReturned then
    begin
      btnOk.ModalResult := mrCancel;
      btnOk.Visible := True;
      btnCancel.Visible := False;
      btnAgree.Visible := False;
    end;

  Caption := 'Confirm Patient Selection';
  if (Patient.Messages.Count > 0) and (ErrorReturned = False) then
    Caption := Caption + ' - Please read notifications carefully';
  if (Patient.Messages.Count > 0) and (ErrorReturned = True) then
    Caption := Caption + ' - Unable to select this record';
end;

procedure TfrmGMV_PtSelect.btnAgreeClick(Sender: TObject);
begin
  if pos('-',FEng.logPatientAccess(PatientDFN)) = 1 then
      MessageDLG('Failed to register access to *SENSITIVE* patient.'+#13#13+
        'Patient name: '+PatientName+#13#10+
        'Patient DFN:  '+PatientDFN +#13#13+
        'Notify IRM or ADPAC',mtWarning,[mbOK],0)
  else
    begin
      btnAgree.Enabled := False;
      btnAgree.Default := False;
      btnAgree.TabStop := False;
      btnAgree.Caption := 'Access has been logged';

      btnOk.Enabled := True;
      btnOk.TabStop := True;
      btnOk.Default := True;
      btnOk.SetFocus;
    end;
end;

procedure TfrmGMV_PtSelect.btnOkClick(Sender: TObject);
begin
  if lblAgree.Visible and
    (pos('-',FEng.logPatientAccess(PatientDFN)) = 1) then
          MessageDLG('Failed to register access to *SENSITIVE* patient.'+#13#13+
            'Patient name: '+PatientName+#13#10+
            'Patient DFN:  '+PatientDFN +#13#13+
            'Notify IRM or ADPAC',mtWarning,[mbOK],0)
  else
    begin
      fSelected := True;
      ModalResult := mrOK;
    end;
end;

procedure TfrmGMV_PtSelect.FormCreate(Sender: TObject);
begin
  fSelected := False;
end;

procedure TfrmGMV_PtSelect.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    ModalResult := mrCancel;
end;

end.
