unit uGMV_FileEntry;
{
================================================================================
*
*       Application:  Vitals
*       Revision:     $Revision: 1 $  $Modtime: 5/26/09 2:22p $
*       Developer:    doma.user@domain.ext
*       Site:         Hines OIFO
*
*       Description:  File Entry
*
*       Notes:
*
================================================================================
}

interface

uses
  SysUtils
  , Classes
  , Dialogs
  ;

type

  TGMV_FileReference = class(TObject)
  private
    FTypes: TStringList;
    FDDNumber: string;
    FFileName: string;
    fLoadedLast: string; // zzzzzzandria 060801
    fLoaded: Boolean; // zzzzzzandria 060801
    function GetName(Index: integer): string;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure LoadEntries(FileNumber: string);
    procedure LoadEntriesConverted(FileNumber: string);//AAN 01/22/2003 sorting problem
    function LoadEntriesStartCount(FileNumber, StartNumber: string;
      Count: Integer): Integer;   // zzzzzzandria 060801
    procedure Display;
    function IndexOfIEN(IEN: string): integer;
    function NameOfIEN(IEN:String):String;
    property Name[Index: integer]: string read GetName;
    property Loaded: Boolean read fLoaded;
  published
    property Entries: TStringList read FTypes;
  end;

  TGMV_WardLocation = class(TGMV_FileReference) // file 42
  public
    constructor Create; override;
    procedure LoadWards;
  end;

  TGMV_Teams = class(TGMV_FileReference)        // file 100.21
  public
    constructor Create; override;
  end;

  TGMV_Clinics = class(TGMV_FileReference)      // file 44
  public
    constructor Create; override;
  end;

  TGMV_NursingUnit = class(TGMV_FileReference)   // file 211.4
  public
    constructor Create; override;
  end;

  TGMV_FileEntry = class(TObject)
  private
    FDDNumber: string;
    FCaption: string;
    FIEN: string;
    FCaptionConverted:String;// AAN - sorting problem 01/21/2003
    procedure SetCaption(const Value: string);
    procedure SetCaptionConverted(const Value: string);// AAN - sorting problem 01/22/2003
    procedure SetDDNumber(const Value: string);
    procedure SetIEN(const Value: string);
    function GetIENS: string;
  public
    constructor Create;
    constructor CreateFromRPC(Data: string);
    destructor Destroy; override;
    function FieldData(FieldNumber: string): string;
  published
    property DDNumber: string read FDDNumber write SetDDNumber;
    property IEN: string read FIEN write SetIEN;
    property Caption: string read FCaption write SetCaption;
    property CaptionConverted: string read FCaptionConverted write SetCaptionConverted; // AAN - sorting problem 01/21/2003
    property IENS: string read GetIENS;
  end;

function CloneFileEntry(aFE: TGMV_FileEntry): TGMV_FileEntry;

var
  GMVWardLocations: TGMV_WardLocation;
  GMVTeams: TGMV_Teams;
  GMVClinics: TGMV_CLinics;
  GMVNursingUnits: TGMV_NursingUnit;

const
  LOAD_LIMIT = 10;

procedure InitGlobalLists;

implementation

uses
  uWV_Utils
  , uGMV_FileEngine
  ;

function CloneFileEntry(aFE: TGMV_FileEntry): TGMV_FileEntry;
begin
  Result := nil;
  if Assigned(aFE) then
    begin
      Result := TGMV_FileEntry.Create;
      Result.FDDNumber := aFE.FDDNumber;
      Result.FCaption := aFE.FCaption;
      Result.FIEN := aFE.FIEN;
      Result.FCaptionConverted := aFE.FCaptionConverted;
    end;
end;

procedure InitGlobalLists;
begin
  if not Assigned(GMVNursingUnits) then
    GMVNursingUnits := TGMV_NursingUnit.Create; // TGMV_NursingUnit.Create(RPCBroker);
  if not Assigned(GMVWardLocations) then
    GMVWardLocations := TGMV_WardLocation.Create; // TGMV_WardLocation.Create(RPCBroker);
  if not Assigned(GMVTeams) then
    GMVTeams := TGMV_Teams.Create;            // TGMV_Teams.Create(RPCBroker);
  if not Assigned(GMVClinics) then
    GMVClinics := TGMV_Clinics.Create;        // TGMV_Clinics.Create(RPCBroker);
end;

{ GMV_VitalType }

constructor TGMV_FileReference.Create;
begin
  FTypes := TStringList.Create;
  fLoaded := False; // zzzzzzandria 060801
  fLoadedLast := ''; // zzzzzzandria 060801
end;

destructor TGMV_FileReference.Destroy;
begin
  FreeAndNil(FTypes);
  inherited;
end;

procedure TGMV_FileReference.LoadEntries(FileNumber: string);
var
  i: integer;
  fe: TGMV_FileEntry;
  RetList: TStrings;
  s: String;
begin
  if not Assigned(FEng) then Exit;

  FDDNumber := FileNumber;

  RetList := fEng.getFileEntries(FDDNumber);
  if RetList.Count > 0 then
    begin
      FFileName := Copy(RetList[0], Pos('^', RetList[0]), Length(RetList[0]));
      FTypes.Sorted := True; //AAN 08/08/2002
      for i := 1 to RetList.Count - 1 do
        begin
          s := RetList[i];
          fe := TGMV_FileEntry.CreateFromRPC(s);
          FTypes.AddObject(fe.Caption, fe);
        end;
    end;
  fLoaded := True;
  RetList.Free;
end;

procedure TGMV_FileReference.LoadEntriesConverted(FileNumber: string);
var
  i: integer;
  fe: TGMV_FileEntry;
  s: String;
  RetList: TStrings;
begin
  if not Assigned(FEng) then Exit;

  FDDNumber := FileNumber;
  // pulling records in several steps to avoid timeouts
  s := '0';
  repeat
    s := FileNumber + '^'+ s;
      RetList := fEng.getFileEntries(s);
      if RetList.Count > 0 then
        begin
          FFileName := Copy(RetList[0], Pos('^', RetList[0]), Length(RetList[0]));
          FTypes.Sorted := True; //AAN 08/08/2002
          for i := 1 to RetList.Count - 1 do
            begin
              fe := TGMV_FileEntry.CreateFromRPC(RetList[i]);
              FTypes.AddObject(fe.CaptionConverted, fe);
              s := fe.FIEN;
            end;
          if RetList.Count < 12 then
            s := '';
        end;
    RetList.Free;
  until s = '';
  fLoaded := True;
end;

function TGMV_FileReference.LoadEntriesStartCount(FileNumber, StartNumber: string;
  Count: Integer): Integer;
var
  iCount, i: integer;
  fe: TGMV_FileEntry;
  s: String;
  RetList: TStrings;
begin
  Result := 0;
  if not Assigned(FEng) then Exit;

  FTypes.Sorted := True;
  FDDNumber := FileNumber;
  fLoadedLast := StartNumber;
  iCount := Count;
  repeat
    s := FileNumber + '^'+ fLoadedLast;
    RetList := fEng.getFileEntries(s);
    if RetList.Count > 1 then
      begin
        FFileName := Copy(RetList[0], Pos('^', RetList[0]), Length(RetList[0]));
        for i := 1 to RetList.Count - 1 do  // zzzzzzandria 060801
          begin
            fe := TGMV_FileEntry.CreateFromRPC(RetList[i]);
            FTypes.AddObject(fe.CaptionConverted, fe);
            fLoadedLast := fe.FIEN;
            dec(iCount);
            if iCount = 0 then break;
          end;
        if i < RetList.Count-1 then
          begin
            s := '';
            if iCount > 0 then
              fLoaded := True;
          end;
      end
    else
      begin
        fLoaded := True;
        break;
      end;
    RetList.Free;
  until (s = '') or (iCount=0);
  Result := Count - iCount;
end;

function TGMV_FileReference.IndexOfIEN(IEN: string): integer;
var
  i: integer;
begin
  Result := -1;
  for i := 0 to FTypes.Count - 1 do
    if TGMV_FileEntry(FTypes.Objects[i]).IEN = IEN then
      begin
        Result := i;
        Exit;
      end;
end;

function TGMV_FileReference.NameOfIEN(IEN:String):String;
begin
  try
    result := FTypes[IndexOfIEN(IEN)];
  except
    Result := '';
  end;
end;

procedure TGMV_FileReference.Display;
var
  x: string;
  i: integer;
begin
  x := Self.ClassName + ' display' + #13 +
    'FileName: ' + FFileName + #13 +
    'Data Dictionary: ' + FDDNumber;
  for i := 0 to FTypes.Count - 1 do
    x := x + #13 + 'IEN: ' + TGMV_FileEntry(FTypes.Objects[i]).IEN + #9 + FTypes[i];
  ShowMessage(x); {Debug tool!}
end;

function TGMV_FileReference.GetName(Index: integer): string;
var
  i: integer;
begin
  if Index = 0 then
    Result := ''
  else
    begin
      Result := '*No Such Entry*';
      for i := 0 to FTypes.Count - 1 do
        if integer(FTypes.Objects[i]) = Index then
          begin
            Result := FTypes[i];
            Exit;
          end;
    end;
end;

{ TGMV_WardLocation }

constructor TGMV_WardLocation.Create;
begin
  inherited  Create;
  //LoadEntries('42');
end;

procedure TGMV_WardLocation.LoadWards;
var
  i: integer;
  fe: TGMV_FileEntry;
  RetList: TStrings;
  s,sName,sID: String;
begin
  if not Assigned(FEng) then Exit;
  FDDNumber := '42';

  RetList := fEng.getWardLocations('A');// Return ALL Wards
  if RetList.Count > 0 then
    begin
      FFileName := '42';
      FTypes.Sorted := True; //AAN 08/08/2002
      for i := 0 to RetList.Count - 1 do
        begin
          s := RetList[i];
          sName := piece(s,'^',2);
          sID := piece(s,'^',3);
          fe := TGMV_FileEntry.CreateFromRPC('42;'+sID+'^'+sName);
          FTypes.AddObject(fe.Caption, fe);
        end;
    end;
  fLoaded := True;
  RetList.Free;
end;

{ TGMV_Teams }

constructor TGMV_Teams.Create;
begin
  inherited  Create;
  //LoadEntries('100.21');
end;

{ TGMV_CLinics }

constructor TGMV_Clinics.Create;
begin
  inherited  Create;
  //LoadEntries('44');
end;

{ TGMV_NursingUnit }

constructor TGMV_NursingUnit.Create;
begin
  inherited  Create;
  //LoadEntries('211.4');
end;

{ TGMV_FileEntry }
constructor TGMV_FileEntry.Create;
begin
  inherited;
end;

constructor TGMV_FileEntry.CreateFromRPC(Data: string);

  function ReplacePunctuation(s:String):String;
  var
    ss: String;
    i: Integer;
  const
    ALPHA = 'qwertyuiopasdfghjklzxcvbnm1234567890#QWERTYUIOPASDFGHJKLZXCVBNM-/';
    // "/" added 2008-02-12 zzzzzzandria ===================================>
  begin
    ss := '';
    for i := 1 to length(s) do
      if pos(copy(s,i,1),ALPHA) = 0 then
        ss := ss + ' '
      else
        ss := ss +copy(s,i,1);
    Result := ss;
  end;

begin
  inherited Create;
  FDDNumber := Piece(Data, ';', 1);
  FIEN := Piece(Piece(Data, ';', 2), '^', 1);
  FCaption := Piece(Data, '^', 2);
  FCaptionConverted := ReplacePunctuation(FCaption)+' ('+FIEN+')'; //AAN 01/21/2003 - Sorting Problem
  FCaptionConverted := ReplacePunctuation(FCaption); //AAN 05/21/2003 - hiding IENS
end;

destructor TGMV_FileEntry.Destroy;
begin
  inherited;
end;

procedure TGMV_FileEntry.SetCaption(const Value: string);
begin
  FCaption := Value;
end;

// AAN - sorting problem 01/22/2003
procedure TGMV_FileEntry.SetCaptionConverted(const Value: string);
begin
  FCaptionConverted := Value;
end;

procedure TGMV_FileEntry.SetDDNumber(const Value: string);
begin
  FDDNumber := Value;
end;

procedure TGMV_FileEntry.SetIEN(const Value: string);
begin
  FIEN := Value;
end;

function TGMV_FileEntry.GetIENS: string;
begin
  GetIENS := FIEN + ',';
end;

function TGMV_FileEntry.FieldData(FieldNumber: string): string;
begin
  if not Assigned(FEng) then Exit;
  Result := fEng.getFileField(FDDNumber,FieldNumber,IENS);
end;


end.
