unit uGMV_Patient;

interface
uses
  Classes, StdCtrls,ExtCtrls
  ;

type
  TMessageLevel = (mlError, mlInfo, mlWarning, mlSecurity);

type
  TPtIdentifier = class(TObject)
  private
    FCaption: string;
    FDisplayNonSensitive: string;
    FDisplaySensitive: string;
    FDisplayLabel: TLabel;
  public
  published
    property Caption: string              read FCaption write FCaption;
    property DisplayNonSensitive: string  read FDisplayNonSensitive write FDisplayNonSensitive;
    property DisplaySensitive: string     read FDisplaySensitive write FDisplaySensitive;
    property DisplayLabel: TLabel         read FDisplayLabel write FDisplayLabel;
  end;

  TPtMessage = class(TObject)
  protected
    FLevel: TMessageLevel;
    FHeader: string;
    FText: TStringList;
    FPanel: TPanel;
  public
    constructor Create(Level: TMessageLevel; HeaderText: string);
    destructor Destroy; override;
    property MessageLevel: TMessageLevel  read FLevel write FLevel;
    property Header: string               read FHeader write FHeader;
    property Text: TStringList            read FText write FText;
    property Panel: TPanel                read FPanel write FPanel;
  end;

  TPatient = class(TObject)
  private
    fDFN: string;
    fIdentifiers: TList;
    fMessages: TList;
    fSensitive: Boolean;

//    fNameLast,
//    fNameFirst,
//    fNameMiddle,
//    fNameSuffix,
    fName,
    fSex,
    fDOB,
    fSSN,
    fAge:String;

    fLocationName:String;
    fLocationID:String;

    procedure SetSensitive(NewVal: Boolean);
  public
    constructor Create;
    constructor CreatePatientByDFN(aDFN:String);
    destructor Destroy; override;
  published
    property DFN: string                  read FDFN write FDFN;
    property Identifiers: TList           read FIdentifiers write FIdentifiers;
    property Messages: TList              read FMessages write FMessages;
    property Sensitive: Boolean           read FSensitive write SetSensitive;

    property Name: String                 read FName;
    property Sex: String                  read FSex;
    property SSN: String                  read FSSN;
    property Age: String                  read FAge;
    property DOB: String                  read FDOB;
    property LocationName: String         read FLocationName;
    property LocationID: String           read FLocationID;

    procedure LoadByDFN(aDFN: String);
    procedure LoadByStringList(SL: TStringList);
  end;

function getPatientByDFN(aDFN: String): TPatient;

implementation

uses
  SysUtils
  , uGMV_FileEngine
  , uWV_Utils
  , uWV_VersionInfo
  , Dialogs
  ;

function getPatientByDFN(aDFN: String): TPatient;
begin
  Result := nil;
  if aDFN <> '' then
    begin
    Result := TPatient.CreatePatientByDFN(aDFN);
    if Assigned(Result) and Result.Sensitive then
      if pos('-',fEng.logPatientAccess(aDFN,getExeNameVersion)) = 1 then
        begin
          MessageDLG('Failed to register access to *SENSITIVE* patient.'+#13#13+
            'Patient name: '+Result.Name+#13#10+
            'Patient DFN:  '+Result.DFN +#13#13+
            'Notify IRM or ADPAC',mtWarning,[mbOK],0);
          FreeAndNil(Result);
        end;
    end;
end;

//* TPtMessage *//

constructor TPtMessage.Create(Level: TMessageLevel; HeaderText: string);
begin
  FText := TStringList.Create;
  FLevel := Level;
  FHeader := HeaderText;
end;

destructor TPtMessage.Destroy;
begin
  FText.Clear;
  FText.free;
end;

//* TPtIdentifier *//

//* TPatient *//

constructor TPatient.Create;
begin
  inherited Create;
  FIdentifiers := TList.Create;
  FMessages := TList.Create;
end;

constructor TPatient.CreatePatientByDFN(aDFN:String);
begin
  inherited Create;
  FIdentifiers := TList.Create;
  FMessages := TList.Create;
  Sensitive := False;
  DFN := aDFN;
  LoadByDFN(aDFN);
end;

destructor TPatient.Destroy;
begin
  while FIdentifiers.Count > 0 do
    begin
      TPtIdentifier(FIdentifiers[0]).free;
      FIdentifiers.Delete(0);
    end;
  FIdentifiers.free;

  while FMessages.Count > 0 do
    begin
      TPtMessage(FMessages[0]).free;
      FMessages.Delete(0);
    end;
  FMessages.free;

  inherited Destroy;
end;

procedure TPatient.SetSensitive(NewVal: Boolean);
var
  i: Integer;
begin
  FSensitive := NewVal;
  for i := 0 to Self.FIdentifiers.Count - 1 do
    with TPtIdentifier(Self.FIdentifiers[i]) do
      case NewVal of
        True: DisplayLabel.Caption := DisplaySensitive;
        False: DisplayLabel.Caption := DisplayNonSensitive;
      end;
end;

procedure TPatient.LoadByStringList(SL: TStringList);
var
  i: Integer;
  PtID: TPtIdentifier;
  PtMsg: TPtMessage;
  S: String;
begin
  try {Build the Patient Identifiers}
    i := 0;
    while i < SL.Count do
      begin
        s := SL[i];
        if Piece(s, '^', 1) = '$$PTID' then
          begin
            PtId := TPtIdentifier.Create;
            PtId.Caption := TitleCase(Piece(s, '^', 2));
            PtId.DisplayNonSensitive := Piece(s, '^', 3);
            PtId.DisplaySensitive := Piece(s, '^', 4);
            if PtId.DisplaySensitive = '' then
              PtId.DisplaySensitive := PtId.DisplayNonSensitive;

            if PtId.Caption = 'Sex' then fSex := PtId.DisplayNonSensitive;
            if PtId.Caption = 'Name' then fName := PtId.DisplayNonSensitive;
            if PtId.FCaption = 'Social Security Number' then fSSN := PtId.DisplayNonSensitive;
            if PtId.Caption = 'Date Of Birth' then
              begin
                fAge := PtId.DisplayNonSensitive;
                fDOB := PtId.DisplaySensitive;
              end;

            if PtId.Caption = 'Ward Location' then  // 050104 zzzzzzandria
              begin
                FLocationName := PtId.DisplayNonSensitive;
                FLocationID := Piece(s,'^',4);
                // zzzzzzandria 2008-02-27 -------------------------------------
                if FLocationID = '' then
                  FLocationID := FEng.getHospitalLocationByID(self.DFN);
                // zzzzzzandria 2008-02-27 -------------------------------------
              end;
            FIdentifiers.Add(ptId);
          end;
        inc(i);
      end;
    {Load any messages}
    i := 0;
    while i < SL.Count - 1 do
      begin
        if Piece(SL[i], '^', 1) = '$$MSGHDR' then
          begin
            case StrToIntDef(Piece(SL[i], '^', 2), -1) of
              0: PtMsg := TPtMessage.Create(mlError, Piece(SL[i], '^', 3));
              1: PtMsg := TPtMessage.Create(mlInfo, Piece(SL[i], '^', 3));
              2: PtMsg := TPtMessage.Create(mlWarning, Piece(SL[i], '^', 3));
              3: begin
                  PtMsg := TPtMessage.Create(mlSecurity, Piece(SL[i], '^', 3));
                  Sensitive := True;
                end;
            else
              PtMsg := TPtMessage.Create(mlInfo, Piece(SL[i], '^', 3));
            end;

            inc(i);
            while (i < SL.Count - 1) and (Piece(SL[i], '^', 1) <> '$$MSGEND') do
              begin
                ptMsg.Text.Add(SL[i]);
                inc(i);
              end;
            FMessages.Add(PtMsg);
          end;
        inc(i);
      end;
    except
    end;
end;

procedure TPatient.LoadByDFN(aDFN: String);
var
  SL: TStringList;
begin
  if fEng.RPCE.VAConnection.RPCBroker = nil then
    Exit;

  SL := FEng.getPatientInfo(aDFN);
  LoadByStringList(SL);
  SL.Free;
end;

end.
