unit uWV_FileEngine;

interface

uses
  uWV_RPCExecutor
  , Classes
  , SysUtils
  ;

type
  TFileEngine = class(TObject)
  public
    RPCE: TWV_RpcExecutor;
    constructor Create(anExecutor: TWV_RPCExecutor);
    destructor Destroy;override;

    function getFileEntries(aFile:String): TStringList;
    function getFileField(aFile, aField, anIEN: string): string;

    function getWardLocations(anOption: string = ''): TStringList;
    function getVitalsIDList: TStringList;
    function getClinicFileEntriesByName(aStartFrom, aCount, aDirection: string): TStringList;
    function getClinicPatients(aClinic, aDate: string): TStringList;
    function getNursingUnitPatients(aUnit: string): TStringList;
    function getWardPatients(aWard: string): TStringList;
    function getTeamPatients(aTeam: string): TStringList;
    function getPatientList(aTarget: string): TStringList;
    function getPatientInfo(aPatient: string): TStringList;
    function getHospitalLocationByID(anID: string): string;

    function logPatientAccess(aPatient: string): string;
  end;

var
  FEng: TFileEngine;

procedure InitFileEngine(anExecutor: TWV_RPCExecutor);

implementation

uses uGMV_FileEntry, uGMV_VersionInfo;

const
  RPC_MANAGER = 'GMV MANAGER';
  RPC_WARD_LOCATION = 'GMV WARD LOCATION';
  RPC_LOCATION_SELECT = 'GMV LOCATION SELECT';
  RPC_CLINIC_PATIENTS = 'GMV CLINIC PT';
  RPC_NUR_UNIT_PATIENTS = 'GMV NUR UNIT PT';
  RPC_TEAM_PATIENTS = 'GMV TEAM PATIENTS';
  RPC_WARD_PATIENTS = 'GMV WARD PT';
  RPC_PATIENT_SELECT = 'GMV PTSELECT';
  RPC_CREATECONTEXT = 'GMV V/M GUI';

procedure InitFileEngine(anExecutor: TWV_RPCExecutor);
begin
  if not Assigned(FEng) then
    FEng := TFileEngine.Create(anExecutor);
end;

////////////////////////////////////////////////////////////////////////////////

constructor TFileEngine.Create(anExecutor:TWV_RPCExecutor);
begin
  inherited Create;
  RPCE := anExecutor;
end;

destructor TFileEngine.Destroy;
begin
  inherited;
end;

function TFileEngine.getFileEntries(aFile: string): TStringList;
var
  SL: TStringList;
begin
  Result := nil;
  if not Assigned(RPCE) then
    Exit;
  SL := TStringList.Create;
  RPCE.RPCExecute(RPC_MANAGER, ['GETLIST', aFile], SL);
  Result := SL;
end;

function TFileEngine.getFileField(aFile, aField, anIEN: string): string;
var
  SL: TStringList;
begin
  Result := '';
  if not Assigned(RPCE) then
    Exit;
  SL := TStringList.Create;
  if RPCE.RPCExecute(RPC_MANAGER, ['GETDATA', aFile + '^' + anIEN + '^' + aField],SL) then
    Result := SL[0];
  FreeAndNil(SL);
end;

function TFileEngine.getWardLocations(anOption: string = ''): TStringList;
var
  SL: TStringList;
begin
  Result := nil;
  if not Assigned(RPCE) then
    Exit;
  SL := TStringList.Create;
  RPCE.RPCExecute(RPC_WARD_LOCATION, [anOption], SL);
  Result := SL;
end;

function TFileEngine.getVitalsIDList: TStringList;
var
  SL: TStringList;
begin
  Result := nil;
  if not Assigned(RPCE) then
    Exit;
  SL := TStringList.Create;
  RPCE.RPCExecute(RPC_MANAGER, ['VT', ''], SL);
  Result := SL;
end;

function TFileEngine.getClinicFileEntriesByName(aStartFrom, aCount, aDirection: string): TStringList;
var
  SL,
  SLS: TStringList;
  i: Integer;
  fe: TGMV_FileEntry;
begin
  Result := nil;
  if not Assigned(RPCE) then
    Exit;
  SL := TStringList.Create;
  RPCE.RPCExecute(RPC_LOCATION_SELECT,
    ['CLINIC', aStartFrom + '^' + aCount + '^' + aDirection], SL);
  if copy(SL.Text, 1, 1) = '-' then
  begin
    Result := nil;
    FreeAndNil(SL);
    Exit;
  end;
  SLS := TStringList.Create;
  for i := 1 to SL.Count - 1 do
  begin
    fe := TGMV_FileEntry.CreateFromRPC(SL[i]);
    SLS.AddObject(fe.Caption, fe);
  end;

  FreeAndNil(SL);
  Result := SLS;
end;

function TFileEngine.getNursingUnitPatients(aUnit: string): TStringList;
var
  SL: TStringList;
begin
  Result := nil;
  if not Assigned(RPCE) then
    Exit;
  SL := TStringList.Create;
  RPCE.RPCExecute(RPC_NUR_UNIT_PATIENTS, [aUnit], SL);
  Result := SL;
end;

function TFileEngine.getWardPatients(aWard: string): TStringList;
var
  SL: TStringList;
begin
  Result := nil;
  if not Assigned(RPCE) then
    Exit;
  SL := TStringList.Create;
  RPCE.RPCExecute(RPC_WARD_PATIENTS, [aWard], SL);
  Result := SL;
end;

function TFileEngine.getTeamPatients(aTeam: string): TStringList;
var
  SL: TStringList;
begin
  Result := nil;
  if not Assigned(RPCE) then
    Exit;
  SL := TStringList.Create;
  RPCE.RPCExecute(RPC_TEAM_PATIENTS, [aTeam], SL);
  Result := SL;
end;

function TFileEngine.getClinicPatients(aClinic, aDate: string): TStringList;
var
  SL: TStringList;
begin
  Result := nil;
  if not Assigned(RPCE) then
    Exit;
  SL := TStringList.Create;
  RPCE.RPCExecute(RPC_CLINIC_PATIENTS, [aClinic, aDate], SL);
  Result := SL;
end;

function TFileEngine.getPatientList(aTarget: string): TStringList;
var
  SL: TStringList;
begin
  Result := nil;
  if not Assigned(RPCE) then
    Exit;
  SL := TStringList.Create;
  RPCE.RPCExecute(RPC_PATIENT_SELECT, ['PTLKUP', '', aTarget], SL);
  Result := SL;
end;

function TFileEngine.getPatientInfo(aPatient: string): TStringList;
var
  SL: TStringList;
begin
  Result := nil;
  if not Assigned(RPCE) then
    Exit;
  SL := TStringList.Create;
  RPCE.RPCExecute(RPC_PATIENT_SELECT, ['SELECT', aPatient], SL);
  Result := SL;
end;

function TFileEngine.getHospitalLocationByID(anID: string): string;
var
  SL: TStringList;
begin
  Result := '';
  if not Assigned(RPCE) then
    Exit;
  if trim(anID) = '' then
    Exit;
  try
    SL := TStringList.Create;
    if RPCE.RPCExecute(RPC_PATIENT_SELECT, ['HOSPLOC', anID], SL) then
      Result := SL[0]
    else
      Result := '';
  except
    Result := '';
  end;
  FreeAndNil(SL);
end;

function TFileEngine.logPatientAccess(aPatient: string): string;
var
  SL: TStringList;
begin
  Result := '';
  if not Assigned(RPCE) then
    Exit;
  SL := TStringList.Create;
  RPCE.RPCExecute(RPC_PATIENT_SELECT,
    ['LOGSEC', aPatient, RPC_CREATECONTEXT + '^' + CurrentExeNameAndVersion],
    SL);
  Result := SL[0];
  FreeAndNil(SL);
end;

end.
