unit UExtendedStringGrid;

interface
   { DONE : Add column auto size double click }
   { DONE : add column moving }
   { TODO : add keep sort selection }

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, stdctrls,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Grids, math, printers, extctrls;

type
  TPrintMode = (pmPrint, pmPreview, pmPageCount);
  TSortStyle = (ssAutomatic, ssNormal, ssNumeric, ssNumericExtended, ssDateTime,
    ssTime, ssCustom);
  TExtGridState = (gsSorting, gsLoading, gsNormal);
  TSortCustom = function(const Str1, Str2: String): Integer;
  TSortDirection = (sdAscending, sdDescending);
  TSortEvent = procedure(Sender: TObject; var Col: Integer) of object;
  TGetSortStyleEvent = procedure(Sender: TObject; Col: Integer;
    var SortStyle: TSortStyle) of object;
 // TAfterSelectCellEvent = procedure (Sender: TObject; ACol, ARow: Longint) of object;
  TUpdateGridEvent = procedure(Sender: TObject; ARow: Integer) of object;
   TSizeChangedEvent = procedure(Sender: TObject;
    OldColCount, OldRowCount: Integer) of object;

  TSortOptions = class(TPersistent)
  private
    fSecondaryIndex: String;
    fCanSort: Boolean;
    fSortStyle: TSortStyle;
    fSortCaseSen: Boolean;
    fSortCol: Integer;
    fSortDirection: TSortDirection;
    fSortCustom: TSortCustom;
    fUpdateOnSizeChange: Boolean;
    fKeepSelection: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    property SortCustom: TSortCustom read fSortCustom write fSortCustom;
  private
  published
    property CanSort: Boolean read fCanSort write fCanSort;
    property SecondaryIndex: String read fSecondaryIndex write fSecondaryIndex;
    property SortStyle: TSortStyle read fSortStyle write fSortStyle;
    property SortCaseSensitive: Boolean read fSortCaseSen write fSortCaseSen;
    property SortCol: Integer read fSortCol write fSortCol;
    property SortDirection: TSortDirection read fSortDirection
      write fSortDirection;
    property UpdateOnSizeChange: Boolean read fUpdateOnSizeChange
      write fUpdateOnSizeChange;
    Property KeepSelection: Boolean read fKeepSelection write fKeepSelection;
  end;

  TPrintOptions = class(TPersistent)
  private
    fJobTitle: String;
    fPageTitle: String;
    fCopies: Cardinal;
    fPrintRange: TPrintRange;
    fPreviewPage, fFromRow, fToRow: Cardinal;
  protected
  public
  private
  published
    property JobTitle: string read fJobTitle write fJobTitle;
    property PageTitle: string read fPageTitle write fPageTitle;
    property Copies: Cardinal read fCopies write fCopies default 1;
    property PrintRange: TPrintRange read fPrintRange write fPrintRange;
    property FromRow: Cardinal read fFromRow write fFromRow;
    property ToRow: Cardinal read fToRow write fToRow;
    property PreviewPage: Cardinal read fPreviewPage write fPreviewPage;
  end;

  TSortedListEntry = record
    Str: String;
    RowNum: Integer;
    SortOption: TSortOptions;
  end;

  PSortedListEntry = ^TSortedListEntry;

  TSortedList = class(TList)
  public
    function GetItem(const I: Integer): PSortedListEntry;
    procedure Reset;
  end;

  TCellBevelStyle = (cbNone, cbRaised, cbLowered);

  { *** NOTE: This is one of the structures in TFormatOptions. }
  TCellBevel = record
    Style: TCellBevelStyle;
    UpperLeftColor: TColor;
    LowerRightColor: TColor;
  end;

   TVertAlignment = (taTopJustify, taBottomJustify, taMiddle);

  { *** NOTE: These are the display options you can set for each cell in OnGetCellFormat. }
  TFormatOptions = record
    Brush: TBrush;
    Font: TFont;
    AlignmentHorz: TAlignment;
    AlignmentVert: TVertAlignment;
    Bevel: TCellBevel;
    HideText: Boolean;
  end;

   TFormatDrawCellEvent = procedure(Sender: TObject; Col, Row: Integer;
    State: TGridDrawState; var FormatOptions: TFormatOptions) of object;

  TSortGrid = Class(Vcl.Grids.TStringGrid)
  private
    fSortedList: TSortedList;
    FSortOptions: TSortOptions;
    fOnBeginUpdate: TNotifyEvent;
    fOnEndUpdate: TNotifyEvent;
    FNowUpdating: Boolean;
    fOnBeginSort: TSortEvent;
    fOnEndSort: TSortEvent;
    fOnGetSortStyle: TGetSortStyleEvent;
    fSorting: Boolean;
    OldUpdateSize: Boolean;
    FPrintOptions: TPrintOptions;
    fPageCount: Cardinal;
    fAlignmentHorz: TAlignment;
    fAlignmentVert: TVertAlignment;
    fBevelStyle: TCellBevelStyle;
    fProportionalScrollBars: Boolean;
    fOnGetCellFormat: TFormatDrawCellEvent;
    fOnRowHeightsChanged: TNotifyEvent;
    fOnColumnWidthsChanged: TNotifyEvent;
    FMouseDownCol: Integer;
//    fPrevSelectedRow: Integer;
    FExtGridState: TExtGridState;
  //  fOnAfterSelectCell: TAfterSelectCellEvent;
    fOnRowDelete: TUpdateGridEvent;
    fOnSizeChanged: TSizeChangedEvent;
    procedure SetSortOptions(Value: TSortOptions);
    function DetermineSortStyle(const ACol: Integer): TSortStyle;
    procedure LockWindowDraw;
    procedure UnlockWindowDraw;
    procedure FixedCellClick(ACol, ARow: Longint); Override;
    procedure DrawToCanvas(ACanvas: TCanvas; Mode: TPrintMode;
      FromRow, ToRow: Integer);
    procedure SetAlignmentHorz(Value: TAlignment);
    procedure SetAlignmentVert(Value: TVertAlignment);
    procedure SetBevelStyle(Value: TCellBevelStyle);
    procedure SetProportionalScrollBars(Value: Boolean);
    procedure InitializeFormatOptions(ACol, ARow: Integer; var FmtOpts: TFormatOptions);
    function GetCellDrawState(const ACol, ARow: Integer): TGridDrawState;
    procedure UpdateScrollPage;
    procedure WM_Size(var Msg: TWMSize); message WM_Size;
    procedure RowHeightsChanged; override;
    procedure ColWidthsChanged; override;
    procedure ColumnMoved(FromIndex, ToIndex: Integer); override;
     procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure SetLoading(aValue: Boolean);
    function GetLoading: Boolean;
    function ResizingCol(X, Y: Integer): Boolean;
    Procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  Public
    constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
        procedure BeginUpdate;
    procedure EndUpdate;
    Procedure AssignRowsByStrings(Items: TStringList);
    Procedure AddRowsByStrings(Items: TStringList);
    procedure AutoSizeColumns(AlwaysSize: Boolean = False);
    procedure AutoSizeCol(const ACol: Integer; AlwaysSize: Boolean = True);
    procedure DrawCell(ACol, ARow: Integer; ARect: TRect;
      AState: TGridDrawState); override;
    procedure SaveToCSV(const FileName: String);
    procedure Print;
    procedure PrintPreview(Image: TImage);
    function PageCount: Integer;
    procedure PrintWithSetup;
    procedure Clear;
    procedure SortColumn(ACol: Integer);
    procedure DeleteRow(ARow: Integer);
    property Sorting: Boolean read fSorting;
    Property Loading: Boolean read GetLoading write SetLoading;
    procedure SizeChanged(OldColCount, OldRowCount: Integer); Override;
    procedure MoveTo(ACol, ARow: Integer; MakeVisible: Boolean = True);
    procedure FocusColRow(ACol, ARow: Integer);
  published
    property AlignmentHorz: TAlignment read fAlignmentHorz
      write SetAlignmentHorz;
    property AlignmentVert: TVertAlignment read fAlignmentVert
      write SetAlignmentVert;
    property BevelStyle: TCellBevelStyle read fBevelStyle write SetBevelStyle;
    // Doesn't work
    property ProportionalScrollBars: Boolean read fProportionalScrollBars
      write SetProportionalScrollBars;
    property OnBeginSort: TSortEvent read fOnBeginSort write fOnBeginSort;
    property OnBeginUpdate: TNotifyEvent read fOnBeginUpdate
      write fOnBeginUpdate;
    property OnEndSort: TSortEvent read fOnEndSort write fOnEndSort;
    property OnEndUpdate: TNotifyEvent read fOnEndUpdate write fOnEndUpdate;
  //  Property OnAfterSelectCell: TAfterSelectCellEvent read fOnAfterSelectCell write fOnAfterSelectCell;
    property OnRowDelete: TUpdateGridEvent read fOnRowDelete write fOnRowDelete;
    property OnGetSortStyle: TGetSortStyleEvent read fOnGetSortStyle
      write fOnGetSortStyle;
    property OnGetCellFormat: TFormatDrawCellEvent read fOnGetCellFormat
      write fOnGetCellFormat;
   property OnRowHeightsChanged: TNotifyEvent read fOnRowHeightsChanged
      write fOnRowHeightsChanged;
   property OnColumnWidthsChanged: TNotifyEvent read fOnColumnWidthsChanged
      write fOnColumnWidthsChanged;
    property OnSizeChanged: TSizeChangedEvent read fOnSizeChanged
      write fOnSizeChanged;
    property PrintOptions: TPrintOptions read FPrintOptions write FPrintOptions;
    property SortOptions: TSortOptions read FSortOptions write SetSortOptions;
  End;

procedure Register;
function ExtendedCompare(const Str1, Str2: String): Integer;
function NormalCompare(const Str1, Str2: String): Integer;
function DateTimeCompare(const Str1, Str2: String): Integer;
function NumericCompare(const Str1, Str2: String): Integer;
function TimeCompare(const Str1, Str2: String): Integer;
function Compare(Item1, Item2: Pointer): Integer;

const
  SortImageWidth = 11;
  SortTitleWidth = 8;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TSortGrid]);
end;

{$REGION 'TSortOptions'}

constructor TSortOptions.Create;
begin
  fSortDirection := sdAscending;
  fSortCol := 0;
  fSortStyle := ssNormal;
  fSortCaseSen := False;
  fCanSort := True;
  fKeepSelection := false;
end;

destructor TSortOptions.Destroy;
begin
  inherited Destroy;
end;

{$ENDREGION}

{$REGION 'TSortedList'}

function TSortedList.GetItem(const I: Integer): PSortedListEntry;
begin
  // Cast the pointer.
  Result := PSortedListEntry(Items[I]);
end;

procedure TSortedList.Reset;
var
  I: Integer;
begin
  // Dispose of anything in the list first.
  for I := 0 to Count - 1 do
    if Items[I] <> nil then
      Dispose(Items[I]);
  // Now clear the list.
  Clear;
end;

{$ENDREGION}

{$REGION 'TSortGrid'}

procedure TSortGrid.BeginUpdate;
begin
  FNowUpdating := True;
  Screen.Cursor := crHourGlass;
  // LockWindowUpdate(Handle);
  LockWindowDraw;
  Enabled := False;
  if Assigned(fOnBeginUpdate) then
    fOnBeginUpdate(Self);
  Loading := true;
  OldUpdateSize := FSortOptions.fUpdateOnSizeChange;
  FSortOptions.fUpdateOnSizeChange := False;
end;

procedure TSortGrid.EndUpdate;
begin

  // ChangeSize(ColCount, RowCount); //update scrollbars mm
  Invalidate;
  // LockWindowUpdate(0);
  UnlockWindowDraw;
  Enabled := True;
  Screen.Cursor := crDefault;
  if ParentWindow <> 0 then
    SetFocus;
  FNowUpdating := False;
  // Skal komme efter changesize, da det vil reste selection i changesize event

  FSortOptions.fUpdateOnSizeChange := OldUpdateSize;
  Loading := false;
  if Assigned(fOnEndUpdate) then
    fOnEndUpdate(Self);

end;

procedure TSortGrid.SizeChanged(OldColCount, OldRowCount: Integer);
begin
  inherited SizeChanged(OldColCount, OldRowCount);

  UpdateScrollPage;

  if FSortOptions.fUpdateOnSizeChange = True then
    SortColumn(SortOptions.fSortCol);

  if FNowUpdating = False then
    try
      MoveTo(0, 1);
    except
    end;

  if Assigned(fOnSizeChanged) then
    fOnSizeChanged(Self, OldColCount, OldRowCount);
end;

procedure TSortGrid.MoveTo(ACol, ARow: Integer; MakeVisible: Boolean = True);
begin
  if ACol < 0 then
    ACol := 0;
  if ACol > ColCount - 1 then
    ACol := ColCount - 1;
  if ARow < 1 then
    ARow := 1;
  if ARow > RowCount - 1 then
    ARow := RowCount - 1;

  if SelectCell(ACol, ARow) = True then
  begin
    LockWindowDraw;

    MoveColRow(ACol, ARow, True, True);

    Col := ACol;
    Row := ARow;

    Invalidate;

    // Make sure the selected Coloumn is Visible
    if MakeVisible then
      FocusColRow(ACol, ARow);

    UnlockWindowDraw
  end;
end;

procedure TSortGrid.FocusColRow(ACol, ARow: Integer);
var
  MaxRow: Integer;
begin
  // Just to make sure
  if ACol < 0 then
    ACol := 0;
  if ACol > ColCount - 1 then
    ACol := ColCount - 1;
  if ARow < 1 then
    ARow := 1;
  if ARow > RowCount - 1 then
    ARow := RowCount - 1;

  // Make sure the selected Coloumn is Visible
  if (ACol > (LeftCol + VisibleColCount - 1)) or (ACol < LeftCol) then
    repeat
      LeftCol := LeftCol + 1;
    until (ACol <= (LeftCol + VisibleColCount - 1)) and (ACol >= LeftCol);

  // Make sure the selected Row is at the top
  MaxRow := RowCount - VisibleRowCount;
  if MaxRow < 1 then
    MaxRow := 1;

  if ARow > MaxRow then
    TopRow := MaxRow
  else
  begin
    if ARow < 2 then
      TopRow := 1
    else
      TopRow := ARow - 1;
  end;
end;

procedure TSortGrid.LockWindowDraw;
begin
  Self.Perform(WM_SETREDRAW, Ord(False), 0);
end;

procedure TSortGrid.UnlockWindowDraw;
begin
  Self.Perform(WM_SETREDRAW, Ord(True), 0);
  Self.Invalidate;
end;

procedure TSortGrid.SetAlignmentHorz(Value: TAlignment);
begin
  fAlignmentHorz := Value;
  Invalidate;
end;

procedure TSortGrid.SetAlignmentVert(Value: TVertAlignment);
begin
  fAlignmentVert := Value;
  Invalidate;
end;

procedure TSortGrid.SetBevelStyle(Value: TCellBevelStyle);
begin
  fBevelStyle := Value;
  Invalidate;
end;

procedure TSortGrid.FixedCellClick(ACol, ARow: Longint);
var
 P: TPoint;
 ACol1, ARow1: integer;
begin
  inherited;
{GetCursorPos(P);
P := ScreenToClient(P);

MouseToCell(p.X, p.Y, ACol1, ARow1);
  if ((FSortOptions.CanSort) and (not (goColMoving in options))) or ((goColMoving in options) and (not ResizingCol(p.X, p.Y))) then
    Self.SortColumn(ACol);
 }
end;

procedure TSortGrid.AutoSizeCol(const ACol: Integer;
  AlwaysSize: Boolean = True);
var
  MaxWidth, TextW, HeaderWidth, iRow: Integer;
  Dummy: Boolean;
  FmtOpts: TFormatOptions;
begin
  if (ColWidths[ACol] = -1) and not AlwaysSize then
    Exit;

  // Resize the column to display the largest value.
  MaxWidth := 0;
  HeaderWidth := 0;
  InitializeFormatOptions(Col, iRow, FmtOpts);
  Canvas.Font := FmtOpts.Font;
  for iRow := 0 to RowCount - 1 do
  begin
    TextW := Canvas.TextWidth(Cells[ACol, iRow]);

    if iRow = 0 then
    begin
      HeaderWidth := TextW;
      if (ACol = SortOptions.fSortCol) and SortOptions.fCanSort then
        Inc(HeaderWidth, 12);
    end
    else if TextW > MaxWidth then
      MaxWidth := TextW;
  end;

  if HeaderWidth > MaxWidth then
    MaxWidth := HeaderWidth;

  if aCol = SortOptions.fSortCol then
   MaxWidth := MaxWidth + SortImageWidth;

  if MaxWidth > 0 then
    ColWidths[ACol] := MaxWidth + Canvas.TextWidth('X')
  else
    ColWidths[ACol] := Canvas.TextWidth('xx');
end;

procedure TSortGrid.AutoSizeColumns(AlwaysSize: Boolean = False);
var
  iCol: Integer;
begin
  BeginUpdate;
  try
    for iCol := 0 to ColCount - 1 do
      AutoSizeCol(iCol, AlwaysSize);
  finally
    EndUpdate;
  end;
end;

procedure TSortGrid.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  ARow, ACol: Integer;
  AColLeft: Integer;
begin
  inherited;
  MouseToCell(X, Y, ACol, ARow);
  if ARow = -1 then
    ARow := RowCount - 1;

  FMouseDownCol := ACol;

  // If col changes betweem mouse Up&Down, don't sort, bacause col has been moved


  // Autosizeclick
  if (Button = mbLeft) and (ssDouble in Shift) and (ARow = 0) and ResizingCol(X, Y) then
  begin
    // ACol changes depening on you are to the left or right of the edge betweem two cells
    // Therefore it is neccesary to check both cells to see if the cursor is betweem two cells
    AColLeft := CellRect(ACol, 0).Right;
    if (AColLeft <= X + 3) and (AColLeft >= X - 3) then
      AutoSizeCol(ACol)
    else if ACol > 0 then
    begin
      AColLeft := CellRect(ACol - 1, 0).Right;
      if (AColLeft <= X + 3) and (AColLeft >= X - 3) then
        AutoSizeCol(ACol - 1);
    end;
  end;
end;


procedure TSortGrid.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  Cell: TGridCoord;
  Resizing: Boolean;
  PrevSel: Integer;
begin
  //Grab the cell before the move (if moved)
  Cell := MouseCoord(X, Y);
  PrevSel := Self.Row;
  Inherited;
  Resizing := ResizingCol(X, Y);

  //If col moving in options and we are not resizing or there is no col moving in options
  //and the column has not changed from the mouse down
  //and fixed col click is in the option and the column is within the fixed range OR fixed row is in the options and the row is within the range
  if (not Resizing) and ((Cell.X = FMouseDownCol) and (((goFixedColClick in Options) and (Cell.X < FixedCols) and (Cell.X >= 0))
    or ((goFixedRowClick in Options) and (Cell.Y < FixedRows) and (Cell.Y >= 0))))
  then
  begin
    if (SortOptions.SortDirection = sdAscending) and
      (SortOptions.fSortCol = Cell.X) then
      SortOptions.fSortDirection := sdDescending
    else
      SortOptions.fSortDirection := sdAscending;
    if Button = mbRight then
      SortOptions.fSortDirection := sdDescending;

      //If colmoving and can sort and clicked on a fixed col
   // if (goColMoving in options) and (FSortOptions.CanSort) then
    // FixedCellClick(Cell.X, Cell.Y);
    Self.SortColumn(Cell.X);
{  end else
  if (not Resizing) and ((Cell.X = FMouseDownCol) and (((Cell.X >= FixedCols) and (Cell.X >= 0))
   or ((Cell.Y >= FixedRows) and (Cell.Y >= 0)))) and (((self.Row <> fPrevSelectedRow) and (PrevSel = Self.row)) or (self.Row <> PrevSel)) then
  begin
  //Call the after selectcell event
   fPrevSelectedRow := Self.Row;
   if Assigned(fOnAfterSelectCell) then
    fOnAfterSelectCell(self, Cell.X, Cell.Y);  }
  end;
end;

function TSortGrid.ResizingCol(X, Y: Integer): Boolean;
var
  ACol, ARow: Integer;
begin
  // If cursor inside +/- 5 pixels of a cell edge it is resizing the col
  MouseToCell(X, Y, ACol, ARow);
  Result := ((CellRect(ACol, 0).Left - 5 <= X) and
    (CellRect(ACol, 0).Left + 5 >= X) or (CellRect(ACol, 0).Right - 5 <= X) and
    (CellRect(ACol, 0).Right + 5 >= X)) and (ARow = 0);
end;


procedure TSortGrid.KeyDown(var Key: Word; Shift: TShiftState);
begin
 inherited;
{ if self.Row <> fPrevSelectedRow then
 begin
    //Call the after selectcell event
   fPrevSelectedRow := Self.Row;
   if Assigned(fOnAfterSelectCell) then
    fOnAfterSelectCell(self, self.Col, self.Row);
 end;    }
end;


procedure TSortGrid.SetLoading(aValue: Boolean);
begin
  if aValue then
   FExtGridState := gsLoading
  else
   FExtGridState := gsNormal;
end;

function TSortGrid.GetLoading: Boolean;
begin
 Result := FExtGridState = gsLoading;
end;

procedure TSortGrid.InitializeFormatOptions(ACol, ARow: Integer; var FmtOpts: TFormatOptions);
begin
  // Setup good defaults for FormatOptions.
  FmtOpts.HideText := False;
  FmtOpts.Font := Self.canvas.Font;
  FmtOpts.Brush := self.Brush;
  FmtOpts.AlignmentHorz := AlignmentHorz;
  FmtOpts.AlignmentVert := AlignmentVert;
  FmtOpts.Bevel.Style := BevelStyle;
  // Set defaults for the bevel colors.
  case BevelStyle of
    cbRaised:
      begin
        FmtOpts.Bevel.UpperLeftColor := clBtnHighlight;
        FmtOpts.Bevel.LowerRightColor := clBtnShadow;
      end;
    cbLowered:
      begin
        FmtOpts.Bevel.UpperLeftColor := clBtnShadow;
        FmtOpts.Bevel.LowerRightColor := clBtnHighlight;
      end;
  else
    FmtOpts.Bevel.UpperLeftColor := clWindow;
    FmtOpts.Bevel.LowerRightColor := clWindow;
  end;

  // Now do the OnGetCellFormat event if necessary.
  if Assigned(fOnGetCellFormat) then
    fOnGetCellFormat(Self, ACol, ARow, GetCellDrawState(ACol, ARow), FmtOpts);
end;

function TSortGrid.GetCellDrawState(const ACol, ARow: Integer): TGridDrawState;
  function PointInGridRect(Col, Row: Integer; const Rect: TGridRect): Boolean;
  begin
    Result := (Col >= Rect.Left) and (Col <= Rect.Right) and (Row >= Rect.Top)
      and (Row <= Rect.Bottom);
  end;

var
  DrawState: TGridDrawState;
begin
  DrawState := [];
  if (ARow < 1) and (ACol < 0) then
    Include(DrawState, gdFixed);
  if Focused and (ARow = Row) and (ACol = Col) then
    Include(DrawState, gdFocused);
  if PointInGridRect(ACol, ACol, Selection) then
    Include(DrawState, gdSelected);
  Result := DrawState;
end;

procedure TSortGrid.DrawCell(ACol, ARow: Integer; ARect: TRect;
  AState: TGridDrawState);
var
  atext: string;
  aWidth, I, xOffset, yOffset: Integer;
  FmtOpts: TFormatOptions;
  OgPenColor: Integer;
begin
  InitializeFormatOptions(ACol, ARow, FmtOpts);
  Canvas.Font := FmtOpts.Font;
 // canvas.Pen.Color;
  atext := Cells[ACol, ARow];
  // Calculate horizontal offset.
  case FmtOpts.AlignmentHorz of
    taRightJustify:
      xOffset := ARect.Right - ARect.Left - Canvas.TextWidth(aText) - 2;
    taCenter:
      xOffset := (ARect.Right - ARect.Left - Canvas.TextWidth(aText)) div 2;
  else
    xOffset := 2;
  end;

  // Calculate vertical offset.
  case FmtOpts.AlignmentVert of
    taBottomJustify:
      yOffset := ARect.Bottom - ARect.Top - Canvas.TextHeight(aText) - 3;
    taMiddle:
      yOffset := (ARect.Bottom - ARect.Top - Canvas.TextHeight(aText)) div 2;
  else
    yOffset := 2;
  end;

  // If this is the header then draw the text
  if ARow = 0 then
  begin
    aWidth := ColWidths[ACol] - 1;
    if (SortOptions.fSortCol = ACol) and FSortOptions.fCanSort then
      Dec(aWidth, (SortImageWidth + SortTitleWidth));

    // do we need to add ... after the text since its to small
    if Canvas.TextWidth(atext) > aWidth then
    begin
      atext[Length(atext) - 1] := '.';
      atext[Length(atext)] := '.';
      while (Canvas.TextWidth(atext) > aWidth) and (Length(atext) > 3) do
      begin
        Delete(atext, Length(atext), 1);
        atext[Length(atext) - 1] := '.';
      end;
    end;

    if (SortOptions.fSortCol = ACol) and FSortOptions.fCanSort then
    begin
      Canvas.TextRect(ARect, ARect.Left + SortTitleWidth + SortImageWidth + xOffset,
        ARect.Top + 2 + yOffset, atext);

      if DefaultDrawing and (atext <> '') then
      begin
        I := (ARect.Bottom - ARect.Top - SortImageWidth) div 2;
        if (SortOptions.fSortDirection = sdAscending) then
        begin
          OgPenColor := Canvas.Pen.Color;
          Canvas.Pen.Color := clBtnShadow;
          Canvas.MoveTo(ARect.Left + (SortTitleWidth div 2), ARect.Top + I);
          Canvas.LineTo(ARect.Left + (SortTitleWidth div 2) + SortImageWidth,
            ARect.Top + I);
          Canvas.LineTo(ARect.Left + (SortTitleWidth div 2) +
            (SortImageWidth div 2), ARect.Bottom - I);
          Canvas.Pen.Color := clBtnHighlight;
          Canvas.LineTo(ARect.Left + (SortTitleWidth div 2), ARect.Top + I);
          Canvas.Pen.Color := OgPenColor;
        end
        else
        begin
          OgPenColor := Canvas.Pen.Color;
          Canvas.Pen.Color := clBtnHighlight;
          Canvas.MoveTo(ARect.Left + (SortTitleWidth div 2) +
            (SortImageWidth div 2), ARect.Top + I);
          Canvas.LineTo(ARect.Left + (SortTitleWidth div 2), ARect.Bottom - I);
          Canvas.LineTo(ARect.Left + (SortTitleWidth div 2) + SortImageWidth,
            ARect.Bottom - I);
          Canvas.Pen.Color := clBtnShadow;
          Canvas.LineTo(ARect.Left + (SortTitleWidth div 2) +
            (SortImageWidth div 2), ARect.Top + I);
            Canvas.Pen.Color := OgPenColor;
        end;
      end;
    end
    else
      Canvas.TextRect(ARect, ARect.Left + xOffset, ARect.Top + 2 + yOffset, atext);

  end
  else
    Canvas.TextRect(ARect, ARect.Left + xOffset, ARect.Top + yOffset, aText);

    // Draw Bevel.
  if (FmtOpts.Bevel.Style <> cbNone) and (ACol >= 0) and (ARow >= 1) then
  begin
    // Draw from bottom-most lines out to mimic behaviour of
    // fixed cells when FixedXXXXLine is toggled.
    with ARect do
    begin
      if goFixedVertLine in Options then
      begin
        Canvas.Pen.Color := FmtOpts.Bevel.LowerRightColor;
        Canvas.PolyLine([Point(Right - 1, Top), Point(Right - 1, Bottom)]);
      end;
      if goFixedHorzLine in Options then
      begin
        Canvas.Pen.Color := FmtOpts.Bevel.LowerRightColor;
        Canvas.PolyLine([Point(Left, Bottom - 1), Point(Right, Bottom - 1)]);
      end;
      if goFixedVertLine in Options then
      begin
        Canvas.Pen.Color := FmtOpts.Bevel.UpperLeftColor;
        Canvas.PolyLine([Point(Left, Top), Point(Left, Bottom)]);
      end;
      if goFixedHorzLine in Options then
      begin
        Canvas.Pen.Color := FmtOpts.Bevel.UpperLeftColor;
        Canvas.PolyLine([Point(Left, Top), Point(Right, Top)]);
      end;
    end;
  end;

  if Assigned(OnDrawCell) then
    OnDrawCell(Self, ACol, ARow, ARect, AState);
end;

procedure TSortGrid.SaveToCSV(const FileName: String);
var
  F: TextFile;
  iRow, iCol: Integer;
  quote, S: string;
begin
  try
    AssignFile(F, FileName);
    Rewrite(F);
    for iRow := 0 to RowCount - 1 do
    begin
      for iCol := 0 to ColCount - 1 do
      begin
        S := Cells[iCol, iRow];
        if Pos(',', S) > 0 then
          quote := '"'
        else
          quote := '';
        if iCol = ColCount - 1 then
          Writeln(F, quote, S, quote)
        else
          Write(F, quote, S, quote, ',');
      end;
    end;
    CloseFile(F);
  except
    MessageDLG('Can''t write to file:' + #13 + FileName, mtError, [mbOK], 0);
  end;
end;

Procedure TSortGrid.AssignRowsByStrings(Items: TStringList);
var
 origFixed: Integer;
begin
 origFixed := self.FixedRows;
 self.RowCount := 0;
 AddRowsByStrings(items);
 self.FixedRows := origFixed;
end;

Procedure TSortGrid.AddRowsByStrings(Items: TStringList);
var
 ColStrLst: TStringList;
 RowStr, ColStr: String;
 ColCnt: integer;
begin
  ColStrLst := TStringList.Create;
  try
  BeginUpdate;
  try
    for RowStr in Items do
    begin
     ColCnt := 0;
     ColStrLst.Clear;
     ColStrLst.CommaText := RowStr;
     if Self.ColCount < ColStrLst.Count then
      Self.ColCount := ColStrLst.Count;
     for ColStr in ColStrLst do
     begin
       Self.Cells[ColCnt, self.RowCount -1] := ColStr;
       Inc(ColCnt);
     end;
     self.RowCount := self.RowCount + 1;
    end;
    self.RowCount := Items.Count;
  finally
   AutoSizeColumns(false);
   EndUpdate
  end;
  finally
   ColStrLst.Free;
  end;
end;

constructor TSortGrid.Create(AOwner: TComponent);
begin
  inherited;
  fSortedList := TSortedList.Create;
  FSortOptions := TSortOptions.Create;
  FPrintOptions := TPrintOptions.Create;
  fAlignmentHorz := taLeftJustify;
  fAlignmentVert := taTopJustify;
  fBevelStyle := cbNone;
  FixedCols := 0;
  FMouseDownCol := -1;
 // fPrevSelectedRow := -1;
  FExtGridState := gsNormal;
  Options := options + [goFixedRowClick];
end;

Destructor TSortGrid.Destroy;
begin
  fSortedList.Free;
  FSortOptions.Free;
  FPrintOptions.Free;
  inherited;
end;

procedure TSortGrid.Clear;
var
  I: Integer;
begin
  BeginUpdate;
  try
    Row := 1;
    for I := 1 to (RowCount - 1) do
      Rows[I].Clear;
    RowCount := 2;
    AutoSizeColumns(True);
  finally
    EndUpdate;
  end;
 // fPrevSelectedRow := -1;
end;

procedure TSortGrid.DeleteRow(ARow: Integer);
begin

  if RowCount > 2 then
    inherited DeleteRow(ARow)
  else
    Rows[1].Clear;

  if Assigned(fOnRowDelete) then
    fOnRowDelete(Self, ARow);
end;

{$ENDREGION}


{$REGION 'TSortGrid - Sort'}

function TSortGrid.DetermineSortStyle(const ACol: Integer): TSortStyle;
var
  I: Integer;
  DoNumeric, DoDateTime, DoTime: Boolean;
begin
  DoNumeric := True;
  DoDateTime := True;
  DoTime := True;

  // Note: We only go through the rows once.
  // This code depends on the fact that no
  // entry can be both a date and number.
  for I := 1 to RowCount - 1 do
  begin
    if DoNumeric then
    begin
      try
        StrToFloat(Cells[ACol, I]);
      except
        on EConvertError do
          DoNumeric := False;
      end;
    end;

    if DoTime then
    begin
      try
        StrToTime(Cells[ACol, I]);
      except
        on EConvertError do
          DoTime := False;
      end;
    end;

    if DoDateTime then
    begin
      try
        StrToDateTime(Cells[ACol, I]);
      except
        on EConvertError do
          DoDateTime := False;
      end;
    end;
  end;

  if DoNumeric then
    Result := ssNumeric
  else if DoDateTime then
    Result := ssDateTime
  else if DoTime then
    Result := ssTime
  else
    Result := ssNormal;
end;

procedure TSortGrid.SetSortOptions(Value: TSortOptions);
begin
  FSortOptions.Assign(Value);
end;

// Sorts the variable rows using Column ACol as a key
procedure TSortGrid.SortColumn(ACol: Integer);
var
  I, I2, iRow, iCol: Integer;
  Item: PSortedListEntry;
  BufferGrid, BufferGrid2: array of TStrings;
  // add to dyn array, because faster than to cells property
  PrevSelected, StringList: TStringList;
  SortStyle: TSortStyle;
  FirstSelectedRow: Boolean;
begin
  FExtGridState := gsSorting;
  try
  iCol := 0;
  if not FSortOptions.fCanSort then
    Exit; // Don't sort

  SortOptions.fSortCol := ACol;
  // set before so it can be changed in OnBeginSort if needed
  fSorting := True; // Sorting
  BeginUpdate;
  try
    if Assigned(fOnBeginSort) then
      fOnBeginSort(Self, ACol);

    // Restore selected rows
    FirstSelectedRow := True;

    if SortOptions.KeepSelection then
    begin
    PrevSelected := TStringList.Create;
    PrevSelected.Sorted := True;
    for I := Self.Selection.Top to Self.Selection.Bottom do
      PrevSelected.Add(IntToStr(I));
    end;

    StringList := TStringList.Create;
    StringList.CommaText := SortOptions.fSecondaryIndex;
    if StringList.IndexOf(IntToStr(ACol)) > -1 then
      StringList.Delete(StringList.IndexOf(IntToStr(ACol)));
    // don't sort col twice
    StringList.Insert(0, IntToStr(ACol));

    // copy grid to dynamic grid, faster
    SetLength(BufferGrid, RowCount);
    SetLength(BufferGrid2, RowCount);
    for iRow := 0 to RowCount - 1 do
    begin
      BufferGrid[iRow] := TStringList.Create;
      BufferGrid[iRow].Assign(Rows[iRow]);
      BufferGrid2[iRow] := TStringList.Create;
    end;

    for I2 := StringList.Count - 1 downto 0 do // Loop to sort all columns
    begin
      iCol := StrToInt(StringList[I2]);

      fSortedList.Reset;
      // Insert the Row Number and Key (Str) into tList class
      for I := 1 to RowCount - 1 do
      begin
        New(Item);
        Item^.Str := BufferGrid[I][iCol];
        Item^.RowNum := I;
        Item^.SortOption := SortOptions;

        fSortedList.Add(Item);
      end;

      if Assigned(fOnGetSortStyle) then
      begin
        fOnGetSortStyle(Self, iCol, SortStyle); // Get sortstyle via OnEvent
        SortOptions.fSortStyle := SortStyle;
      end
      else if SortOptions.fSortStyle = ssAutomatic then
        // Determine sortstyle auto.
        SortOptions.fSortStyle := DetermineSortStyle(iCol);

      // Sortere
      fSortedList.Sort(Compare);

      // Now rearrange the rows of the grid in sorted order.
      if iCol = StrToInt(StringList[0]) then
        // write to cells property if last sorted col
        for I := 0 to fSortedList.Count - 1 do
        begin
          Item := fSortedList.GetItem(I);
          Rows[I + 1].Assign(BufferGrid[Item^.RowNum]);

          if SortOptions.KeepSelection then
          begin
          if PrevSelected.IndexOf(IntToStr(Item^.RowNum)) > -1 then
          begin
            if FirstSelectedRow then
            begin
              FirstSelectedRow := False;
              // Delete selected rows info
              Row := I + 1; // Set row frste gang
            end;
          end;
         end;
        end
      else // else write to temp dynamic array
      begin
        for I := 0 to fSortedList.Count - 1 do
        begin
          Item := fSortedList.GetItem(I);
          BufferGrid2[I + 1].Assign(BufferGrid[Item^.RowNum]);
        end;

        for I := 0 to RowCount - 1 do
          BufferGrid[I].Assign(BufferGrid2[I]);
      end;
    end;

    StringList.Free;
    if SortOptions.KeepSelection then
      PrevSelected.Free;
    for iRow := 0 to RowCount - 1 do
    begin
      BufferGrid[iRow].Free;
      BufferGrid2[iRow].Free;
    end;

    fSorting := False;
    SortOptions.SortCustom := nil;
    if Assigned(fOnEndSort) then
      fOnEndSort(Self, iCol);
  finally
    EndUpdate;
  end;
  self.SetFocus;
  finally
    FExtGridState := gsNormal;
  end;
end;


{$ENDREGION}

{$REGION 'TSortGrid - ProportionalScrollBars'}

procedure TSortGrid.SetProportionalScrollBars(Value: Boolean);
begin
  fProportionalScrollBars := Value;
  UpdateScrollPage;
end;

procedure TSortGrid.ColWidthsChanged;
begin
  inherited ColWidthsChanged;
  if Assigned(fOnColumnWidthsChanged) then
    fOnColumnWidthsChanged(Self);
  UpdateScrollPage;
end;

procedure TSortGrid.ColumnMoved(FromIndex, ToIndex: Integer);
var
  ACol: Integer;
begin

  ACol := SortOptions.fSortCol;
  if (ACol = FromIndex) then
    SortOptions.fSortCol := ToIndex // SoctCol flyttes hertil
  else if (ACol >= ToIndex) and (ACol <= FromIndex) then
  // SortCol flyttes mod hjre
    SortOptions.fSortCol := SortOptions.fSortCol + 1
  else if (ACol <= ToIndex) and (ACol >= FromIndex) then
  // SortCol flyttes mod venstre
    SortOptions.fSortCol := SortOptions.fSortCol - 1;

  inherited ColumnMoved(FromIndex, ToIndex);
end;

procedure TSortGrid.RowHeightsChanged;
begin
  inherited RowHeightsChanged;
  if Assigned(fOnRowHeightsChanged) then
    fOnRowHeightsChanged(Self);
  UpdateScrollPage;
end;

procedure TSortGrid.WM_Size(var Msg: TWMSize);
var
  iCol, iWidth: Integer;
begin
  inherited;

  // Stretch LastCol
  if ColCount = (LeftCol + VisibleColCount) then // last col visible
  begin
    iWidth := 0;
    for iCol := LeftCol to ColCount - 2 do
      Inc(iWidth, ColWidths[iCol]);
    iWidth := Width - iWidth - (VisibleColCount + 3);
    if iWidth > 0 then
      ColWidths[ColCount - 1] := iWidth;
  end;

  Invalidate; // Avoid some gfx mistakes, but the grid flickers :-(
  UpdateScrollPage;
end;

procedure TSortGrid.UpdateScrollPage;
  function LMax(const A, B: Integer): Integer;
  begin
    Result := B;
    if A > B then
      Result := A;
  end;

var
  SI: TScrollInfo;
begin
  if not ProportionalScrollBars then
    Exit;
  { Make the scroll bar(s) proportional. }
  { To do this correctly, I should sum colwidths and rowheights,
    but I just approximate by basing the proportion on visible rows or cols
    divided by row or col count... }
  { Also, I can't really figure out Borland's scroll bar range and position
    scheme.  Thus, sometimes when you click on the end of the scroll bar, you
    still have to scroll farther with the arrows to actually get to the end
    of the grid.  If anyone knows how to fix this, PLEASE let me know... }
  if (ScrollBars = ssVertical) or (ScrollBars = ssBoth) then
  begin
    SI.cbSize := sizeof(SI);
    SI.fMask := SIF_PAGE or SIF_POS or SIF_RANGE;
    GetScrollInfo(Handle, SB_VERT, SI);
    SI.fMask := SIF_PAGE;
    SI.nPage := LMax(Round((((VisibleRowCount + 1)) / RowCount) *
      (SI.nMax - SI.nMin + 1)), 1);
    SetScrollInfo(Handle, SB_VERT, SI, False);
  end;

  if (ScrollBars = ssHorizontal) or (ScrollBars = ssBoth) then
  begin
    SI.cbSize := sizeof(SI);
    SI.fMask := SIF_PAGE or SIF_POS or SIF_RANGE;
    GetScrollInfo(Handle, SB_HORZ, SI);
    SI.fMask := SIF_PAGE;
    SI.nPage :=
      LMax(Round((((VisibleColCount) / ColCount) * (SI.nMax - SI.nMin +
      1))), 1);
    SetScrollInfo(Handle, SB_HORZ, SI, True);
  end;
end;

{$ENDREGION}

{$REGION 'TSortGrid - Print'}

function TSortGrid.PageCount: Integer;
begin
  PrintOptions.PrintRange := prAllPages;
  fPageCount := 0;
  DrawToCanvas(nil, pmPageCount, 1, RowCount - 1);
  Result := fPageCount;
end;

procedure TSortGrid.PrintPreview(Image: TImage);
begin
  PrintOptions.PrintRange := prAllPages;
  fPageCount := 0;
  DrawToCanvas(Image.Canvas, pmPreview, 1, RowCount - 1);
end;

procedure TSortGrid.DrawToCanvas(ACanvas: TCanvas; Mode: TPrintMode;
  FromRow, ToRow: Integer);
var
  PageWidth, PageHeight, PageRow, PageCol, I, iRow, FromCol, ToCol, X,
    Y: Integer;
  DoPaint: Boolean;

  function ScaleX(I: Integer): Integer;
  begin
    if Mode = pmPreview then
      Result := I
    else
      Result := I * (GetDeviceCaps(Printer.Handle, LOGPIXELSX)
        div Screen.PixelsPerInch);
  end;
  function ScaleY(I: Integer): Integer;
  begin
    if Mode = pmPreview then
      Result := I
    else
      Result := I * (GetDeviceCaps(Printer.Handle, LOGPIXELSY)
        div Screen.PixelsPerInch);
  end;

  procedure DrawCells(iRow: Integer);
  var
    iCol, I: Integer;
    r: TRect;
  begin
    // Alignment must be done another day
    for iCol := FromCol to ToCol do
    begin
      // X Offset
      X := 0; // set to 380 for 1" left margin
      for I := FromCol to iCol - 1 do
        Inc(X, ScaleX(ColWidths[I] + 1));
      // Text Rect
      r := Rect(X, Y, X + ScaleX(ColWidths[iCol]),
        Y + ScaleY(RowHeights[iRow]));
      // Draw on the Canvas
      if DoPaint then
        ACanvas.TextRect(r, X, Y, Cells[iCol, iRow]);
      // Note:
      // TextRect doesn't support text wrapping.
      // Use DrawText with Option dt_wordbreak instead.
    end;
  end;

  procedure DrawTitle; // draw Header and Footer
  var
    S: String;
    RR: TRect;
    tmpfont: TFont;
    // I have no idea why you can't use gettextwidth when acanvas = printer.canvas, it returns wrong value
  begin
    if DoPaint then
    begin
      ACanvas.Font.Size := 10;
      Canvas.Font := ACanvas.Font;
    end;
    tmpfont := Font;

    // Title
    S := PrintOptions.PageTitle;
    if DoPaint then
      ACanvas.TextOut((PageWidth div 2) - (ScaleX(Canvas.TextWidth(S) div 2)),
        ScaleY(1), S);
    // Page nr
    S := 'Page ' + IntToStr(PageRow);
    if (ToCol < ColCount - 1) or (PageCol > 1) then
      S := S + '-' + IntToStr(PageCol);
    if DoPaint then
      ACanvas.TextOut(PageWidth - ScaleX(Canvas.TextWidth(S) + 10),
        PageHeight - ScaleY(19), S);

    if DoPaint then
    begin
      ACanvas.Font.Size := Font.Size;
      Canvas.Font := tmpfont; // Delphi 4.0 warning is wrong
    end;
    RR := Rect(0, Y, X + PageWidth, Y + ScaleY(RowHeights[iRow]));
    // cleaning title background
    ACanvas.FillRect(RR); // 20141001

    Y := ScaleY(20); // header top offset
    DrawCells(0); // table headed
  end;

begin
  // page size
  PageWidth := Printer.PageWidth;
  PageHeight := Printer.PageHeight;
  if Mode = pmPreview then
  begin
    PageWidth := PageWidth div ((GetDeviceCaps(Printer.Handle, LOGPIXELSX)
      div Screen.PixelsPerInch));
    PageHeight := PageHeight div ((GetDeviceCaps(Printer.Handle, LOGPIXELSY)
      div Screen.PixelsPerInch));
    ACanvas.Brush.Color := ClWhite;
    ACanvas.FillRect(Rect(0, 0, PageWidth, PageHeight));
  end;
  if Mode <> pmPageCount then
  begin
    ACanvas.Font := Font;
    ACanvas.Font.Color := clBlack;
  end;
  PageCol := 0;
  FromCol := -2;
  ToCol := -1;
  // scan cols
  repeat
    // Scan missing cols
    if FromCol = ToCol then
      Inc(FromCol)
    else
      FromCol := ToCol + 1;
    Inc(ToCol);
    // Get Cols with width that fits page
    X := 0;
    for I := FromCol to ColCount - 1 do
    begin
      Inc(X, ScaleX(ColWidths[I] + 1));
      if X <= PageWidth then
        ToCol := I;
    end;
    PageRow := 1;
    Inc(PageCol);
    // Mode = PageCount
    Inc(fPageCount);
    // preview mode
    DoPaint := (((Mode = pmPreview) and (fPageCount = PrintOptions.PreviewPage))
      or (Mode = pmPrint));
    // Header & Footer
    DrawTitle;
    // Contents
    iRow := FromRow;
    repeat
      Inc(Y, ScaleY(RowHeights[iRow]));
      if Y <= PageHeight - ScaleY(20) then
      begin // draw contents to canvas
        DrawCells(iRow);
        Inc(iRow);
      end
      else // New page
      begin
        if (DoPaint = True) and (Mode = pmPreview) then
          Exit;
        if Mode = pmPrint then
          Printer.NewPage;
        Inc(fPageCount); // pagecount
        DoPaint :=
          (((Mode = pmPreview) and (fPageCount = PrintOptions.PreviewPage)) or
          (Mode = pmPrint));
        Inc(PageRow);
        DrawTitle;
      end;
      if (iRow = ToRow + 1) and (ToCol < ColCount - 1) and
        (Y <= PageHeight - ScaleY(20)) then
      begin
        if (DoPaint = True) and (Mode = pmPreview) then
          Exit;
        if Mode = pmPrint then
          Printer.NewPage;
        DrawTitle;
      end;
    until iRow = ToRow + 1;
  until ToCol = ColCount - 1;
end;

procedure TSortGrid.PrintWithSetup;
var
  printDialog: tprintdialog;
begin
  printDialog := tprintdialog.Create(Self);
  try
    printDialog.Options := [poSelection];

    if printDialog.Execute then
    begin
      PrintOptions.Copies := printDialog.Copies;
      PrintOptions.PrintRange := printDialog.PrintRange;
      PrintOptions.Copies := printDialog.Copies;

      if printDialog.PrintRange = prSelection then
      begin
        PrintOptions.FromRow := Self.Selection.Top;
        PrintOptions.ToRow := Self.Selection.Bottom;
      end
      else
      begin
        PrintOptions.FromRow := 1;
        PrintOptions.ToRow := Self.RowCount;
      end;

      if Printer.printers.Count = 0 then
      begin
        MessageDLG('No Printer is installed', mtError, [mbOK], 0);
        Exit;
      end;
      Printer.Title := PrintOptions.JobTitle;
      Printer.BeginDoc;
      DrawToCanvas(Printer.Canvas, pmPrint, PrintOptions.FromRow,
        PrintOptions.ToRow);
      Printer.EndDoc;
    end;
  finally
    printDialog.Free;
  end;
end;

procedure TSortGrid.Print;
begin
  if Printer.printers.Count = 0 then
  begin
    MessageDLG('No Printer is installed', mtError, [mbOK], 0);
    Exit;
  end;
  Printer.Title := PrintOptions.JobTitle;
  Printer.Copies := PrintOptions.fCopies;
  Printer.BeginDoc;
  DrawToCanvas(Printer.Canvas, pmPrint, PrintOptions.FromRow,
    PrintOptions.ToRow);
  Printer.EndDoc;
end;

{$ENDREGION}

{$REGION 'Class'}

function ExtendedCompare(const Str1, Str2: String): Integer;
var
  St1, St2, S1, S2: String;
  I, len, iPos1, iPos2: Integer;
begin
  S1 := Str1;
  S2 := Str2;

  if (S1 = '') or (S2 = '') then
  begin
    Result := -1;
    Exit;
  end;

  /// ***Deleting first chars if equal and not numbers
  len := Length(Str1);
  if Length(Str2) < len then
    len := Length(Str2); // finder korteste string

  S1 := '';
  S2 := '';
  for I := 1 to len do // Copy all non equal chars to S and S2, but all numbers
    if (Str1[I] <> Str2[I]) or (Str1[I] in ['0' .. '9']) then
    begin
      if I = len then
      begin
        S1 := S1 + Copy(Str1, I, Length(Str1));
        S2 := S2 + Copy(Str2, I, Length(Str2));
      end
      else
      begin
        S1 := S1 + Str1[I];
        S2 := S2 + Str2[I];
      end;
    end;

  /// ***Deleting last chars if equal and not numbers
  St1 := S1;
  St2 := S2;
  len := Length(St1);
  if Length(St2) < len then
    len := Length(St2); // finder korteste string

  iPos1 := Length(St1);
  iPos2 := Length(St2);
  for I := 0 to len - 1 do
    // Copy all non equal chars to S and S2, but all numbers
    if (St1[iPos1 - I] = St2[iPos2 - I]) and not(St1[iPos1 - I] in ['0' .. '9'])
    then
    begin
      Delete(S1, Length(S1), 1);
      Delete(S2, Length(S2), 1);
    end;

  try
    if S1 = S2 then
      Result := 0
    else if StrToInt(S1) < StrToInt(S2) then
      Result := -1
    else
      Result := 1;
  except
    Result := NormalCompare(S1, S2);
  end;
end;

function NormalCompare(const Str1, Str2: String): Integer;
begin
  Result := StrComp(PChar(Str1), PChar(Str2));
end;

function DateTimeCompare(const Str1, Str2: String): Integer;
var
  Val1, Val2: TDateTime;
begin
  try
    Val1 := StrToDateTime(Str1);
    Val2 := StrToDateTime(Str2);
    if Val1 < Val2 then
      Result := -1
    else if Val2 < Val1 then
      Result := 1
    else
      Result := 0;
  except
    Result := NormalCompare(Str1, Str2);
  end;
end;

function TimeCompare(const Str1, Str2: String): Integer;
var
  Val1, Val2: TDateTime;
begin
  try
    Val1 := StrToTime(Str1);
    Val2 := StrToTime(Str2);
    if Val1 < Val2 then
      Result := -1
    else if Val2 < Val1 then
      Result := 1
    else
      Result := 0;
  except
    Result := NormalCompare(Str1, Str2);
  end;
end;

function NumericCompare(const Str1, Str2: String): Integer;
var
  Val1, Val2: Extended;
begin
  try
    Val1 := StrToFloat(Str1);
    Val2 := StrToFloat(Str2);
    if Val1 < Val2 then
      Result := -1
    else if Val2 < Val1 then
      Result := 1
    else
      Result := 0;
  except
    Result := NormalCompare(Str1, Str2);
  end;
end;

function Compare(Item1, Item2: Pointer): Integer;
var
  Entry1, Entry2: PSortedListEntry;
begin
  Entry1 := Item1;
  Entry2 := Item2;

  // Handle Case-Insensitivity.
  if Entry1^.SortOption.fSortCaseSen = False then
  begin
    Entry1^.Str := LowerCase(Entry1^.Str);
    Entry2^.Str := LowerCase(Entry2^.Str);
  end;

  // Determine compare type and do the comparison.
  case Entry1^.SortOption.SortStyle of
    ssNumericExtended:
      Result := ExtendedCompare(Entry1^.Str, Entry2^.Str);
    ssNumeric:
      Result := NumericCompare(Entry1^.Str, Entry2^.Str);
    ssDateTime:
      Result := DateTimeCompare(Entry1^.Str, Entry2^.Str);
    ssTime:
      Result := TimeCompare(Entry1^.Str, Entry2^.Str);
    ssCustom:
      Result := Entry1^.SortOption.SortCustom(Entry1^.Str, Entry2^.Str)
  else
    Result := NormalCompare(Entry1^.Str, Entry2^.Str);
  end;

  // Now, make sure we don't swap the rows if the Keys are equal.
  // If they're equal then we sort by row number.
  if Result = 0 then
  begin
    if Entry1^.RowNum < Entry2^.RowNum then
      Result := -1
    else if Entry1^.RowNum > Entry2^.RowNum then
      Result := 1
    else
      Result := 0; // Sometimes an item does get compared to itself.
  end
  else // Reverse polarity if descending sort.
    if Entry1^.SortOption.SortDirection = sdDescending then
      Result := -1 * Result;
end;

{$ENDREGION}

end.

