{*****************************************************
  NPPD Detail Display - GUI    ver 3.6.75   November,2005
  Office of Information Field Office
  Department of Veterans Affairs
  Hines,Illinois

  Developers: Helen Corkwell, Sam Spencer, Ken Nelson
  Description: GUI to view and print NPPD purchase data
  Requires: RMPR*3.0*71 - RMPR*3.0*59 - RMPR*3.0*75 - RMPR*3.0*136

*******************************************************}

unit PROS_NPPDDetailDisplay_Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls,MFunStr, MyGrids, SortGrid, SRGrad, ESBDates, Buttons, Trpcb,
  ExtCtrls, ShellAPI, Rpcconf1, Menus, NumEdit, Registry, FileCtrl, ActnList,
  Printers, ProgBar, Fmcntrls, AppEvnts
  , CCOWRPCBroker
  ;

type
  TfrmMain = class(TForm)
    imgGradient: TSRGradient;
    grdMain: TSortGrid;
    edtBegDate: TEdit;
    edtEndDate: TEdit;
    dtpkrBegDate: TDateTimePicker;
    dtpkrEndDate: TDateTimePicker;
    btnDisplay: TBitBtn;
    bnrMain: TLabel;
    lblBegDate: TLabel;
    lblEndDate: TLabel;
    btnPrint: TBitBtn;
    dlgPrint: TPrintDialog;
    btnExcel: TBitBtn;
    btnClose: TBitBtn;
    lblRecFound: TLabel;
    bnrExclude: TLabel;
    lblDisRecFound: TLabel;
    mnuMain: TMainMenu;
    mnuItemHelp: TMenuItem;
    mnuItemFile: TMenuItem;
    mnuItemBegDate: TMenuItem;
    mnuItemEndDate: TMenuItem;
    mnuItemDisplay: TMenuItem;
    menuItemPrint: TMenuItem;
    mnuItemExcel: TMenuItem;
    mnuItemExit: TMenuItem;
    mnuSep1: TMenuItem;
    mnuSep2: TMenuItem;
    stsbrMain: TStatusBar;
    mnuItemContents: TMenuItem;
    mnuSep3: TMenuItem;
    mnuSep4: TMenuItem;
    mnuItemSec508: TMenuItem;
    btnClear: TBitBtn;
    mnuItemAbout: TMenuItem;
    pnlProgressBar: TPanel;
    cbxCustomData1: TFMComboBoxLookUp;
    lblSelectCustomDataBanner: TLabel;
    lblCustomData1: TFMLabel;
    lblCustomData2: TFMLabel;
    lblCustomData3: TFMLabel;
    cbxCustomData2: TFMComboBoxLookUp;
    cbxCustomData3: TFMComboBoxLookUp;
    appNPPDDetailDisplay: TApplicationEvents;
    svdlgMain: TSaveDialog;
    lblExcelPath: TLabel;
    edNPPD_DUZ: TEdit;
    procedure edtBegDateChange(Sender: TObject);
    procedure dtpkrBegDateChange(Sender: TObject);
    procedure edtEndDateChange(Sender: TObject);
    procedure dtpkrEndDateChange(Sender: TObject);
    procedure btnDisplayClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure dtpkrBegDateEnter(Sender: TObject);
    procedure btnExcelClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure CheckSec508;
    procedure mnuItemBegDateClick(Sender: TObject);
    procedure mnuItemDisplayClick(Sender: TObject);
    procedure menuItemPrintClick(Sender: TObject);
    procedure mnuItemExcelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DisplayHint(Sender: TObject);
    procedure mnuItemAboutClick(Sender: TObject);
    procedure mnuItemContentsClick(Sender: TObject);
    procedure mnuItemExitClick(Sender: TObject);
    procedure mnuItemEndDateClick(Sender: TObject);
    procedure dtpkrBegDateKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure dtpkrEndDateKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure dtpkrEndDateEnter(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mnuItemSec508Click(Sender: TObject);
    procedure grdMainGetCellFormat(Sender: TObject; Col, Row: Integer;
      State: TGridDrawState; var FormatOptions: TFormatOptions;
      var CheckBox, Combobox, Ellipsis: Boolean);
    procedure grdMainGetSortStyle(Sender: TObject; Col: Integer;
      var SortStyle: TSortStyle);
    procedure btnClearClick(Sender: TObject);
    procedure cbxCustomData1DropDown(Sender: TObject);
    procedure cbxCustomData1Exit(Sender: TObject);
    procedure cbxCustomData1Enter(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure cbxCustomData2DropDown(Sender: TObject);
    procedure cbxCustomData3DropDown(Sender: TObject);
    procedure cbxCustomData2Enter(Sender: TObject);
    procedure cbxCustomData3Enter(Sender: TObject);
    procedure cbxCustomData2Exit(Sender: TObject);
    procedure cbxCustomData3Exit(Sender: TObject);
    procedure appNPPDDetailDisplayMessage(var Msg: tagMSG;
      var Handled: Boolean);
  private
  ExcelFile : string;
    { Private declarations }
//    MyAutoResize : TAutoResize;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;
  Sec508,                  //Sec508 status from registry
  MDate1, MDate2,          // First and second 'Mumps' dates for search
  CustomData1_FieldName, CustomData1_FieldNumber,
  CustomData2_FieldName, CustomData2_FieldNumber,
  CustomData3_FieldName, CustomData3_FieldNumber : string;
  BegDtpkrDate, EndDtpkrDate : TDateTime;



function executeMyMainForm(aDuz: String;const aBroker: TCCOWRPCBroker): Integer;

implementation

uses PROS_Menu_About;

//uses PROS_NPPDDetailDisplay_About;

{$R *.DFM}

var
  LocalBroker: TCCOWRPCBroker;

function executeMyMainForm(aDuz: String;const aBroker: TCCOWRPCBroker):Integer;
begin
  if not assigned(aBroker) then Exit;
  localBroker := aBroker;
  if not assigned(frmMain) then
    Application.CreateForm(TfrmMain,frmMain);

  frmMain.edNPPD_DUZ.Text := aDUZ;
  Result := frmMain.ShowModal;
end;

procedure TfrmMain.CheckSec508; //Checks the SEC508 registry Key
var
  reg : TRegistry;
begin
   reg := TRegistry.Create;
   reg.rootkey := HKEY_LOCAL_MACHINE;
   reg.OpenKey('\SOFTWARE\VISTA\PROS\NPPDDetailDisplay',False);
   Sec508 := reg.ReadString('Sec508');  //Assign result to global
   reg.Free;
end;


procedure TfrmMain.DisplayHint(Sender: TObject);
begin
  stsbrMain.SimpleText := Application.Hint; //For Sec508
end;


procedure DeleteFiles(sMask, sPath: string);
var
  SearchRec: TSearchRec;
  Found: Integer;
begin
  sPath := (sPath);
  Found := SysUtils.FindFirst(sPath + sMask, faAnyFile, SearchRec); 
  try
    while (Found = 0) do
    begin
      if not (SearchRec.Attr and faDirectory > 0) then
        SysUtils.DeleteFile(sPath + SearchRec.Name);
      Found := SysUtils.FindNext(SearchRec);
    end;
  finally
    SysUtils.FindClose(SearchRec);
  end;
end; // Usage:  DeleteFiles('*.txt', 'c:\myfiles');



//First Date
procedure TfrmMain.edtBegDateChange(Sender: TObject);
var
  s1,d1,d2,d3 : string;        //Creates the Jun 01,2002 display date
begin                          //from the tossed calendar pick
s1 := '';                      //Clears everything
  if edtBegDate.Text <> '' then
   begin
        d1 := Date2ShortMonth(dtpkrBegDate.Date);
       if Date2Day(dtpkrBegDate.Date) < 10 then  //Pad with 0 if needed
        d2:= '0'+ IntToStr(Date2Day(dtpkrBegDate.Date))
       else
        d2 := IntToStr(Date2Day(dtpkrBegDate.Date));  //Or not
        d3 := IntToStr(Date2Year(dtpkrBegDate.Date));

   s1 := d1+' '+d2+', '+d3; //Assembles the pieces
   edtBegDate.text := s1         //Display

   end;
end;

procedure TfrmMain.dtpkrBegDateChange(Sender: TObject);
var
d1,d2,d3,d4 : string;         //Gets the user input from the calendar
begin
      MDate1 := '';
      if (Date2Year(dtpkrBegDate.Date) < 1900) then  //Checks century
    d1 := '1';
      if (Date2Year(dtpkrBegDate.Date) >= 1900) and
         (Date2Year(dtpkrBegDate.Date) < 2000) then
    d1 := '2';
      if (Date2Year(dtpkrBegDate.Date) >= 2000) then
    d1 := '3';

    d2 := Copy(IntToStr(Date2Year(dtpkrBegDate.Date)),3,2); //Gets year

      if Date2Month(dtpkrBegDate.Date) < 10 then //Pad month with 0 if needed
    d3:= '0'+ IntToStr(Date2Month(dtpkrBegDate.Date))
       else
    d3 := IntToStr(Date2Month(dtpkrBegDate.Date));

     if Date2Day(dtpkrBegDate.Date) < 10 then  //Pad day with 0 if needed
    d4:= '0'+ IntToStr(Date2Day(dtpkrBegDate.Date))
       else
    d4 := IntToStr(Date2Day(dtpkrBegDate.Date));

MDate1 := d1 + d2+ d3 + d4;                    //Assemble MUMPS date style
edtBegDate.Text := Date2Str(dtpkrBegDate.Date);  //Toss the date to Edit

end;


//*************************

//Second Date
procedure TfrmMain.edtEndDateChange(Sender: TObject);
var
  s1,d1,d2,d3: string;        //Creates the Jun 01,2002 display date
begin                         //from the tossed calendar pick
s1 := '';                     //Clears everything
  if edtEndDate.Text <> '' then
   begin
        d1 := Date2ShortMonth(dtpkrEndDate.Date);
       if Date2Day(dtpkrEndDate.Date) < 10 then  //Pad with 0 if needed
        d2:= '0'+ IntToStr(Date2Day(dtpkrEndDate.Date))
       else
        d2 := IntToStr(Date2Day(dtpkrEndDate.Date));  //Or not
        d3 := IntToStr(Date2Year(dtpkrEndDate.Date));

   s1 := d1+' '+d2+', '+d3;    //Assembles the pieces
   edtEndDate.text := s1          //Display

   end;
end;


procedure TfrmMain.dtpkrEndDateChange(Sender: TObject);
var
d1,d2,d3,d4 : string;
                            //Gets the user input from the calendar
begin
      MDate2 := '';
      if (Date2Year(dtpkrEndDate.Date) < 1900) then  //Checks century
    d1 := '1';
      if (Date2Year(dtpkrEndDate.Date) >= 1900) and
         (Date2Year(dtpkrEndDate.Date) < 2000) then
    d1 := '2';
      if (Date2Year(dtpkrEndDate.Date) >= 2000) then
    d1 := '3';

    d2 := Copy(IntToStr(Date2Year(dtpkrEndDate.Date)),3,2);//Gets year

        if Date2Month(dtpkrEndDate.Date) < 10 then //Pad month with 0 if needed
    d3:= '0'+ IntToStr(Date2Month(dtpkrEndDate.Date))
       else
    d3 := IntToStr(Date2Month(dtpkrEndDate.Date));

        if Date2Day(dtpkrEndDate.Date) < 10 then  //Pad day with 0 if needed
    d4:= '0'+ IntToStr(Date2Day(dtpkrEndDate.Date))
       else
    d4 := IntToStr(Date2Day(dtpkrEndDate.Date));

  MDate2 := d1 + d2+ d3 + d4;          //Assemble MUMPS date style
  edtEndDate.Text := Date2Str(dtpkrBegDate.Date); //Toss the date to Edit
 end;


procedure TfrmMain.btnDisplayClick(Sender: TObject);  //Fill the grid
var
i : integer;

  ani: TAnimationThread;
  r: TRect;

begin
  lblDisRecFound.Caption := '0';
  if LocalBroker.Connected = false then      //Checks if user is logged on
      MessageDlg('Please Sign On first.', mtInformation,
      [mbOk], 0)
  else
    begin
     if (MDate1 = '') or (MDate2 = '') then
       MessageDlg('Please select a date range.', mtInformation,
      [mbOk], 0)
     else
   begin
    if StrToInt(MDate1) <= StrToInt(Mdate2) then //Test if Date1 before Date2
   begin
     begin
      for i :=1 to grdMain.RowCount -1 do   //If selected will clear the grid
      grdMain.Rows[i].Clear;
      grdMain.Row := 1;
      grdMain.Cells[0,1] :='';

    end;

  if DaysApart (dtpkrBegDate.Date,dtpkrEndDate.Date) > 100 //Date Range checking
  then  MessageDlg('You have selected a range'+#13+'of more than 100 days.', mtInformation,
      [mbOk], 0)
   else

begin
    btnDisplay.Caption := 'Searching';
    LocalBroker.CreateContext('RMPR NPPD GUI');
    LocalBroker.RemoteProcedure := 'RMPR NPPD LIST';
    LocalBroker.Param[0].Value := MDate1;
    LocalBroker.Param[0].PType := literal;
    LocalBroker.Param[1].Value := MDate2;
    LocalBroker.Param[1].PType := literal;
    LocalBroker.Param[2].Value := CustomData1_FieldNumber;
    LocalBroker.Param[2].PType := literal;
    LocalBroker.Param[3].Value := CustomData2_FieldNumber;
    LocalBroker.Param[3].PType := literal;
    LocalBroker.Param[4].Value := CustomData3_FieldNumber;
    LocalBroker.Param[4].PType := literal;
    LocalBroker.RPCTimeLimit := 600;    //10 Min ?!?

    Try
    //
     pnlProgressBar.Visible := true;
     r := pnlProgressBar.clientrect;
     InflateRect( r, - pnlProgressBar.bevelwidth, - pnlProgressBar.bevelwidth );
     ani := TanimationThread.Create( pnlProgressBar, r, pnlProgressBar.color, clBlue, 25 );
     Application.ProcessMessages;
     LocalBroker.Call;
     //Sleep( 1000 );  {replace with query.Open or such}
     pnlProgressBar.Visible := False;
     ani.Terminate;
    except
     on E:EBrokerError do
       begin
       Showmessage('Broker Error: '+ E.Message);
       end;
     on E:EAccessViolation do
       Showmessage('E.Message');
     else
       raise;
     end;
    btnDisplay.Caption := '&Display';
    lblDisRecFound.Caption := IntToStr(LocalBroker.Results.Count);


 for i:= 0 to LocalBroker.Results.Count -1  do
   begin

   with grdMain do           // Sets column headers on new DISPLAY
 begin
    Cells[0,0] := 'Site';
    Cells[1,0] := 'Date';
    Cells[2,0] := 'Type';
    Cells[3,0] := 'Form';
    Cells[4,0] := 'Patient';
    Cells[5,0] := 'SSN';
    Cells[6,0] := 'IEN';
    Cells[7,0] := 'Brief Description';
    Cells[8,0] := 'HCPCS';
    Cells[9,0] := 'HCPCS Desc';
    Cells[10,0] := 'NPPD';
    Cells[11,0] := 'Initi';
    Cells[12,0] := 'Sus IA';
    Cells[13,0] := 'Cost';
    Cells[14,0] := 'Qty';
    Cells[15,0] := 'V/C';
    Cells[16,0] := 'Vendor';
    Cells[17,0] := 'Grp#';
    Cells[18,0] := CustomData1_FieldName;
    Cells[19,0] := CustomData2_FieldName;
    Cells[20,0] := CustomData3_FieldName;
                                                      //Field/RPC position
   Cells[0,i+1]  := Piece(LocalBroker.Results[i],'^',11);  //Site  11
   Cells[1,i+1]  := Date2Str(EnhStr2Date (Piece(LocalBroker.Results[i],'^',1)));   //Date  1
   Cells[2,i+1]  := Piece(LocalBroker.Results[i],'^',14);  //Type  14
   Cells[3,i+1]  := Piece(LocalBroker.Results[i],'^',13);  //Form  13
   Cells[4,i+1]  := Piece(LocalBroker.Results[i],'^',2);   //Pat   2
   Cells[5,i+1]  := Piece(LocalBroker.Results[i],'^',15);  //SSN  15
   Cells[6,i+1]  := Piece(LocalBroker.Results[i],'^',16);  //IEN  16
   Cells[7,i+1]  := Piece(LocalBroker.Results[i],'^',8);   //Desc   8
   Cells[8,i+1]  := Piece(LocalBroker.Results[i],'^',3);   //HCPCS  3
   Cells[9,i+1]  := Piece(LocalBroker.Results[i],'^',17);  //HCPCS Des 17
   Cells[10,i+1] := Piece(LocalBroker.Results[i],'^',10);  //NPPD 10
   Cells[11,i+1] := Piece(LocalBroker.Results[i],'^',9);   //Init  9
   Cells[12,i+1] := Piece(LocalBroker.Results[i],'^',6);   //IA Date 6
   Cells[13,i+1] := Piece(LocalBroker.Results[i],'^',7);   //Cost  7
   Cells[14,i+1] := Piece(LocalBroker.Results[i],'^',4);   //Qty  4
   Cells[15,i+1] := Piece(LocalBroker.Results[i],'^',18);  //Source 18
   Cells[16,i+1] := Piece(LocalBroker.Results[i],'^',5);   //Vendor 5
   Cells[17,i+1] := Piece(LocalBroker.Results[i],'^',12);  //Grp#   12
   Cells[18,i+1] := Piece(LocalBroker.Results[i],'^',19);  //CustomData1
   Cells[19,i+1] := Piece(LocalBroker.Results[i],'^',20);  //CustomData2
   Cells[20,i+1] := Piece(LocalBroker.Results[i],'^',21);  //CustomData3
   end
  end;

    //Sets the grid rows to match result for sort to work correct
    //The Column moving is OFF by default because of a error if empty
    //After data is retrieved, sets to ON.
   if LocalBroker.Results.Count > 1 then
     begin
      grdMain.RowCount := LocalBroker.Results.Count +1;
      grdMain.Options := grdMain.Options+[goColMoving]; //ON
     end;
  end;
 end
  else  //Result if date1 NOT before date2
      MessageDlg('The Starting Date must be before the Ending Date', mtInformation,
      [mbOk], 0);
  end;
 end;
end;


procedure TfrmMain.FormActivate(Sender: TObject);
begin
Application.HelpFile := 'PROS_NPPDDETAILDISPLAY.HLP'; //Needed here
Application.OnHint := DisplayHint;  //Displays hints in Status Bar
frmMain.Caption := 'NPPD Detail Display . . . You are logged onto '+LocalBroker.Server;

with grdMain do               // Sets column headers at start of App
 begin
      Cells[0,0]  := 'Site';
      Cells[1,0]  := 'Date';
      Cells[2,0]  := 'Type';
      Cells[3,0]  := 'Form';
      Cells[4,0]  := 'Patient';
      Cells[5,0]  := 'SSN';
      Cells[6,0]  := 'IEN';
      Cells[7,0]  := 'Brief Description';
      Cells[8,0]  := 'HCPCS';
      Cells[9,0]  := 'HCPCS Desc';
      Cells[10,0] := 'NPPD';
      Cells[11,0] := 'Initi';
      Cells[12,0] := 'Sus IA';
      Cells[13,0] := 'Cost';
      Cells[14,0] := 'Qty';
      Cells[15,0] := 'V/C';
      Cells[16,0] := 'Vendor';
      Cells[17,0] := 'Grp#';
      Cells[18,0] := CustomData1_FieldName;
      Cells[19,0] := CustomData2_FieldName;
      Cells[20,0] := CustomData3_FieldName;

 end;

end;

procedure TfrmMain.btnPrintClick(Sender: TObject);  //Print form
var
poOriginal : TPrinterOrientation;
begin
  if LocalBroker.Connected = false then      //Checks if user is logged on
      MessageDlg('Please Sign On first.', mtInformation,
      [mbOk], 0)
  else
  begin
  if grdMain.Cells[1,1] = '' then
      MessageDlg('Please DISPLAY the data to print.', mtInformation,
      [mbOk], 0)
  else
  begin
  poOriginal := Printer.Orientation;   //Save original Printer Orientation
  Printer.Orientation := poLandscape;  //Set to landscape
   if dlgPrint.Execute = true then
    begin
     grdMain.PrintOptions.PageTitle :=      //Form HEADER
      'NPPD DETAIL DISPLAY    Source: '+LocalBroker.Server+'      '+edtBegDate.Text+'  to  '+
       edtEndDate.Text+'           Printed: '+Date2Str(Now);
     grdMain.PrintOptions.ToRow := grdmain.RowCount;
     grdMain.Print;     //PRINT
  end; //Dialog
  Printer.Orientation := poOriginal;  //reset to original
 end;
 end;
end;

procedure TfrmMain.btnExcelClick(Sender: TObject); //Save and launch as Excel
var
    DIR : String;
    buttonSelected : Integer;
    currdir : String;
 begin
  if LocalBroker.Connected = false then      //Checks if user is logged on
      MessageDlg('Please Sign On first.', mtInformation,
      [mbOk], 0)
  else

   begin

   if fileexists(excelfile) then
    begin
      if SysUtils.DeleteFile(ExcelFile) = False then
        raise Exception.Create('Please close the temporary Excel (.csv) file before continuing')     //rmpr*136
      else
        lblExcelPath.Caption := '';
    end;

  
  begin
  if grdMain.Cells[1,1] = '' then
      MessageDlg('Please DISPLAY the data to send to Excel.', mtInformation,
      [mbOk], 0)
  else
  begin
     begin    //Check for default CSV folder and create it
      if not DirectoryExists('c:\NPPDDownload') then
      if not CreateDir('c:\NPPDDownload') then
      raise Exception.Create('Cannot create c:\NPPDDownload');
     end;

     DIR := 'c:\NPPDDownload';

      buttonSelected := Application.MessageBox('The data you are about to export may'+
      ' contain Patient Identifiable Information, which requires appropriate measures '+
      'to ensure its confidentiality.  Please select only a secure, encrypted location'+
      ' to save this data.','Security Reminder',MB_OK + MB_DEFBUTTON1);


    if buttonSelected = ID_OK  then
     begin
        currdir := GetCurrentDir;    //capture current directory
        if SelectDirectory(DIR, [sdAllowCreate, sdPerformCreate, sdPrompt],0) then
        begin
          ExcelFile := DIR+'\'+edtBegDate.Text+'_'+edtEndDate.Text+'.csv';
          grdMain.SaveToCSV(ExcelFile);
          ShellExecute(frmMain.Handle, nil, PChar(ExcelFile), nil, nil, SW_SHOWNORMAL);
          lblExcelPath.Caption := DIR;

        end;
        SetCurrentDir(currdir);  //reset current directory
     end;
    end;
    end;
 end;
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
Close;
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
     if fileexists(excelfile) then
    begin
      if SysUtils.DeleteFile(ExcelFile) = False then
        raise Exception.Create('Please close the temporary Excel (.csv) file before navigating from this form')    //RMPR*136
      else
        lblExcelPath.Caption := '';
    end;
end;



procedure TfrmMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
//if (ssAlt in Shift) and (chr(Key) = '2') then frmMenu.btnDelayedOrderReport.Click;
//if (ssAlt in Shift) and (chr(Key) in ['D','d']) then btnDisplay.Click;
//if (ssAlt in Shift) and (chr(Key) in ['P','p']) then btnPrint.Click;
//if (ssAlt in Shift) and (chr(Key) in ['X','x']) then btnExcel.Click;
if (ssCtrl in Shift) and (chr(Key) in ['C','c']) then btnClose.Click;
end;

procedure TfrmMain.mnuItemBegDateClick(Sender: TObject);
begin
dtpkrBegDate.SetFocus;
dtpkrBegDate.Perform(WM_KEYDOWN, VK_F4, 0); //Calls Calendar from Menu
dtpkrBegDate.Perform(WM_KEYUP, VK_F4, 0);
end;

procedure TfrmMain.mnuItemDisplayClick(Sender: TObject);
begin
btnDisplay.Click;
end;

procedure TfrmMain.menuItemPrintClick(Sender: TObject);
begin
btnPrint.Click;
end;

procedure TfrmMain.mnuItemExcelClick(Sender: TObject);
begin
btnExcel.Click;
end;

procedure TfrmMain.FormCreate(Sender: TObject);

begin

   CheckSec508;
   if Sec508 = '1' then
   begin
     imgGradient.EndColor   := clWhite;   //Change colors
     imgGradient.StartColor := clWhite;
     bnrMain.Font.Color     := clBlack;
     lblBegDate.Font.Color  := clBlack;
     lblEndDate.Font.Color  := clBlack;
     bnrExclude.Font.Color  := clBlack;
     lblRecFound.Font.Color := clBlack;
     btnDisplay.Font.Color  := clBlack;
     btnExcel.Font.Color    := clBlack;
     grdMain.FixedColor     := clWhite;
//     grdMain.Options := grdMain.Options-[goRowSelect]; //Turn Off
  end;
end;

procedure TfrmMain.mnuItemAboutClick(Sender: TObject);
begin
dlgAbout.ShowModal;
end;

procedure TfrmMain.mnuItemContentsClick(Sender: TObject);
begin
Application.HelpCommand (Help_Index,0);
end;

procedure TfrmMain.mnuItemExitClick(Sender: TObject);
begin
Close;
end;

procedure TfrmMain.mnuItemEndDateClick(Sender: TObject);
begin
dtpkrEndDate.SetFocus;
dtpkrEndDate.Perform(WM_KEYDOWN, VK_F4, 0); //Calls Calendar from menu
dtpkrEndDate.Perform(WM_KEYUP, VK_F4, 0);
end;

procedure TfrmMain.dtpkrBegDateKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if edtBegDate.text = '' then  //Checks if date had been selected, else
 dtpkrBegDate.Date := Now;  //Sets date from Menu call
end;

procedure TfrmMain.dtpkrEndDateKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);     //Sets date from Menu call
begin                      //Needed when calling from Menu ?!?
 if edtEndDate.text = '' then
 dtpkrEndDate.Date := Now;
end;

procedure TfrmMain.dtpkrBegDateEnter(Sender: TObject);
begin

  if edtBegDate.text = '' then   //Checks if date had been selected, else
  dtpkrBegDate.Date := Now;     //force today date
end;

procedure TfrmMain.dtpkrEndDateEnter(Sender: TObject);
begin
  if edtEndDate.text = '' then   //Checks if date had been selected, else
  dtpkrEndDate.Date := dtpkrBegDate.Date;  //start w/begin date
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
Application.HelpCommand(HELP_QUIT, 0); //Close Help if still open

end;

procedure TfrmMain.mnuItemSec508Click(Sender: TObject);
var
  reg : TRegistry;          //Toggle Sec 508 screen settings in registry
begin
  reg := TRegistry.Create;
  reg.rootkey := HKEY_LOCAL_MACHINE;
  reg.OpenKey('\SOFTWARE\VISTA\PROS\NPPDDetailDisplay',False);
  Sec508 := reg.ReadString('Sec508');
   begin  //Toggle the registry key
    if Sec508 = '0' then
       begin
        if MessageDlg('Do you wish to activate Section 508 assistance?',
        mtConfirmation, [mbYes, mbNo], 0) = mrYes then
          begin
            reg.WriteString('Sec508','1');    //Turn it on
            ShowMessage('Please EXIT and RESTART the program for the changes to take effect');
          end;
       end
   else
      begin
       if MessageDlg('Do you wish to turn off Section 508 assistance?',
       mtConfirmation, [mbYes, mbNo], 0) = mrYes then
         begin
           reg.WriteString('Sec508', '0');  //Turn it off
           ShowMessage('Please EXIT and RESTART the program for the changes to take effect');
         end;
      end;
 end;
  reg.CloseKey;
  reg.Free;


end;

procedure TfrmMain.grdMainGetCellFormat(Sender: TObject; Col, Row: Integer;
  State: TGridDrawState; var FormatOptions: TFormatOptions; var CheckBox,
  Combobox, Ellipsis: Boolean);
begin
if (Col=13) and (Row >0) then   //Sets COST column for proper sorting
   formatoptions.AlignmentHorz := taRightJustify;
end;

procedure TfrmMain.grdMainGetSortStyle(Sender: TObject; Col: Integer;
  var SortStyle: TSortStyle);
begin
if Col=13 then SortStyle := ssNumeric ; //Sort COST as a number
if Col=1  then SortStyle := ssDateTime; //Sort DATE as a date
end;

procedure TfrmMain.btnClearClick(Sender: TObject);
var
  i : integer;
begin
  lblDisRecFound.Caption := '0';
  for i :=1 to grdMain.RowCount -1 do   //If selected will clear the grid
   grdMain.Rows[i].Text := '';
   grdMain.Row := 1;
   grdMain.Cells[0,1] :='';
   grdMain.RowCount := 25;
end;

procedure TfrmMain.cbxCustomData1DropDown(Sender: TObject);
var
 i : integer;
begin

    cbxCustomData1.Clear;
    LocalBroker.CreateContext('RMPR NPPD GUI');
    LocalBroker.RemoteProcedure := 'RMPR NPPD LIST X';

    Try
    //
     LocalBroker.Call;

    except
     on E:EBrokerError do
       begin
       Showmessage('Broker Error: '+ E.Message);
       end;
     on E:EAccessViolation do
       Showmessage('E.Message');
     else
       raise;
     end;


 for i:= 0 to LocalBroker.Results.Count -1  do
  begin
  cbxCustomData1.Items.Add(Piece(LocalBroker.Results[i],'^',1) +' - '+
                          Piece(LocalBroker.Results[i],'^',2));
  end;

end;

procedure TfrmMain.cbxCustomData1Exit(Sender: TObject);
begin
CustomData1_FieldName := Piece(cbxCustomData1.Text,'- ',1); //extra space needed.
CustomData1_FieldNumber := Piece(cbxCustomData1.Text,'- ',2); //extra space needed.
end;

procedure TfrmMain.cbxCustomData1Enter(Sender: TObject);
begin
cbxCustomData1.Perform(CB_SETDROPPEDWIDTH, 300, 0);
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
//     MyAutoResize := TAutoResize.Create(Self);
//     MyAutoResize.GetFormInfo(frmMain);
end;

procedure TfrmMain.FormResize(Sender: TObject);
begin
//MyAutoResize.ResizeTheF(frmMain);
end;

procedure TfrmMain.cbxCustomData2DropDown(Sender: TObject);
var
 i : integer;
begin

    cbxCustomData2.Clear;
    LocalBroker.CreateContext('RMPR NPPD GUI');
    LocalBroker.RemoteProcedure := 'RMPR NPPD LIST X';

    Try
    //
     LocalBroker.Call;

    except
     on E:EBrokerError do
       begin
       Showmessage('Broker Error: '+ E.Message);
       end;
     on E:EAccessViolation do
       Showmessage('E.Message');
     else
       raise;
     end;


 for i:= 0 to LocalBroker.Results.Count -1  do
  begin
  cbxCustomData2.Items.Add(Piece(LocalBroker.Results[i],'^',1) +' - '+
                          Piece(LocalBroker.Results[i],'^',2));
  end;

end;


procedure TfrmMain.cbxCustomData3DropDown(Sender: TObject);
var
 i : integer;
begin

    cbxCustomData3.Clear;
    LocalBroker.CreateContext('RMPR NPPD GUI');
    LocalBroker.RemoteProcedure := 'RMPR NPPD LIST X';

    Try
    //
     LocalBroker.Call;

    except
     on E:EBrokerError do
       begin
       Showmessage('Broker Error: '+ E.Message);
       end;
     on E:EAccessViolation do
       Showmessage('E.Message');
     else
       raise;
     end;


 for i:= 0 to LocalBroker.Results.Count -1  do
  begin
  cbxCustomData3.Items.Add(Piece(LocalBroker.Results[i],'^',1) +' - '+
                          Piece(LocalBroker.Results[i],'^',2));
  end;

end;


procedure TfrmMain.cbxCustomData2Enter(Sender: TObject);
begin
cbxCustomData2.Perform(CB_SETDROPPEDWIDTH, 300, 0);
end;

procedure TfrmMain.cbxCustomData3Enter(Sender: TObject);
begin
cbxCustomData3.Perform(CB_SETDROPPEDWIDTH, 300, 0);
end;

procedure TfrmMain.cbxCustomData2Exit(Sender: TObject);
begin
CustomData2_FieldName := Piece(cbxCustomData2.Text,'- ',1); //extra space needed.
CustomData2_FieldNumber := Piece(cbxCustomData2.Text,'- ',2); //extra space needed.

end;

procedure TfrmMain.cbxCustomData3Exit(Sender: TObject);
begin
CustomData3_FieldName := Piece(cbxCustomData3.Text,'- ',1); //extra space needed.
CustomData3_FieldNumber := Piece(cbxCustomData3.Text,'- ',2); //extra space needed.

end;

procedure TfrmMain.appNPPDDetailDisplayMessage(var Msg: tagMSG;
  var Handled: Boolean);
var
   i: SmallInt;  
begin
  if grdMain.Focused then
  begin
   if Msg.message = WM_MOUSEWHEEL then
   begin
     Msg.message := WM_KEYDOWN;
     Msg.lParam := 0;
     i := HiWord(Msg.wParam) ;
     if i > 0 then
      begin

    SendMessage(grdMain.Handle,  { HWND of the Memo Control }
                WM_VSCROLL,    { Windows Message }
                SB_LINEUP,     { Scroll Command }
                0);            { Not Used }

      end
     else
    SendMessage(grdMain.Handle,  { HWND of the Memo Control }
                WM_VSCROLL,    { Windows Message }
                SB_LINEDOWN,     { Scroll Command }
                0);            { Not Used }


     Handled := False;
   end;
  end;

end;

end.
