{*****************************************************
  NPPD Detail Display - GUI    ver 1.0.3   Nov 7,2002
  Office of Information Field Office
  Department of Veterans Affairs
  Hines,Illinois

  Developers: Helen Corkwell, Ken Nelson
  Description: GUI to view and print NPPD purchase data
  Requires: RMPR*3.0*71    -   RMPR NPPD LIST

*******************************************************}

unit PROS_NPPDDetailDisplay_Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls,MFunStr, MyGrids, SortGrid, SRGrad, ESBDates, Buttons, Trpcb,
  ExtCtrls, ShellAPI, Rpcconf1, Menus, NumEdit, Registry, FileCtrl, ActnList,
  TrayIcon;

type
  TfrmMain = class(TForm)
    imgGradient: TSRGradient;
    grdMain: TSortGrid;
    edtBegDate: TEdit;
    edtEndDate: TEdit;
    dtpkrBegDate: TDateTimePicker;
    dtpkrEndDate: TDateTimePicker;
    brkrRPC: TRPCBroker;
    btnDisplay: TBitBtn;
    bnrMain: TLabel;
    lblBegDate: TLabel;
    lblEndDate: TLabel;
    btnPrint: TBitBtn;
    dlgPrint: TPrintDialog;
    btnExcel: TBitBtn;
    btnClose: TBitBtn;
    lblRecFound: TLabel;
    bnrExclude: TLabel;
    lblDisRecFound: TLabel;
    mnuMain: TMainMenu;
    mnuItemHelp: TMenuItem;
    mnuItemVistaSignOn: TMenuItem;
    mnuItemFile: TMenuItem;
    mnuItemBegDate: TMenuItem;
    mnuItemEndDate: TMenuItem;
    mnuItemDisplay: TMenuItem;
    menuItemPrint: TMenuItem;
    mnuItemExcel: TMenuItem;
    mnuItemExit: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    stsbrMain: TStatusBar;
    mnuItemContents: TMenuItem;
    mnuItemAbout: TMenuItem;
    N3: TMenuItem;
    N4: TMenuItem;
    mnuItemSec508: TMenuItem;
    procedure edtBegDateChange(Sender: TObject);
    procedure dtpkrBegDateChange(Sender: TObject);
    procedure edtEndDateChange(Sender: TObject);
    procedure dtpkrEndDateChange(Sender: TObject);
    procedure btnDisplayClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure dtpkrBegDateEnter(Sender: TObject);
    procedure btnExcelClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure mnuItemVistaSignOnClick(Sender: TObject);
    procedure SignOnVista;
    procedure CheckSec508;
    procedure mnuItemBegDateClick(Sender: TObject);
    procedure mnuItemDisplayClick(Sender: TObject);
    procedure menuItemPrintClick(Sender: TObject);
    procedure mnuItemExcelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DisplayHint(Sender: TObject);
    procedure mnuItemAboutClick(Sender: TObject);
    procedure mnuItemContentsClick(Sender: TObject);
    procedure mnuItemExitClick(Sender: TObject);
    procedure mnuItemEndDateClick(Sender: TObject);
    procedure dtpkrBegDateKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure dtpkrEndDateKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure dtpkrEndDateEnter(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mnuItemSec508Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;
  Sec508,                  //Sec508 status from registry
  MDate1, MDate2 : string; // First and second 'Mumps' dates for search

implementation

uses PROS_NPPDDetailDisplay_About;

{$R *.DFM}

procedure TfrmMain.CheckSec508; //Checks the SEC508 registry Key
var
  reg : TRegistry;
begin
   reg := TRegistry.Create;
   reg.rootkey := HKEY_LOCAL_MACHINE;
   reg.OpenKey('\SOFTWARE\VISTA\PROS\NPPDDetailDisplay',False);
   Sec508 := reg.ReadString('Sec508');  //Assign result to global
   reg.Free;
end;

procedure TfrmMain.SignOnVista;  //Sign onto the Vista System
var
strServer, strPort: string;
begin
  brkrRPC.Connected := false;  //Forces disconnection
  brkrRPC.Server := '';    //Clears specified server name/addr
  brkrRPC.ListenerPort := 0 ;

  if ( brkrRPC.Connected = False) and (GetServerInfo(strServer, strPort)<> mrCancel) then
   begin //getsvrinfo
     brkrRPC.Server := strServer;          //use specified server name/addr
     brkrRPC.ListenerPort := StrToInt(strPort);  //use specified port
     brkrRPC.Connected := true;
   end;
 Application.HelpFile := 'PROS_NPPDDETAILDISPLAY.HLP'; //Needed here because of
end;                                                 //broker grabbing variable?

procedure TfrmMain.DisplayHint(Sender: TObject);
begin
  stsbrMain.SimpleText := Application.Hint; //For Sec508
end;

//First Date
procedure TfrmMain.edtBegDateChange(Sender: TObject);
var
  s1,d1,d2,d3 : string;        //Creates the Jun 01,2002 display date
begin                          //from the tossed calendar pick
s1 := '';                    //Clears everything
  if edtBegDate.Text <> '' then
   begin
        d1 := Date2ShortMonth(dtpkrBegDate.Date);
       if Date2Day(dtpkrBegDate.Date) < 10 then  //Pad with 0 if needed
        d2:= '0'+ IntToStr(Date2Day(dtpkrBegDate.Date))
       else
        d2 := IntToStr(Date2Day(dtpkrBegDate.Date));  //Or not
        d3 := IntToStr(Date2Year(dtpkrBegDate.Date));

   s1 := d1+' '+d2+', '+d3; //Assembles the pieces
   edtBegDate.text := s1         //Display

   end;
end;

procedure TfrmMain.dtpkrBegDateChange(Sender: TObject);
var
d1,d2,d3,d4 : string;         //Gets the user input from the calendar
begin
      MDate1 := '';
      if (Date2Year(dtpkrBegDate.Date) < 1900) then  //Checks century
    d1 := '1';
      if (Date2Year(dtpkrBegDate.Date) >= 1900) and
         (Date2Year(dtpkrBegDate.Date) < 2000) then
    d1 := '2';
      if (Date2Year(dtpkrBegDate.Date) >= 2000) then
    d1 := '3';

    d2 := Copy(IntToStr(Date2Year(dtpkrBegDate.Date)),3,2); //Gets year

      if Date2Month(dtpkrBegDate.Date) < 10 then //Pad month with 0 if needed
    d3:= '0'+ IntToStr(Date2Month(dtpkrBegDate.Date))
       else
    d3 := IntToStr(Date2Month(dtpkrBegDate.Date));

     if Date2Day(dtpkrBegDate.Date) < 10 then  //Pad day with 0 if needed
    d4:= '0'+ IntToStr(Date2Day(dtpkrBegDate.Date))
       else
    d4 := IntToStr(Date2Day(dtpkrBegDate.Date));

MDate1 := d1 + d2+ d3 + d4;                    //Assemble MUMPS date style
edtBegDate.Text := Date2Str(dtpkrBegDate.Date);  //Toss the date to Edit

end;


//*************************

//Second Date
procedure TfrmMain.edtEndDateChange(Sender: TObject);
var
  s1,d1,d2,d3: string;        //Creates the Jun 01,2002 display date
begin                         //from the tossed calendar pick
s1 := '';                     //Clears everything
  if edtEndDate.Text <> '' then
   begin
        d1 := Date2ShortMonth(dtpkrEndDate.Date);
       if Date2Day(dtpkrEndDate.Date) < 10 then  //Pad with 0 if needed
        d2:= '0'+ IntToStr(Date2Day(dtpkrEndDate.Date))
       else
        d2 := IntToStr(Date2Day(dtpkrEndDate.Date));  //Or not
        d3 := IntToStr(Date2Year(dtpkrEndDate.Date));

   s1 := d1+' '+d2+', '+d3;    //Assembles the pieces
   edtEndDate.text := s1          //Display

   end;
end;


procedure TfrmMain.dtpkrEndDateChange(Sender: TObject);
var
d1,d2,d3,d4 : string;
                            //Gets the user input from the calendar
begin
      MDate2 := '';
      if (Date2Year(dtpkrEndDate.Date) < 1900) then  //Checks century
    d1 := '1';
      if (Date2Year(dtpkrEndDate.Date) >= 1900) and
         (Date2Year(dtpkrEndDate.Date) < 2000) then
    d1 := '2';
      if (Date2Year(dtpkrEndDate.Date) >= 2000) then
    d1 := '3';

    d2 := Copy(IntToStr(Date2Year(dtpkrEndDate.Date)),3,2);//Gets year

        if Date2Month(dtpkrEndDate.Date) < 10 then //Pad month with 0 if needed
    d3:= '0'+ IntToStr(Date2Month(dtpkrEndDate.Date))
       else
    d3 := IntToStr(Date2Month(dtpkrEndDate.Date));

        if Date2Day(dtpkrEndDate.Date) < 10 then  //Pad day with 0 if needed
    d4:= '0'+ IntToStr(Date2Day(dtpkrEndDate.Date))
       else
    d4 := IntToStr(Date2Day(dtpkrEndDate.Date));

  MDate2 := d1 + d2+ d3 + d4;          //Assemble MUMPS date style
  edtEndDate.Text := Date2Str(dtpkrBegDate.Date); //Toss the date to Edit
 end;


procedure TfrmMain.btnDisplayClick(Sender: TObject);  //Fill the grid
var
i : integer;

begin
  if brkrRPC.Connected = false then      //Checks if user is logged on
      MessageDlg('Please Sign On first.', mtInformation,
      [mbOk], 0)
  else
    begin
     if (MDate1 = '') or (MDate2 = '') then
       MessageDlg('Please select a date range.', mtInformation,
      [mbOk], 0)
     else
   begin
    if StrToInt(MDate1) <= StrToInt(Mdate2) then //Test if Date1 before Date2
   begin
     begin
      for i :=1 to grdMain.RowCount -1 do   //If selected will clear the grid
      grdMain.Rows[i].Clear;
      grdMain.Row := 1;
      grdMain.Cells[0,1] :='';

    end;

  if DaysApart (dtpkrBegDate.Date,dtpkrEndDate.Date) > 100 //Date Range checking
  then  MessageDlg('You have selected a range'+#13+'of more than 100 days.', mtInformation,
      [mbOk], 0)
   else

begin

    brkrRPC.CreateContext('RMPR NPPD GUI');
    brkrRPC.RemoteProcedure := 'RMPR NPPD LIST';
    frmMain.brkrRPC.Param[0].Value := MDate1;
    frmMain.brkrRPC.Param[0].PType := literal;
    frmMain.brkrRPC.Param[1].Value := MDate2;
    frmMain.brkrRPC.Param[1].PType := literal;
    frmMain.brkrRPC.RPCTimeLimit := 600;    //10 Min ?!?

    Try
     frmMain.brkrRPC.Call;
     except
     on E:EBrokerError do
       begin
       Showmessage('Broker Error: '+ E.Message);
       end;
     on E:EAccessViolation do
       Showmessage('E.Message');
     else
       raise;
     end;

    lblDisRecFound.Caption := IntToStr(brkrRPC.Results.Count);


 for i:= 0 to brkrRPC.Results.Count -1  do
   begin

   with grdMain do           // Sets column headers on new DISPLAY
 begin
    Cells[0,0] := 'Site';
    Cells[1,0] := 'Date';
    Cells[2,0] := 'Type';
    Cells[3,0] := 'Form';
    Cells[4,0] := 'Patient';
    Cells[5,0] := 'SSN';
    Cells[6,0] := 'IEN';
    Cells[7,0] := 'Brief Description';
    Cells[8,0] := 'HCPCS';
    Cells[9,0] := 'HCPCS Desc';
    Cells[10,0] := 'NPPD';
    Cells[11,0] := 'Initi';
    Cells[12,0] := 'Sus IA';
    Cells[13,0] := 'Cost';
    Cells[14,0] := 'Qty';
    Cells[15,0] := 'V/C';
    Cells[16,0] := 'Vendor';
    Cells[17,0] := 'Grp#';
                                                      //Field/RPC position
   Cells[0,i+1]  := Piece(brkrRPC.Results[i],'^',11);  //Site  11
   Cells[1,i+1]  := Piece(brkrRPC.Results[i],'^',1);   //Date  1
   Cells[2,i+1]  := Piece(brkrRPC.Results[i],'^',14);  //Type  14
   Cells[3,i+1]  := Piece(brkrRPC.Results[i],'^',13);  //Form  13
   Cells[4,i+1]  := Piece(brkrRPC.Results[i],'^',2);   //Pat   2
   Cells[5,i+1]  := Piece(brkrRPC.Results[i],'^',15);  //SSN  15
   Cells[6,i+1]  := Piece(brkrRPC.Results[i],'^',16);  //IEN  16
   Cells[7,i+1]  := Piece(brkrRPC.Results[i],'^',8);   //Desc   8
   Cells[8,i+1]  := Piece(brkrRPC.Results[i],'^',3);   //HCPCS  3
   Cells[9,i+1]  := Piece(brkrRPC.Results[i],'^',17);  //HCPCS Des 17
   Cells[10,i+1] := Piece(brkrRPC.Results[i],'^',10);  //NPPD 10
   Cells[11,i+1] := Piece(brkrRPC.Results[i],'^',9);   //Init  9
   Cells[12,i+1] := Piece(brkrRPC.Results[i],'^',6);   //IA Date 6
   Cells[13,i+1] := Piece(brkrRPC.Results[i],'^',7);   //Cost  7
   Cells[14,i+1] := Piece(brkrRPC.Results[i],'^',4);   //Qty  4
   Cells[15,i+1] := Piece(brkrRPC.Results[i],'^',18);  //Source 18
   Cells[16,i+1] := Piece(brkrRPC.Results[i],'^',5);   //Vendor 5
   Cells[17,i+1] := Piece(brkrRPC.Results[i],'^',12);  //Grp#   12
   end
  end;

    //Sets the grid rows to match result for sort to work correct
    //The Column moving is OFF by default because of a error if empty
    //After data is retrieved, sets to ON.
   if brkrRPC.Results.Count > 1 then
     begin
      grdMain.RowCount := brkrRPC.Results.Count +1;
      grdMain.Options := grdMain.Options+[goColMoving]; //ON
     end;
  end;
 end
  else  //Result if date1 NOT before date2
      MessageDlg('The Starting Date must be before the Ending Date', mtInformation,
      [mbOk], 0);
  end;
 end;
end;


procedure TfrmMain.FormActivate(Sender: TObject);
begin

SignOnVista;

with grdMain do               // Sets column headers at start of App
 begin
      Cells[0,0]  := 'Site';
      Cells[1,0]  := 'Date';
      Cells[2,0]  := 'Type';
      Cells[3,0]  := 'Form';
      Cells[4,0]  := 'Patient';
      Cells[5,0]  := 'SSN';
      Cells[6,0]  := 'IEN';
      Cells[7,0]  := 'Brief Description';
      Cells[8,0]  := 'HCPCS';
      Cells[9,0]  := 'HCPCS Desc';
      Cells[10,0] := 'NPPD';
      Cells[11,0] := 'Initi';
      Cells[12,0] := 'Sus IA';
      Cells[13,0] := 'Cost';
      Cells[14,0] := 'Qty';
      Cells[15,0] := 'V/C';
      Cells[16,0] := 'Vendor';
      Cells[17,0] := 'Grp#';

 end;

end;

procedure TfrmMain.btnPrintClick(Sender: TObject);  //Print form
begin
  if brkrRPC.Connected = false then      //Checks if user is logged on
      MessageDlg('Please Sign On first.', mtInformation,
      [mbOk], 0)
  else
  begin
  if grdMain.Cells[1,1] = '' then
      MessageDlg('Please DISPLAY the data to print.', mtInformation,
      [mbOk], 0)
  else
  begin
   if dlgPrint.Execute = true then
    begin
     grdMain.PrintOptions.PageTitle :=      //Form HEADER
      'NPPD Detail Display      '+edtBegDate.Text+'  to  '+
       edtEndDate.Text+'                Printed: '+Date2Str(Now);
     grdMain.PrintOptions.ToRow :=(brkrRPC.Results.Count +1);
     grdMain.Print;     //PRINT
  end //Dialog
 end;
 end;
end;

procedure TfrmMain.btnExcelClick(Sender: TObject); //Save and launch as Excel
var
  ExcelFile : string;
 begin
  if brkrRPC.Connected = false then      //Checks if user is logged on
      MessageDlg('Please Sign On first.', mtInformation,
      [mbOk], 0)
  else

  begin
  if grdMain.Cells[1,1] = '' then
      MessageDlg('Please DISPLAY the data to print.', mtInformation,
      [mbOk], 0)
  else
  begin
     begin    //Check for default CSV folder and create it
      if not DirectoryExists('c:\NPPDDownload') then
      if not CreateDir('c:\NPPDDownload') then
      raise Exception.Create('Cannot create c:\NPPDDownload');
     end;

  ExcelFile := 'c:\NPPDDownload\'+edtBegDate.Text+'_'+edtEndDate.Text+'.csv';
  grdMain.SaveToCSV(ExcelFile);
  ShellExecute(frmMain.Handle, nil, PChar(ExcelFile), nil, nil, SW_SHOWNORMAL);
  end;
 end; 
end;


//The next two force the second calendar to use the first calendar date
//and help speed of work flow

procedure TfrmMain.dtpkrBegDateEnter(Sender: TObject);
begin
if DateToStr(dtpkrBegDate.Date) = '1/1/1980' then  //epic date to
  dtpkrBegDate.Date := Now;                        //force today date
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
Close;
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
brkrRPC.Connected := False; //Disconnect from the server
end;



procedure TfrmMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
if (ssAlt in Shift) and (chr(Key) in ['D','d']) then btnDisplay.Click;
if (ssAlt in Shift) and (chr(Key) in ['P','p']) then btnPrint.Click;
if (ssAlt in Shift) and (chr(Key) in ['X','x']) then btnExcel.Click;
IF (ssAlt in Shift) and (chr(Key) in ['C','c']) then btnClose.Click;
end;

procedure TfrmMain.mnuItemVistaSignOnClick(Sender: TObject);
begin
if  brkrRPC.Connected = true then      //Checks if user really wants to
begin                                  //log off then log on again
if MessageDlg('You are about to log off of Vista - Continue?',
    mtConfirmation, [mbYes, mbNo], 0) = mrYes then SignOnVista;
end
else SignOnVista;
end;

procedure TfrmMain.mnuItemBegDateClick(Sender: TObject);
begin
dtpkrBegDate.Perform(WM_KEYDOWN, VK_F4, 0); //Calls Calendar from Menu
dtpkrBegDate.Perform(WM_KEYUP, VK_F4, 0);
end;

procedure TfrmMain.mnuItemDisplayClick(Sender: TObject);
begin
btnDisplay.Click;
end;

procedure TfrmMain.menuItemPrintClick(Sender: TObject);
begin
btnPrint.Click;
end;

procedure TfrmMain.mnuItemExcelClick(Sender: TObject);
begin
btnExcel.Click;
end;

procedure TfrmMain.FormCreate(Sender: TObject);

begin
   CheckSec508;
   if Sec508 = '1' then
   begin
     imgGradient.EndColor   := clWhite;   //Change colors
     imgGradient.StartColor := clWhite;
     bnrMain.Font.Color     := clBlack;
     lblBegDate.Font.Color  := clBlack;
     lblEndDate.Font.Color  := clBlack;
     bnrExclude.Font.Color  := clBlack;
     lblRecFound.Font.Color := clBlack;
     btnDisplay.Font.Color  := clBlack;
     btnExcel.Font.Color    := clBlack;
     grdMain.FixedColor     := clWhite;
//     grdMain.Options := grdMain.Options-[goRowSelect]; //Turn Off
  end;

Application.OnHint := DisplayHint;  //Displays hints in Status Bar
end;

procedure TfrmMain.mnuItemAboutClick(Sender: TObject);
begin
dlgAbout.ShowModal;
end;

procedure TfrmMain.mnuItemContentsClick(Sender: TObject);
begin
Application.HelpCommand (Help_Index,0);
end;

procedure TfrmMain.mnuItemExitClick(Sender: TObject);
begin
Close;
end;

procedure TfrmMain.mnuItemEndDateClick(Sender: TObject);
begin
dtpkrEndDate.Perform(WM_KEYDOWN, VK_F4, 0); //Calls Calendar from menu
dtpkrEndDate.Perform(WM_KEYUP, VK_F4, 0);
end;

procedure TfrmMain.dtpkrBegDateKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 dtpkrBegDate.Date := Now;  //Sets date from Menu call
end;

procedure TfrmMain.dtpkrEndDateKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);     //Sets date from Menu call
begin                      //Needed when calling from Menu ?!?
if DateToStr(dtpkrEndDate.Date) = '1/1/1980' then  //epic date to
 dtpkrEndDate.Date := dtpkrBegDate.Date;          //force Date1
end;

procedure TfrmMain.dtpkrEndDateEnter(Sender: TObject);
begin
if DateToStr(dtpkrEndDate.Date) = '1/1/1980' then  //epic date to
 dtpkrEndDate.Date := dtpkrBegDate.Date;          //force Date1
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
Application.HelpCommand(HELP_QUIT, 0); //Close Help if still open

end;

procedure TfrmMain.mnuItemSec508Click(Sender: TObject);
var
  reg : TRegistry;          //Toggle Sec 508 screen settings in registry
begin
  reg := TRegistry.Create;
  reg.rootkey := HKEY_LOCAL_MACHINE;
  reg.OpenKey('\SOFTWARE\VISTA\PROS\NPPDDetailDisplay',False);
  Sec508 := reg.ReadString('Sec508');
   begin  //Toggle the registry key
    if Sec508 = '0' then
       begin
        if MessageDlg('Do you wish to activate Section 508 assistance?',
        mtConfirmation, [mbYes, mbNo], 0) = mrYes then
          begin
            reg.WriteString('Sec508','1');    //Turn it on
            ShowMessage('Please EXIT and RESTART the program for the changes to take effect');
          end;
       end
   else
      begin
       if MessageDlg('Do you wish to turn off Section 508 assistance?',
       mtConfirmation, [mbYes, mbNo], 0) = mrYes then
         begin
           reg.WriteString('Sec508', '0');  //Turn it off
           ShowMessage('Please EXIT and RESTART the program for the changes to take effect');
         end;
      end;
 end;
  reg.CloseKey;
  reg.Free;


end;

end.
